; NAME: 
;      APPLY_ATRAN
; 
; PURPOSE: 
;      Selects shift for spectra to match to an ATRAN file, chooses 
;      water amount, and divides spectrum by ATRAN transmittance. 
; 
; CALLING SEQUENCE: 
;      result = apply_atran(spectrum, atran, order, [water, 
;               outp=outp, help=help])
; 
; INPUTS: 
;      spectrum - Input spectrum (wavelength, reflectance, error)
;      atran - 61x9000 ATRAN model array
;      order - 1 (3.5-4.2 microns),
;              2 (3.0-3.6 microns), 
;              3 (2.6-3.1 microns), and 
;              4 (2.3-2.7 microns)
; 
; OPTIONAL INPUTS: 
;      water - Water amount or range to check for best fit
;              (default is 0.5 to 3.0 mm)
; 
; OPTIONAL KEYWORD INPUTS: 
;      help - Set to nonzero scalar to print syntax
; 
; OUTPUT: 
;      3xn spectrum array of spectrum/atran is returned
; 
; OPTIONAL OUTPUT KEYWORDS: 
;      outp - Returns shift and water amount selected
; 
; EXTERNAL CALLS: 
;      SUBSPEC, TSHIFT
; 
; COMMON BLOCKS: 
;      NONE
; 
; MODIFICATION HISTORY: 
;      - Written by Eric Volquardsen, NASA IRTF, April 2004
; 
; =======================================================================
FUNCTION apply_atran, spectrum, atran, order, water, outp = outp, $ 
                      help = help

; CHECK INPUTS
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN help = 1 
     ENDIF ELSE help = 0 
     IF n_elements(spectrum) LT 30 THEN help = 1
     IF n_elements(order) NE 1 THEN help = 1
     IF help EQ 1 THEN BEGIN
          print, '****************************************************'
          print, '***              APPLY_ATRAN SYNTAX              ***'  
          print, '*** IDL> result = apply_atran(spectrum, atran,   ***' 
          print, '***                order, water, outp = outp)    ***'
          print, '***                                              ***' 
          print, '*** spectrum - Input spectrum                    ***'
          print, '*** atran - 61x4758 ATRAN model array            ***'
          print, '*** order - 1 (3.5-4.2 microns),                 ***'
          print, '***         2 (3.0-3.6 microns),                 ***'
          print, '***         3 (2.6-3.1 microns), or              ***'
          print, '***         4 (2.3-2.7 microns)                  ***'
          print, '*** water - Water amount or range to check for   ***' 
          print, '***         best fit (default = [0.4, 5.0]       ***'
          print, '*** outp - Returns shift and water amount        ***'
          print, '****************************************************'   
          RETURN, -1
     ENDIF
     nw = n_elements(water)
     IF nw EQ 0 THEN water = [0.4, 5.0]
     IF nw EQ 1 THEN water = [water, water]
     IF water[1] LT 0 THEN water = [0.4, 5.0] 
     water = abs(water)
     IF water[0] GT water[1] THEN water = [water[1], water[0]]
     CASE order OF 

          1 : BEGIN 
               ni1 = 140
               stwl1 = 3.78
               f_o = 3
          END  

          3 : BEGIN 
               ni1 = 140
               stwl1 = 2.83
               f_o = 3
          END  

          4 : BEGIN 
               ni1 = 170
               stwl1 = 2.305
               f_o = 3
          END  

          ELSE : BEGIN 
               ni1 = 200
               stwl1 = 3.2 
               f_o = 3
          END  
     ENDCASE

; SCALE ATRAN TO SPECTRUM
     n = n_elements(spectrum[0, *])
     atran1 = fltarr(61, n)
     atran1[0, *] = spectrum[0, *]
     FOR i =  1, 60 DO BEGIN 
          atran1[i, *] = (interpol(atran[i, *], atran[0, *], $ 
                                  atran1[0, *], /spline)) > 0.05
     ENDFOR 

; DEFINE FIT SUBREGION
     suba = SUBSPEC(spectrum, stwl1, ni1)
     wla = suba[0, *]
     suba[0, *] = wla
     nsa = n_elements(wla)-3
     suba = suba[*, 2:nsa]

; GET SHIFT AND WATER AMOUNT
     afin = TSHIFT(suba[1, *], suba[0, *], suba[2, *], atran1, $ 
                   water = water, f_o = f_o)
     print, 'Shift (microns):       ', afin[0]
     print, 'Water Amount (mm):     ', afin[1]
     print, '  '
     outp = [afin[0], afin[1]]

; DIVIDE SPECTRUM BY ATRAN
     result = spectrum
     wlb = spectrum[0, *] + outp[0]
     result[1, *] = interpol(spectrum[1, *], spectrum[0, *], wlb, /spline)
     w1 = fix(afin[1]*10.0) + 1
     subaa1 = interpol(atran1[w1, *], atran1[0, *], $ 
                      result[0, *], /spline)
     subaa2 = interpol(atran1[w1+1, *], atran1[0, *], $ 
                      result[0, *], /spline)
     dif = subaa1 - subaa2
     frac = (afin[1] - fix(afin[1]*10.0)/10.0)*10.0
     subaa = subaa1 - frac*dif
     result[1, *] = result[1, *]/subaa

     RETURN, result
END
