; NAME: 
;      APPLY_MASK
; 
; VERSION: 1.1
; 
; PURPOSE: 
;      Applies a bad-pixel mask to an image file.  Bad pixel locations
;      are replaced by the average of the surrounding 8 pixels.  If
;      all of the surrounding pixels are bad, then the pixel is
;      replaced by the image median.
; 
; CALLING SEQUENCE: 
;      apply_mask, mask, infile, outfile, paths, [scale=scale, help=help]
; 
; INPUTS: 
;      mask - Mask filename 
;      infile - Image filename
;      outfile - Filename where result should be saved
;      paths - Three element string vector of mask, infile, and 
;              outfile paths
; 
; OPTIONAL KEYWORD INPUTS: 
;      scale - Desired median value of image
;      help - Set to nonzero scalar to print syntax
; 
; OUTPUT: 
;      Resulting image is saved to outfile
; 
; EXTERNAL CALLS: 
;      FXADDPAR, FXWRITE, GETIM
; 
; COMMON BLOCKS: 
;      NONE
; 
; MODIFICATION HISTORY: 
;      - Written by Eric Volquardsen, NASA IRTF, March 2004
;      - Changed image to floating point array, ELV, May 2004
; 
; =======================================================================
PRO apply_mask, mask, infile, outfile, paths, scale = scale, help = help

; CHECK INPUTS
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN help = 1 
     ENDIF ELSE help = 0 
     IF n_params() LT 4 THEN help = 1
     IF help EQ 1 THEN BEGIN
          print, '*****************************************************'
          print, '***              APPLY_MASK SYNTAX                ***'  
          print, '*** IDL> apply_mask, mask, infile, outfile, paths,***' 
          print, '***              scale=scale                      ***' 
          print, '***                                               ***'
          print, '*** mask - Filename of bad pixel mask             ***'
          print, '*** infile - Image filename                       ***'
          print, '*** outfile - Filename of resulting image         ***'
          print, '*** paths - Three element string vector of mask,  ***' 
          print, '***         infile, and outfile                   ***'
          print, '*** scale - Desired median value of image         ***'
          print, '*****************************************************'   
          RETURN
     ENDIF
     np = n_elements(paths)
     CASE np OF 
          0 : BEGIN
               v = !version.os_family
               CASE v OF
                    'vms' : root = '[]'
                    'windows' : root = 'c:\'
                    ELSE : root = '/'
                ENDCASE
                cd, root, current=pat
                cd, pat
                paths = [pat, pat, pat]
          END
          1 : paths = [paths, paths, paths]
          2 : paths = [paths[0], paths[0], paths[1]]
          ELSE : paths = paths[0:2]
     ENDCASE
     IF n_elements(scale) NE 1 THEN scale = 1

; LOAD FILES
     GETIM, mask, msk, paths[0]
     GETIM, infile, img, paths[1], hdr = h
     img = float(img)
     szm = size(msk, /dimension)
     IF n_elements(msk) NE n_elements(img) THEN BEGIN 
          print, '  '
          print, 'MASK AND IMAGE ARE DIFFERENT SIZES'
          print, '  '
          RETURN
     ENDIF 

; APPLY MASK 
; Get coordinates of bad pixels
     w = where(msk EQ 1)
     IF w[0] NE -1 THEN BEGIN 
          xm = w MOD szm[0]
          ym = w / szm[0]

; Replace bad pixels
          img[w] = !values.f_nan
          mimg = median(img, /even)
          FOR k = 0, (n_elements(w)-1) DO BEGIN 
               xl1 = (xm[k]-2) > 0
               xl2 = (xm[k]+2) < (szm[0]-1)
               yl1 = (ym[k]-2) > 0
               yl2 = (ym[k]+2) < (szm[1]-1)
               vals = img[xl1:xl2, yl1:yl2]
               fv =  where(finite(vals) EQ 1)
               IF n_elements(fv) LT 3 THEN img[w[k]] = mimg $ 
                    ELSE img[w[k]] = median(vals, /even)
          ENDFOR 
     ENDIF 

; SCALE IMAGE (if desired)
     IF scale EQ 1 THEN medm = 1 ELSE medm = median(img)
     img = img*scale / medm 

; ALTER HEADER
     img = float(img)
     FXADDPAR, h, 'BITPIX', -32
     FXADDPAR, h, 'COMMENT', 'BAD PIXEL MAP APPLIED'

; SAVE RESULTS
     cd, paths[2], current=nowp
     FXWRITE, outfile, h, img
     cd, nowp

END
