; NAME: 
;      BATCH_SV
; 
; VERSION : 1
; 
; PURPOSE:
;      Widget-based program that generates a batch file with a string 
;      vector of filenames.  Then names are generated from a pattern.
;      The parts entered by the user.  There are potentially three parts 
;      to filenames:
;           (1) a constant part, 
;           (2) a varying part, and 
;           (3) the extension after the varying part that could vary or  
;               remain constant.  
;      The varying part could be a sequential list (a,b,c,...etc) or an 
;      inclusive list (23-40).  The extension can be a sequential list
;      only.  If the batch file already exists, then the string vector
;      is appended to it.
; 
; CALLING SEQUENCE:
;      batch_sv
; 
; INPUTS: 
;      NONE
; 
; OUTPUT: 
;      The string vector is saved to the file specified at runtime.
; 
; EXTERNAL CALLS: 
;      COYOTE_FIELD, FILEX, GETWRD, NUMCHAR, TAG_EXIST
; 
; COMMON BLOCKS: 
;      NONE 
; 
; MODIFICATION HISTORY:
;      - Written by Eric Volquardsen, NASA IRTF, March 2004
; 
; =========================================================================
;                           EVENT HANDLER MODULE
;                          ----------------------
PRO batch_sv_event, event

; GET EVENTS AND COMMON BLOCK INFO
  widget_control, event.id, get_uvalue=fp
  widget_control, event.top, get_uvalue=fn
  CASE fp OF

; -----FILENAME FIELD-----

       'fnm' : BEGIN

            fnm = fn.fnm
            widget_control, event.id, get_value=f
            fnm = (*f)
            fn.fnm = fnm
            widget_control, event.top, set_uvalue=fn, /no_copy

       END ; filename field

; -----BROWSE BUTTON-----

       'fbs' : BEGIN

; Use native file browser to select path
            br = dialog_pickfile(filter='*', /read)

; Put filename in data structure and corresponding field
            fnm = fn.fnm
            fn.fnm = br
            widget_control, event.handler, set_value=br
            widget_control, event.top, set_uvalue=fn, /no_copy

       END ; browse button

; -----STRING NAME FIELD-----

       'snm' : BEGIN

            snm = fn.snm
            widget_control, event.id, get_value=f
            snm = (*f)
            fn.snm = snm
            widget_control, event.top, set_uvalue=fn, /no_copy

       END ; string name field


; ----- CONSTANT FIELD -----

       'con' : BEGIN

            widget_control, event.id, get_value=co
            fn.con = (*co)
            widget_control, event.top, set_uvalue=fn, /no_copy

       END ; constant field

; ----- VARIATION FIELD ------

       'var' : BEGIN

            widget_control, event.id, get_value=va
            fn.var = (*va)
            widget_control, event.top, set_uvalue=fn, /no_copy

       END ; variation field

; ----- EXTENSION FIELD -----

       'ext' : BEGIN

            widget_control, event.id, get_value=ex
            fn.ext = (*ex)
            widget_control, event.top, set_uvalue=fn, $ 
                 /no_copy

       END ; extension field

; ----- EXECUTE BUTTON -----

       'exe' : BEGIN

; Get number of delimitations in var and ext
            cmav = numchar(fn.var, ',')
            cmae = numchar(fn.ext, ',')
            hypv = numchar(fn.var, '-')

; Generate filenames
            IF cmav[0] EQ 0 THEN BEGIN   ; constant var
                 IF cmae[0] EQ 0 THEN BEGIN  ; constant ext
                      IF hypv[0] EQ 0 THEN BEGIN
                           list = fn.con+fn.var+fn.ext
                      ENDIF ELSE BEGIN
                           snums = getwrd(fn.var, 0, $ 
                                delimiter='-')
                           bnums = getwrd(fn.var, 1, $ 
                                delimiter='-')
                           snum = fix(snums)
                           bnum = fix(bnums)
                           rng = bnum-snum
                           list = strarr(rng+1)
                           FOR i = snum, bnum DO BEGIN
                                test0 = strtrim(string(i), 2)
                                test1 = strlen(test0)
                                test2 = strlen(snums)
                                IF test1 LT test2 THEN BEGIN
                                     list[(i-snum)] = fn.con+'0'+test0
                                ENDIF ELSE $
                                     list[(i-snum)] = fn.con+test0
                           ENDFOR   ; i loop
                           list[*] = list[*] + fn.ext
                      ENDELSE
                 ENDIF ELSE BEGIN   ; variable ext
                      IF hypv[0] EQ 0 THEN BEGIN
                           list = strarr(cmae[0]+1)
                           FOR i = 0, cmae[0] DO BEGIN
                                x = getwrd(fn.ext,i, delimiter=',')
                                list[i] = fn.con+fn.var+x
                           ENDFOR
                      ENDIF ELSE BEGIN
                           snums = getwrd(fn.var, 0, $ 
                                delimiter='-')
                           bnums = getwrd(fn.var, 1, $ 
                                delimiter='-')
                           snum = fix(snums)
                           bnum = fix(bnums)
                           rng = bnum-snum
                           list = strarr(rng+1)
                           FOR i = snum, bnum DO BEGIN
                                test0 = strtrim(string(i), 2)
                                test1 = strlen(test0)
                                test2 = strlen(snums)
                                IF test1 LT test2 THEN BEGIN
                                     list[(i-snum)] = fn.con+'0'+test0
                                ENDIF ELSE list[(i-snum)] = fn.con+test0
                           ENDFOR   ; i loop
                           x = strarr(cmae[0]+1)
                           FOR i = 0, cmae[0] DO BEGIN
                                x[i] = getwrd(fn.ext,i, $ 
                                     delimiter=',')
                           ENDFOR
                           j = 0 
                           FOR i = 0, rng DO BEGIN
                                IF j GT cmae[0] THEN j = 0 
                                list[i] = list[i] + x[j]
                                j = j + 1 
                           ENDFOR
                      ENDELSE
                 ENDELSE
            ENDIF ELSE BEGIN   ; variable var case
                 IF cmae[0] EQ 0 THEN BEGIN  ; constant ext
                      list = strarr(cmav[0]+1)
                      FOR i = 0, cmav[0] DO BEGIN
                           x = getwrd(fn.var,i, delimiter=',')
                           list[i] = fn.con+x+fn.ext
                      ENDFOR
                 ENDIF ELSE BEGIN   ; variable ext
                      list = strarr(cmav[0]+1)
                      extv = strarr(cmae[0]+1)
                      FOR j = 0, cmae[0] DO BEGIN
                           extv[j] = getwrd(fn.ext, j, $ 
                                delimiter=',')
                      ENDFOR
                      j = 0
                      FOR i = 0, cmav[0] DO BEGIN
                           IF j GT cmae[0] THEN j = 0
                           x = getwrd(fn.var,i, delimiter=',')
                           list[i] = fn.con+x+extv[j]
                           j = j + 1
                      ENDFOR
                 ENDELSE
            ENDELSE

; SAVE TO BATCH FILE
            IF fn.fnm EQ '' THEN BEGIN 
                 print, ' PLEASE ENTER FILENAME '
                 print, string(7b), format='(a,$)'
                 wait, 0.3
                 print, string(7b), format='(a,$)'
                 wait, 0.3
                 print, string(7b), format='(a,$)'
                 RETURN
            ENDIF 
            nl = n_elements(list)
            IF nl LT 2 THEN BEGIN 
                 print, ' NO STRING VECTOR CREATED '
                 print, string(7b), format='(a,$)'
                 wait, 0.3
                 print, string(7b), format='(a,$)'
                 wait, 0.3
                 print, string(7b), format='(a,$)'
            ENDIF ELSE BEGIN 
                 d_chk = FILEX(fn.fnm, '')
                 IF d_chk EQ 0 THEN openw, logun, fn.fnm, /get_lun $ 
                      ELSE openu, logun, fn.fnm, /append, /get_lun
                 printf, logun, '  '
                 pline = fn.snm + " = ['" + list[0] + "', $ "
                 printf, logun, pline
                 FOR i = 1, (nl-2) DO BEGIN 
                      pline = "'" + list[i] + "', $ "
                      printf, logun, pline
                 ENDFOR  
                 pline = "'" + list[nl-1] + "'] "
                 printf, logun, pline
                 free_lun, logun
                 print, string(7b), format='(a,$)'   
            ENDELSE 

       END ; execute button

; -----QUIT BUTTON-----

       'ret' : BEGIN

            widget_control, event.top, /destroy

       END ; quit button

  ENDCASE   ; event separation

END ; event handler

; ==========================================================================
;                         WIDGET DEFINITION MODULE
;                        --------------------------
PRO batch_sv

; VALUE INITIALIZATION
  fna = ''      ; full filename of batch file
  sna = ''      ; name of string vector
  con = ''      ; constant part of filename
  var = ''      ; varying part of filename
  ext = ''      ; filename extension 

; WIDGETS
; top level base
  fileb = widget_base(title='BATCH FILE GENERATOR', /column)

; other bases
  nambs = widget_base(fileb, /column, /frame)
  fldbs = widget_base(fileb, /row, /frame)
  btnbs = widget_base(fileb, /row, /align_center)

; fields
  fname = coyote_field(nambs, uvalue='fnm', $ 
       value=fnm, /cr_only, title=' BATCH NAME', $ 
       event_pro='batch_sv_event', xsize=50)
  sname = coyote_field(nambs, uvalue='snm', $ 
       value=snm, /cr_only, title='STRING NAME', $ 
       event_pro='batch_sv_event', xsize=12)
  conid = coyote_field(fldbs, uvalue='con', value=con, $ 
       /cr_only, title='Constant', /column, $ 
       event_pro='batch_sv_event', xsize=25)
  varid = coyote_field(fldbs, uvalue='var', value=var, $ 
       /cr_only, title='Variation', /column, $ 
       event_pro='batch_sv_event', xsize=30)
  extid = coyote_field(fldbs, uvalue='ext', value=ext, $ 
       /cr_only, title='Extension', /column, $ 
       event_pro='batch_sv_event', xsize=5)

; buttons  
  fbrws = widget_button(fname, value='BROWSE', $ 
       uvalue='fbs')
  exebt = widget_button(btnbs, value='EXECUTE', uvalue='exe', xsize=70, $ 
       ysize=30)
  retbt = widget_button(btnbs, value='QUIT', uvalue='ret', xsize=70, $ 
       ysize=30)

; WIDGET ACTIVATION
  fn = { fnm:fnm, snm:snm, con:con, var:var, ext:ext }
  widget_control, fileb, set_uvalue=fn
  widget_control, fileb, /realize
  xmanager, 'batch_sv', fileb

END ; widget definition
