; NAME: 
;      BP_MASK
; 
; VERSION: 1.1
; 
; PURPOSE: 
;      Creates a bad pixel mask from an image by limiting the 
;      allowable difference between a pixel and its surrounding 
;      pixels.  Upper and lower limits can be placed also.
;      Good pixels are labeled with 0 and bad with 1.
; 
; CALLING SEQUENCE: 
;      bp_mask, infile, outfile, paths, [box=box, sd=sd, plt=plt, 
;               cutoff=cutoff, help=help]
; 
; INPUTS: 
;      infile - Filename of image to use in creating mask
;      outfile - Filename of mask
;      paths - Two element string vector of infile and outfile paths
; 
; OPTIONAL KEYWORD INPUTS: 
;      box - Box size in determining local standard deviation
;            ( 3 is default)
;      sd - Limit for determining bad pixels ( 3 is default)
;      plt - Set to plot mask
;      cutoff - Vector with lower and upper allowed values.
;               Image values < cutoff[0] or > cutoff[1] are marked
;               as bad pixels.
;      help - Set to nonzero scalar to print syntax
; 
; OUTPUT: 
;      Mask is created and saved at outfile with header similar to 
;      infile.  The box size and standard deviation limit parameters 
;      are saved to the outfile header under the MASK_VAL keyword.
; 
; EXTERNAL CALLS: 
;      ACRE, FXADDPAR, FXWRITE, GETIM
; 
; COMMON BLOCKS: 
;      NONE
; 
; MODIFICATION HISTORY: 
;      - Written by Eric Volquardsen, NASA IRTF, March 2004
;      - Added cutoff keyword, ELV, May 2004
; 
; =======================================================================
PRO bp_mask, infile, outfile, paths, box = box, sd = sd, plt = plt, $ 
             cutoff = cutoff, help = help

; CHECK INPUTS
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN help = 1 
     ENDIF ELSE help = 0 
     nin = n_elements(infile)
     IF nin NE n_elements(outfile) THEN help = 1
     IF nin NE 1 THEN help = 1
     IF help EQ 1 THEN BEGIN
          print, '******************************************************'
          print, '***               BP_MASK  SYNTAX                  ***'  
          print, '*** IDL> bp_mask, infile, outfile, paths, box=box, ***' 
          print, '***               sd=sd, plt=plt, cutoff=cutoff    ***' 
          print, '***                                                ***'
          print, '*** infile - Filename of image                     ***'
          print, '*** outfile - Filename of mask                     ***' 
          print, '*** paths - Two element string vector of infile    ***'
          print, '***         and outfile paths                      ***'
          print, '*** box - Box size used for finding bad pixels     ***' 
          print, '*** sd - Deviation limits used to determine bad px ***'
          print, '*** plt - Set to plot mask                         ***' 
          print, '*** cutoff - lower and upper allowed values        ***'
          print, '******************************************************'   
          RETURN
     ENDIF
     np = n_elements(paths)
     CASE np OF 
          0 : BEGIN
               v = !version.os_family
               CASE v OF
                    'vms' : root = '[]'
                    'windows' : root = 'c:\'
                    ELSE : root = '/'
                ENDCASE
                cd, root, current=pat
                cd, pat
                paths = [pat, pat]
          END
          1 : paths = [paths, paths]
          ELSE : paths = paths[0:1]
     ENDCASE
     IF n_elements(box) NE 1 THEN box = 3
     IF n_elements(sd) NE 1 THEN sd = 3
     IF n_elements(plt) NE 1 THEN plt = 0

; LOAD IMAGE 
     GETIM, infile, im, paths[0], hdr = h
     im = float(im)

; CREATE MASK
     ACRE, im, im, sd, box, mask = mask

; APPLY CUTOFF VALUES
     IF n_elements(cutoff) EQ 2 THEN BEGIN 
          IF cutoff[0] GT cutoff[1] THEN cutoff = [cutoff[1], cutoff[0]]
          w1 = where(im LT cutoff[0])
          w2 = where(im GT cutoff[1])
          IF w1[0] NE -1 THEN mask[w1] = 1
          IF w2[0] NE -1 THEN mask[w2] = 1
     ENDIF 
     mask = byte(mask)

; PLOT MASK (if desired)
     IF plt GE 1 THEN BEGIN 
          sz = size(mask,  /dimension)
          window, /free, xsize = sz[0], ysize = sz[1]
          tvscl, mask
     ENDIF 

; ALTER HEADER
     mask_val = string(box) + string(sd)
     print, '  '
     print, 'BAD PIXEL MASK CREATED'
     print, mask_val
     print, '  '
     FXADDPAR, h, 'MASK_VAL', mask_val
     FXADDPAR, h, 'BITPIX', 8
     FXADDPAR, h, 'OBJECT', 'BAD PIXEL MASK'

; SAVE RESULTS
     cd, paths[1], current=nowp
     FXWRITE, outfile, h, mask
     cd, nowp

END
