; COMB_SPEC
; 
; VERSION: 1.1
;
; PURPOSE: 
;      Combines multiple spectra produced by TELLCOR.PRO by 
;      obtaining the weighted mean for each line.  Results can 
;      also be rebinned.
; 
; CALLING SEQUENCE: 
;      comb_spec, fn, outn, [rebin=rebin, help=help]
; 
; INPUTS: 
;      fn - String of full spectral filenames (including path)
;      outn - Name of output file
; 
; KEYWORD INPUTS: 
;      rebin - Set to number of lines to combine (integer)
;      help - Set to nonzero scalar to print syntax
; 
; OUTPUT: 
;      Results saved in OUTN
; 
; EXTERNAL CALLS: 
;      READ_SPEC, REBIN_SPEC
; 
; COMMON BLOCKS: 
;      NONE 
; 
; MODIFICATION HISTORY: 
;      - Written by Eric Volquardsen, NASA IRTF, October 2004
; 
; =======================================================================
PRO comb_spec, fn, outn, rebin = rebin, help = help

; CHECK INPUTS
     na = n_elements(fn)
     IF na LT 1 THEN help = 1
     IF n_elements(outn) NE 1 THEN help = 1
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN BEGIN 
               print, '***********************************************'
               print, '***          COMB_SPEC SYNTAX               ***' 
               print, '*** > comb_spec, fn, outn, rebin=rebin      ***' 
               print, '***                                         ***' 
               print, '*** fn - String of full spectral filenames  ***' 
               print, '*** outn - Output filename                  ***' 
               print, '*** rebin - Set to # of lines to combine    ***'
               print, '***********************************************' 
              RETURN
         ENDIF 
     ENDIF 
     IF n_elements(rebin) NE 1 THEN rebin = 1
     rebin = ABS(FIX(rebin)) > 1

; SEPARATE PATH FROM FILENAME
     p1 = STRPOS(fn, '/', /reverse_search)
     p2 = STRPOS(outn, '/', /reverse_search)
     p = STRMID(fn, 0, p1+1)
     p = TRANSPOSE(p[0, *])
     pp = STRMID(outn, 0, p2+1)
     fnn = STRMID(fn, p1+1)
     fnn = TRANSPOSE(fnn[0, *])
     outnn = STRMID(outn, p2+1)

; READ FILES AND SCALE CONTINUA
     speca = READ_SPEC(fnn[0], p[0], /flag)
     IF na EQ 1 THEN GOTO, skp1
     nl = n_elements(speca[0, *])
     spec = fltarr(4, nl, na)
     w = WHERE(speca[0, *] LE 2.5)
     cont = speca[*, w]
     w = WHERE(cont[3, *] GE 1)
     cont1 = SDEV(cont[1, w])
     sde = MEDIAN(cont[2, w])
     speca[1, *] = speca[1, *]/cont1[0]
     speca[2, *] = speca[2, *]*cont1[1]/sde/cont1[0]
     spec[*, *, 0] = speca
     FOR i = 1, (na-1) DO BEGIN 
          spc = READ_SPEC(fnn[i], p[i], /flag)
          w = WHERE(spc[0, *] LE 2.5)
          cont = spc[*, w]
          w = WHERE(cont[3, *] GE 1)
          cont1 = SDEV(cont[1, w])
          sde = MEDIAN(cont[2, w])
          spc[1, *] = spc[1, *]/cont1[0]
          spc[2, *] = spc[2, *]*cont1[1]/sde/cont1[0]
          nll = n_elements(spc[0, *])
          IF nll GE nl THEN BEGIN 
               spec[*, *, i] = spc[*, 0:(nl-1)]
          ENDIF ELSE BEGIN 
               spec[*, 0:(nll-1), i] = spc
          ENDELSE 
     ENDFOR 

; GET WEIGHTED MEAN FLUX
     FOR i = 0, (nl-1) DO BEGIN 
          w = where(spec[3, i, *] GE 1, good)
          IF good GT 0 THEN BEGIN 
               wts = 1.0/(spec[2, i, w]^2)
               sum = total(wts*spec[1, i, w])
               speca[1, i] = sum/total(wts)
               speca[2, i] = sqrt(1/total(wts))
               IF n_elements(spec[1, i, w]) GE 2 THEN BEGIN 
                    tempdev = sdev(spec[1, i, w])
                    IF n_elements(tempdev) EQ 2 THEN BEGIN 
                         IF speca[2, i] GT tempdev[1] THEN $ 
                              speca[2, i] = tempdev[1] 
                    ENDIF 
               ENDIF 
               speca[3, i] = good
          ENDIF ELSE BEGIN 
               speca[1:2, i] = median(spec[1:2, i, *])
               speca[3, i] = -1
          ENDELSE 
     ENDFOR 

; SCALE CONTINUUM TO 1
     skp1: 
     w = WHERE(speca[0, *] LE 2.5)
     cont = speca[*, w]
     w = WHERE(cont[3, *] GE 1)
     cont1 = SDEV(cont[1, w])
     sde = MEDIAN(cont[2, w])
     speca[1, *] = speca[1, *]/cont1[0]
     speca[2, *] = speca[2, *]*cont1[1]/sde/cont1[0]

; REBIN SPECTRUM
     IF rebin NE 1 THEN speca = REBIN_SPEC(speca, rebin, /flags)

; SAVE RESULTS
     w = WHERE(speca[3, *] GE 1)
     IF na EQ 1 THEN txt = fn ELSE txt = 'COMBINED SPECTRUM'
     plot, speca[0, w], speca[1, w], psym = 1, title = txt
     cd, pp
     openw, logun, outnn, /get_lun
     printf, logun, speca
     free_lun, logun

END
