; NAME: 
;      DEBIAS
; 
; VERSION: 1
; 
; PURPOSE: 
;      Subtracts bias and dark values from CCD images.
; 
; CALLING SEQUENCE: 
;      debias, darks, bias, others, outfn, [path=path, help=help])
; 
; INPUTS: 
;      darks - String vector of dark filenames.  Set equal to ''
;              to only subtract bias value.
;      bias - String vector of bias filenames.  If bias set equal
;             to '', then bias level will be determined from darks.
;      others - String vector of CCD images and flats that have
;               the overscan removed.
;      outfn - String vector of output filenames (must have same
;              number of elements as others).
; 
; OPTIONAL KEYWORD INPUTS: 
;      path - String vector of the paths for darks, bias, others, and
;             output filenames.  If only two paths are given, then 
;             input, output form is assumed.  If three paths are
;             given, then darks and bias are assumed to be in same
;             directory.
;      help - Set to nonzero scalar to print syntax
; 
; OUTPUT: 
;      Results are saved to outfn filenames
; 
; EXTERNAL CALLS: 
;      FXADDPAR, FXPAR, FXWRITE, GETIM
; 
; COMMON BLOCKS: 
;      NONE
; 
; MODIFICATION HISTORY: 
;      - Written by Eric Volquardsen, NASA IRTF, October 2003
; 
; =======================================================================
PRO debias, darks, bias, others, outfn, path=path, help=help

; CHECK INPUTS
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN help = 1 
     ENDIF ELSE help = 0 
     nd = n_elements(darks)
     nb = n_elements(bias)
     noth = n_elements(others)
     nout = n_elements(outfn)
     IF noth NE nout THEN help = 1 
     IF (nd*nb) EQ 0 THEN help = 1 
     IF help EQ 1 THEN BEGIN
          print, '**************************************************'
          print, '***                DEBIAS SYNTAX               ***'  
          print, '*** IDL> debias, darks, bias, others, outfn,   ***' 
          print, '***              path=path                     ***' 
          print, '***                                            ***' 
          print, '*** darks - String vector of dark filenames    ***' 
          print, '*** bias - String vector of bias filenames     ***' 
          print, '*** others - String vector of CCD image files  ***' 
          print, '*** outfn - String vector of output filenames  ***' 
          print, '*** path - String vector of the paths for      ***'
          print, '***        darks and bias files                ***' 
          print, '**************************************************'   
          RETURN
     ENDIF
     np = n_elements(path)
     CASE np OF 
          0 : BEGIN
               v = !version.os_family
               CASE v OF
                    'vms' : root = '[]'
                    'windows' : root = 'c:\'
                    ELSE : root = '/'
                ENDCASE
                cd, root, current=pat
                cd, pat
                path = [pat, pat, pat, pat]
          END
          1 : path = [path, path, path, path]
          2 : path = [path[0], path[0], path[0], path[1]]
          3 : path = [path[0], path[0], path[1], path[2]]
          ELSE : path = path[0:3]
     ENDCASE

; READ DARK AND BIAS FILES
     ar = fltarr(2, (nd+nb))
     FOR i = 0, (nd+nb-1) DO BEGIN
          IF i LE (nd-1) THEN BEGIN 
               IF darks[0] NE '' THEN BEGIN
                    GETIM, darks[i], im, path[0], hdr=h 
                    IF n_elements(im) LT 10 THEN RETURN
                    ar[1,i] = MEDIAN(im)
                    ar[0,i] = float(FXPAR(h, 'EXPTIME'))
               ENDIF
          ENDIF ELSE BEGIN 
               IF bias[0] NE '' THEN BEGIN
                    GETIM, bias[i-nd], im, path[1], hdr=h 
                    IF n_elements(im) LT 10 THEN RETURN
                    ar[1,i] = MEDIAN(im)
                    ar[0,i] = float(FXPAR(h, 'EXPTIME'))
               ENDIF
          ENDELSE 
     ENDFOR

; DETERMINE BIAS AND DARK VALUES
     IF bias[0] NE '' THEN b = MEAN(ar[1,nd:(nd+nb-1)]) ELSE b = 0.0
     ar[1,*] = ar[1,*] - b 
     IF darks[0] NE '' THEN $ 
          dc = LINFIT(ar[0,0:(nd-1)], ar[1,0:(nd-1)], yfit=y) $ 
          ELSE dc = [0.0, 0.0]
     IF bias[0] NE '' THEN dc[0] = b

; READ OTHER FILES
     FOR j = 0, (noth-1) DO BEGIN
          GETIM, others[j], img, path[2], hdr=h
          t = float(FXPAR(h, 'EXPTIME'))
          subt = dc[1]*t + dc[0]

; SUBTRACT BIAS AND DARK VALUES
          img = img - subt
          c1 = '= Bias subtraction value of ' + strtrim(string(dc[0]), 2)
          c2 = '= Dark subtraction value of ' + strtrim(string(dc[1]*t), 2)
          FXADDPAR, h, 'BIASSUB', c1
          FXADDPAR, h, 'DARKSUB', c2
          FXADDPAR, h, 'BITPIX', -32

; SAVE RESULTS
          cd, path[3], current=nowp
          FXWRITE, outfn[j], h, img
          cd, nowp
     ENDFOR

END
