; NAME: 
;      ELAPSE_T
; 
; PURPOSE: 
;      To convert a string vector of clock times to elapsed times
; 
; CALLING SEQUENCE: 
;      answer = elapse_t(c_times, [st_time=st_time, help=help])
; 
; INPUTS: 
;      c_times- String array of clock times. Must be in format HH:MM:SS 
;      
; OPTIONAL KEYWORD INPUTS: 
;      st_time- String with starting time.  If not passed ELAPSE_T
;               will use first element of c_times.
;      help- If set to nonzero value, ELAPSE_T will print syntax
;            and exit.
; 
; OUTPUT: 
;      answer- Integer vector of elapsed times in hours.
;              Has as many elements as c_times.
; 
; EXTERNAL CALLS: 
;      NUMCHAR, STRNUMBER
; 
; COMMON BLOCKS: 
;      NONE 
; 
; MODIFICATION HISTORY: 
;      -Written by Eric Volquardsen, NASA IRTF, Feb. 2003
; 
; =======================================================================
FUNCTION elapse_t, c_times, answer, st_time=st_time, help=help

; CHECK INPUTS
   IF n_elements(help) EQ 1 THEN BEGIN
        IF help NE 0 THEN help = 1 
   ENDIF ELSE help = 0 
   n = n_elements(c_times)
   IF n LT 1 THEN help = 1 
   s = size(c_times)
   IF s[0] EQ 0 THEN c_times = [c_times]
   cct = reform(c_times, n)
   sz = size(cct)
   IF sz[2] NE 7 THEN cct = string(cct)
   IF n_elements(st_time) NE 1 THEN st_time = cct[0]
   sz1 = size(st_time)
   IF sz1[1] NE 7 THEN st_time = string(st_time)
   FOR i = 0, (n-1) DO BEGIN
        test = cct[i]
        test1 = numchar(test, ':')
        ss = strtrim(test, 2)
        IF strlen(ss) NE 8 THEN test1 = 0 
        IF test1 NE 2 THEN help = 1 
   ENDFOR
   test2 = numchar(st_time, ':')
   ss = strtrim(st_time, 2)
   IF strlen(ss) NE 8 THEN test2 = 0 
   IF test1 NE 2 THEN help = 1 
   IF help EQ 1 THEN BEGIN
       print, '***************************************************'
       print, '***               ELAPSE_T SYNTAX               ***'
       print, '*** > answer=elapse_t(c_times, st_time=st_time) ***'
       print, '***                                             ***'
       print, '*** c_times- string array of times in HH:MM:SS  ***'
       print, '***          format                             ***'
       print, '*** st_time- starting time (optional)           ***'
       print, '*** answer - vector output of elapsed times     ***' 
       print, '***************************************************'
       RETURN, 0 
   ENDIF

; CONVERT TIME STRINGS TO NUMERIC VALUES
; Initialize values
   answer = fltarr(n)
   ztime = 0.0
   qt = 0 

; Get start time in seconds
   zhr = getwrd(st_time, 0, delimiter=':')
   zminit = getwrd(st_time, 1, delimiter=':')
   zsecnd = getwrd(st_time, 2, delimiter=':')
   IF strnumber(zhr) EQ 0 THEN qt = 1 
   IF strnumber(zminit) EQ 0 THEN qt = 1 
   IF strnumber(zsecnd) EQ 0 THEN qt = 1 
   IF qt EQ 1 THEN BEGIN
        print, '*************************************'
        print, '***            ELAPSE_T           ***'
        print, '*** Times must be numeric strings ***'
        print, '*************************************'
        print, string(7b), format='(a,$)'
        wait, 0.2   
        print, string(7b), format='(a,$)'   
        wait, 0.2
        print, string(7b), format='(a,$)'   
        RETURN, 0 
   ENDIF
   zhr = float(zhr)
   zminit = float(zminit)
   zsecnd = float(zsecnd)
   ztim = 3600*zhr+60*zminit+zsecnd

; Get elapse times
   FOR ii = 0, (n-1) DO BEGIN
        current = cct[ii]
        hr = getwrd(current, 0, delimiter=':')
        minit = getwrd(current, 1, delimiter=':')
        secnd = getwrd(current, 2, delimiter=':')
        IF strnumber(hr) EQ 0 THEN qt = 1 
        IF strnumber(minit) EQ 0 THEN qt = 1 
        IF strnumber(secnd) EQ 0 THEN qt = 1 
        IF qt EQ 1 THEN BEGIN
             print, '*************************************'
             print, '***            ELAPSE_T           ***'
             print, '*** Times must be numeric strings ***'
             print, '*************************************'
             print, string(7b), format='(a,$)'
             wait, 0.2   
             print, string(7b), format='(a,$)'   
             wait, 0.2
             print, string(7b), format='(a,$)'   
             RETURN, 0 
        ENDIF
        hr = float(hr)
        minit = float(minit)
        secnd = float(secnd)
        tim = 3600*hr+60*minit+secnd
        tim = tim/3600.0 
        ztim = ztim/3600.0
        answer[ii] = tim - ztim
   ENDFOR
   
   RETURN, answer

END     