; NAME: 
;      EXTRACT1D
; 
; VERSION: 1
; 
; PURPOSE: 
;      Extracts 1D spectrum by finding line profile centers, fitting
;      an aperture, and finding the difference between line aperture
;      and sky values.  Line count error is also calculated.  
;      Color correction is also performed.
; 
; CALLING SEQUENCE: 
;      extract1d, sfile, [xfile, paths=paths, aper=aper, sky=sky, 
;                arc=arc, fshft=fshft, coefs=coefs, help=help]
; 
; INPUTS: 
;      SFILE - Filename(s) of spectra to be extracted
;      
; OPTIONAL KEYWORD INPUTS: 
;      PATHS - Vector of paths for sfile, arc, and xfile 
;              (current is default)
;      APER - Set extraction aperture in pixels (max is 20, min is 1)
;             (default is radius equal to calculated fwhm)
;      SKY - Vector of inner and outer sky values
;            (default goes from aperture to 25 pixels from center)
;      ARC - Set to filename of calibration file to be extracted using 
;            sfile profile.
;      FSHFT - Set to full filename of extracted 2-D flat with line 
;              shift values in last row (GET_ORDERS output)
;      COEFS - 5 element vector giving color correction coefficients.
;              Set to order number to use default values.  Set to 
;              negative scalar to not perform correction.
;               1- [0.5661, 0.002053, -2.512e-6, 0.0, 0.0]
;               2- [0.3445, 0.001791, 1.959e-6, -8.76e-9, 4.993e-12]
;               3- [0.4367, 0.005397, -1.915e-5, 2.247e-8, -9.113e-12]
;               4- [0.2722, 0.003017, -4.899e-7, -9.618e-9, 6.923e-12]
;      HELP - Set to nonzero scalar to have XTRACT1D print syntax
;    
; OUTPUT: 
;      XFILE - Filename for extracted spectra saved in ascii format
;              If arc keyword is set then it will be saved. 
;              (default is named spectra.dat)
;              Spectra table has line number, counts, and count error.
; 
; EXTERNAL CALLS: 
;      BOOST_ARRAY, FULLWID_HALFMAX, GETIM, SDEV
; 
; COMMON BLOCKS: 
;      NONE 
; 
; MODIFICATION HISTORY: 
;      - Written by Eric Volquardsen, NASA IRTF, April 2003
; 
; =======================================================================
PRO extract1d, sfile, xfile, paths=paths, aper=aper, sky=sky, arc=arc, $ 
               fshft=fshft, coefs = coefs, help=help

; CHECK INPUTS
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN help = 1 
     ENDIF ELSE help = 0 
     nsf = n_elements(sfile) 
     IF nsf LT 1 THEN help = 1 
     IF help EQ 1 THEN BEGIN
          print, '**************************************************'
          print, '***             EXTRACT1D SYNTAX               ***'
          print, '*** > extract1d, sfile, xfile, paths=paths,    ***' 
          print, '***             aper=aper, sky=sky, arc=arc,   ***' 
          print, '***             fshft=fshft, coefs=coefs       ***' 
          print, '***                                            ***' 
          print, '*** SFILE - Files to be extracted              ***' 
          print, '*** XFILE - Filename for extracted spectra     ***' 
          print, '***         saved in ascii format.  If arc is  ***'
          print, '***         set then it will be saved.         ***'  
          print, '***         (default is named spectra.dat)     ***' 
          print, '*** PATHS - Vector of paths for sfile, arc,    ***' 
          print, '***         and xfile (current is default)     ***' 
          print, '*** APER - Set extraction aperture in pixels   ***' 
          print, '***        (default is equal to fwhm)          ***' 
          print, '*** SKY - Vector of inner and outer sky values ***' 
          print, '***       (default is aperture to 25 pixels)   ***' 
          print, '*** ARC - Set to filename of calibration file  ***' 
          print, '***       to be extracted using sfile profile. ***' 
          print, '*** FSHFT - Set to full filename of extracted  ***'
          print, '***         2-D flat with line shift values    ***' 
          print, '*** COEFS - 5 element vector of color          ***' 
          print, '***         correction coefficients or the     ***'
          print, '***         order number to use defaults       ***' 
          print, '**************************************************'
          RETURN 
     ENDIF 
     IF n_elements(xfile) NE 1 THEN xfile = 'spectra.dat'
     nep = n_elements(paths)
     CASE nep OF
          1 : paths = [paths, paths, paths]
          2 : paths = [paths[0], paths[0], paths[1]]
          3 : 
          else : BEGIN
               v = !version.os_family
               CASE v OF
                    'vms' : root = '[]'
                    'windows' : root = 'c:\'
                    ELSE : root = '/'
               ENDCASE
               cd, root, current=nowp
               paths = [nowp, nowp, nowp]
               cd, nowp
          END
     ENDCASE
     IF n_elements(aper) NE 1 THEN BEGIN 
          aper = 5
          ac = 1 
     ENDIF ELSE ac = 0 
     aper = aper > 1 < 20
     IF n_elements(sky) NE 2 THEN BEGIN 
          sky = [aper, 25]
          sc = 1 
     ENDIF ELSE sc = 0  
     IF n_elements(fshft) EQ 1 THEN BEGIN
          junk = findfile(fshft, count=checkf)
          IF checkf NE 0 THEN BEGIN
               fdecomp, fshft, dis, dr, nam, qual
               fnam = nam + '.' + qual
               getim, fnam, fim, dr
               sfim = size(fim, /dimensions)
               fsim = fltarr(sfim[1])
               fsim = fim[*, (sfim[1]-1)]
          ENDIF ELSE BEGIN 
               fshft = 0
               msg = 'FILE DOES NOT EXIST'
               print, msg
          ENDELSE
     ENDIF ELSE fshft = 0 
     IF n_elements(coefs) EQ 1 THEN BEGIN 
          CASE coefs OF 
               1 : coefs = [0.5661, 0.002053, -2.512e-6, 0.0, 0.0]
               2 : coefs = [0.3445, 0.001791, 1.959e-6, -8.76e-9, $ 
                            4.993e-12]
               3 : coefs = [0.4367, 0.005397, -1.915e-5, 2.247e-8, $ 
                            -9.113e-12]
               4 : coefs = [0.2722, 0.003017, -4.899e-7, -9.618e-9, $ 
                            6.923e-12]  
               ELSE : coefs = [1.0, 0.0, 0.0, 0.0, 0.0]
          ENDCASE     
     ENDIF 
     IF n_elements(coefs) NE 5 THEN coefs = [1.0, 0.0, 0.0, 0.0, 0.0]

; LOOP THROUGH FILES
     FOR i = 0, (nsf-1) DO BEGIN

; LOAD FILES
          getim, sfile[i], img, paths[0], hdr=hdr
          IF n_elements(img) EQ 1 THEN RETURN
          arcim = 0 
          IF n_elements(arc) GT 0 THEN BEGIN 
               FOR a_in = 0, (n_elements(arc)-1) DO BEGIN
                    getim, arc[a_in], arci, paths[1], hdr=hdr
                    IF n_elements(arcim) EQ 1 THEN arcim = arci $
                         ELSE arcim = arcim + arci
               ENDFOR 
          ENDIF
          ims = size(img, /dimensions)
          ma = min(arcim)
          arcim = arcim - ma + 1.0

; GET APERTURE POSITION
          subim1 = img[*, 0:(ims[1]/2-1)]
          subim2 = img[*, (ims[1]/2):(ims[1]-1)]
          si1t = total(subim1)
          si2t = total(subim2)
          IF si1t GT si2t THEN BEGIN 
               subim3 = subim1 
               IF n_elements(arcim) GT 1 THEN $ 
                    arcsim = arcim[*, 0:(ims[1]/2-1)]
          ENDIF ELSE BEGIN 
               subim3 = subim2
               IF n_elements(arcim) GT 1 THEN $ 
                    arcsim = arcim[*, (ims[1]/2+1):(ims[1]-1)]
          ENDELSE 
          fwtot = 0.0         

; Get average location of slit
          centol = 0 
          s = size(subim3, /dimensions)
          line = fltarr(s[1])
          subim3 = subim3 - min(subim3) + 1.0
          FOR k = 0, (ims[0]-1) DO BEGIN
               line[*] = subim3[k, *]
               fwhm = fullwid_halfmax(line, centroid=ctr)
               centol = centol+ctr
          ENDFOR
          centol = centol/ims[0]

; Get line statistics
          FOR k = 0, (ims[0]-1) DO BEGIN
               line[*] = subim3[k, *]
               fwhm = fullwid_halfmax(line, centroid=ctr)
               ctr = ctr >(centol-2) <(centol+2)
               shiftl = 25 - ctr
               fwtot = fwtot + fwhm

; Shift profile center to row 25
               IF n_elements(arcsim) GT 1 THEN BEGIN
                    aline = arcsim[k, *]
                    aline = shift(aline, shiftl)
                    arcsim[k, *] = aline 
               ENDIF ELSE BEGIN 
                    line = shift(line, shiftl)
                    subim3[k, *] = line
        
               ENDELSE 
          ENDFOR   ; line loop


; Calculate aperture and subimage
          IF ac EQ 1 THEN aper = (fwtot / ims[0]) > 1 < 15
          IF sc EQ 1 THEN sky = [aper+1, 25]
          aper = fix(aper)
          sky = fix(sky)
          IF n_elements(arcsim) GT 1 THEN $ 
               subim = extrac(arcsim, 0, (25 - sky[1]), $ 
               ims[0], (2*sky[1]+1)) $ 
               ELSE subim = extrac(subim3, 0, (25 - sky[1]), $ 
               ims[0], (2*sky[1]+1))

; Combine files
          IF n_elements(sumi) EQ 1 THEN sumi = subim $ 
               ELSE boost_array, sumi, subim
     ENDFOR
     IF nsf EQ 1 THEN subim = sumi ELSE subim = total(sumi, 3)

; EXTRACT LINE VALUES
     xsp = strarr(3, ims[0])
     skycf = 0.0 
     FOR j = 0, (ims[0]-1) DO BEGIN
          line = fltarr(2*sky[1]+1)
          line[*] = subim[j, *]
          apline = extrac(line, (sky[1]-aper), (2*aper+1))
          counts = total(apline)

; Adjust aperture for fractional pixel shifts
          IF n_elements(fsim) GT 1 THEN BEGIN
               fb = fsim[j] - fix(fsim[j])
               bm1 = (line[sky[1]-aper] + line[sky[1]-aper+1])/2.0
               bm2 = (line[sky[1]+aper+1] + line[sky[1]+aper+2])/2.0
               counts = counts - (bm1 - bm2)*fb
          ENDIF
          ctf = counts

; Get aperture counts
          skyc = (total(line) - counts)/(2*sky[1]-2*aper+1)
          counts = counts - skyc*(2*aper+1)
          skyl = fltarr(2*sky[1]-2*sky[0]+2)
          skyl[0:(sky[1]-sky[0])] = line[0:(sky[1]-sky[0])]
          skyl[(sky[1]-sky[0]+1):(2*sky[1]-2*sky[0]+1)] = $ 
               line[(sky[1]+sky[0]):2*sky[1]]

; Calculate line uncertainty
          sss = sdev(skyl)
          sss = sss[1]*sss[1]
          scat = ((2*aper+2)/(2*aper+1))*sss
          cerr = sqrt(abs(scat + counts))

; Create output array
          xsp[0, j] = string(j)
          xsp[1, j] = string(counts)
          xsp[2, j] = string(cerr)

     ENDFOR

; PERFORM COLOR CORRECTION
     ind = float(xsp[0, *])
     cc = coefs[4]*(ind^4) + coefs[3]*(ind^3) + $ 
          coefs[2]*(ind^2) + coefs[1]*ind + coefs[0]
     counts = float(xsp[1, *])/cc
     xsp[1, *] = string(counts)

; OUTPUT
     mct = min(float(xsp[1, *]))
     xsp[1, *] = string((float(xsp[1, *]) - mct + 1.0))
     apc = ctf/(2.0*aper+1)
     window, 1, xsize=700, ysize=500, title='LINE EXTRACTION'
     wset, 1
     plot, xsp[0,*], xsp[1,*]
     print, 'Selected aperture radius = ', strtrim(string(aper), 2)
     print, '  Ave. aperture value    = ', strtrim(string(apc), 2)
     print, 'Selected sky radii       = ', strtrim(string(sky), 2)
     print, '  Ave. sky value         = ', strtrim(string(skyc), 2)
     cd, paths[2], current=nowp
     openw, logun2, xfile, /get_lun
     printf, logun2, xsp
     free_lun, logun2
     cd, nowp

END
