; NAME: 
;      FLAT_FLD
; 
; VERSION: 1
; 
; PURPOSE: 
;      Performs flat-fielding on files using superflat and bad pixel mask.
; 
; CALLING SEQUENCE: 
;      flat_fld, inames, fname, [ffnames, paths=paths, boxw=boxw, 
;                help=help]
; 
; INPUTS: 
;      INAMES - Names of files to be flat-fielded
;      FNAME - Name of flat
; 
; OPTIONAL KEYWORD INPUTS: 
;      PATHS - Vector of path names for inames, fname, and ffnames
;      BOXW - Two-element vector with dimensions of moving box
;             used to find median for replacing bad pixels (default is 7).
;      HELP - Set to nonzero scalar to have flat_fld print syntax
; 
; OUTPUT: 
;      FFNAMES - Output filenames of flat fielded files.  If not specified, 
;                results will be saved in original files.  If fewer names
;                listed than in inames, then images will be combined into
;                single output file.
; 
; EXTERNAL CALLS: 
;      FITS_CLOSE, FITS_WRITE, FXADDPAR, GETIM
; 
; COMMON BLOCKS: 
;      NONE 
; 
; MODIFICATION HISTORY: 
;      - Written by Eric Volquardsen, NASA IRTF, April 2003
; 
; =========================================================================
PRO flat_fld, inames, fname, ffnames, paths=paths, boxw=boxw, help=help

; CHECK INPUTS
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN help = 1 
     ENDIF ELSE help = 0 
     nf = n_elements(inames)
     IF nf LT 1 THEN help = 1 
     IF n_elements(fname) NE 1 THEN help = 1
     IF n_elements(ffnames) LT nf THEN ffnames = inames 
     IF help EQ 1 THEN BEGIN
          print, '********************************************************'
          print, '***                 FLAT_FLD SYNTAX                  ***'
          print, '*** > flat_fld, inames, fname, ffnames, paths=paths, ***'
          print, '***             boxw=boxw                            ***'
          print, '***                                                  ***'
          print, '*** INAMES - Names of files to be flat-fielded       ***'
          print, '*** FNAME - Filename of flat                         ***'
          print, '*** FFNAMES - Output filenames of flat fielded files ***'
          print, '***           If not specified, results will be      ***'
          print, '***           saved under original file names.       ***'
          print, '*** PATHS - Vector of path names for inames, fname,  ***'
          print, '***         and ffnames                              ***'
          print, '*** BOXW - Two-element vector with dimensions of     ***'
          print, '***        moving box used to find median for        ***' 
          print, '***        replacing bad pixels (default is 7).      ***'
          print, '********************************************************'
          RETURN
     ENDIF
     IF n_elements(boxw) NE 2 THEN boxw = [7,7]
     nep = n_elements(paths)
     CASE nep OF
          1 : paths = [paths, paths, paths]
          3 : 
          else : BEGIN
               v = !version.os_family
               CASE v OF
                    'vms' : root = '[]'
                    'windows' : root = 'c:\'
                    ELSE : root = '/'
               ENDCASE
               cd, root, current=nowp
               pa = [nowp, nowp, nowp]
               cd, nowp
          END
     ENDCASE

; LOAD FLAT AND MASK
     getim, fname, flat, paths[1]
     IF n_elements(flat) EQ 1 THEN RETURN
     getim, fname, mask, paths[1], extend=1
     pts = where(mask EQ 1)
     nbadp = n_elements(pts)
     sf = size(flat, /dimension)
     pxlx = pts mod sf[0]
     pxly = pts/sf[0]

; LOOP THROUGH IMAGE FILES
     nsif = n_elements(size(inames, /dimensions))
     IF nsif GT 1 THEN inames = reform(inames, nf)
     FOR i = 0, (nf-1) DO BEGIN

; Load file 
          getim, inames[i], image, paths[0], hdr=hdr
          IF n_elements(image) EQ 1 THEN BEGIN 
               print, inames[i] + ' DOES NOT EXIST'
               GOTO, nofile
          ENDIF 
          image = float(image)

; Remove bad pixels
          image[pxlx, pxly] = !values.f_nan

; Loop through bad pixels
          FOR nbp = 0, (nbadp-1) DO BEGIN

; Get median of surrounding pixels
               subox = fltarr(2*boxw[0]+1, 2*boxw[1]+1)
               xlm1 = (pxlx[nbp]-boxw[0]) > 0 
               xlm2 = (pxlx[nbp]+boxw[0]) < (sf[0]-1)
               ylm1 = (pxly[nbp]-boxw[1]) > 0 
               ylm2 = (pxly[nbp]+boxw[1]) < (sf[1]-1)
               subox = image(xlm1:xlm2, ylm1:ylm2)
               sbxmed = median(subox, /even)
               IF where(finite(sbxmed)) EQ -1 THEN sbxmed = median(image)

; Replace bad pixel with median
               image[pxlx[nbp], pxly[nbp]] = sbxmed

          ENDFOR   ; pixel mask loop   

; Perform flat fielding
          fimage = image / flat

; Alter header
          fxaddpar, hdr, 'BITPIX', -32 
          ffl = '=  Flat-fielded with ' + fname
          fxaddpar, hdr, 'COMMENT', ffl

; Save flat-fielded file
          cd, paths[2], current=nowp
          fits_write, ffnames[i], fimage, hdr
          fits_close, fcb
          cd, nowp
          nofile:   ; skips flat fielding when no file 

     ENDFOR   ; image loop

END
