; NAME: 
;      FLUX
; 
; VERSION: 1.3
; 
; PURPOSE: 
;      To calculate aperture photometry values of images. 
;      Uses method in DAOPHOT.
;      
; CALLING SEQUENCE: 
;      flux, infile, outfile, path, [apert, skyrad, peak=peak, $
;      time=time, jy=jy, instrum=instrum, help=help]
; 
; INPUTS: 
;      infile - Filename of image
;      outfile - Filename where aperture photometry results are saved
;      path- Two element string vector with path for input FITS 
;            image and output ASCII file.
;      apert- Three element vector with:
;                1) Smallest aperture radius (default is 1.0)
;                   should be greater than 0.2
;                2) Largest aperture radius (default is 15.0)
;                   should not reach outer radius of sky annulus
;                3) Number of divisions (default is 15)
;                   up to 1000 divisions.  If a value less than 2 
;                   is passed, then it is set to 2.
;      skyrad- Two element vector defining sky annulus inner and 
;              outer radii (default is [20, 30]).
; 
; OPTIONAL KEYWORD INPUTS: 
;      peak- Estimated coordinates of peak.  Setting to [-1,-1] 
;            will result in searching FITS header for LOCATION 
;            keyword.
;      time- Exposure time to use in place of that in header
;      jy - Set to filter wavelength to print flux in Jy in place 
;           of magnitudes
;      instrum - Instrument used to take images
;                    0 - MIRSI (default)
;                    1 - APOGEE
;                    2 - NSFCAM
;      help- Set to nonzero scalar to print syntax
; 
; OUTPUT: 
;      Observer, instrument, telescope, object name, image file
;      name, observation date, observation time, filter wavelength, 
;      airmass, sky annulus radii, number of pixels in sky annulus, 
;      sky mode, sky standard deviation, sky skew, and image full-
;      width-half-max are saved in an ASCII file, along with a 
;      table of pixels in aperture, flux in aperture, flux
;      uncertainty, instrument magnitude, and magnitude uncertainty
;      for all aperture radii.  Instrument magnitude is calibrated
;      so that 1 count per second corresponds to a magnitude of 12.5
;      for MIRSI, 20 for NSFCAM, and 25 for APOGEE.  Flux uncertainty  
;      is given by 2.5 * fractional flux error.
; 
; EXTERNAL CALLS: 
;      DIST_CIRCLE, FULLWID_HALFMAX, FXPAR, GETIM, IM_STATS, JFLUX, 
;      MMM, NUMCHAR, PIXWT
; 
; COMMON BLOCKS: 
;      NONE
; 
; MODIFICATION HISTORY: 
;      - Written by Eric Volquardsen, NASA IRTF, March 2004
;      - Corrected problem with APOGEE data, ELV, June 2004
;      - Added JY keyword, ELV, December 2004
; 
; =========================================================================
PRO flux, infile, outfile, path, apert, skyrad, peak = peak, $ 
                time=time, jy = jy, instrum = instrum, help=help

; CHECK INPUTS 
   IF n_elements(infile) NE 1 THEN help = 1 
   IF n_elements(outfile) NE 1 THEN help = 1 
   IF n_elements(apert) NE 3 THEN apert = [1, 15, 15]
   IF n_elements(skyrad) NE 2 THEN skyrad = [20, 30]
   IF n_elements(help) EQ 1 THEN BEGIN
        IF help NE 0 THEN BEGIN
             print, '***********************************************'
             print, '***              FLUX SYNTAX                ***'
             print, '*** > flux, infile, outfile, path, apert,   ***'
             print, '***              skyrad, peak=peak, jy=jy,  ***'
             print, '***              time=time, instrum=instrum ***'
             print, '***                                         ***'
             print, '*** infile - Input filename                 ***' 
             print, '*** outfile - Output filename               ***'  
             print, '*** path- Two element string vector with    ***' 
             print, '***       path for input FITS image and     ***' 
             print, '***       output ASCII file.                ***'  
             print, '*** apert- Three element vector with:       ***' 
             print, '***         1) Smallest aperture radius     ***' 
             print, '***            (default is 1.0)             ***' 
             print, '***            should be > 0.2              ***' 
             print, '***         2) Largest aperture radius      ***' 
             print, '***            (default is 15.0)            ***'
             print, '***            should be < sky annulus      ***' 
             print, '***         3) Number of divisions          ***' 
             print, '***            (default is 15)              ***' 
             print, '***            up to 1000 divisions         ***' 
             print, '*** skyrad- Two element vector defining     ***' 
             print, '***         inner and outer sky radii       ***' 
             print, '***         (default is [20, 30])           ***'
             print, '*** peak- Coordinates of peak               ***' 
             print, '*** jy - Set to filter wavelength to print  ***' 
             print, '***      flux in Jy in place of magnitudes  ***'
             print, '*** time- Time to use in place of exposure  ***'
             print, '***       time in header                    ***'
             print, '*** instrum - Instrument used to take image ***'
             print, '***                0 - MIRSI (default)      ***'
             print, '***                1 - APOGEE               ***' 
             print, '***                2 - NSFCAM               ***'
             print, '***********************************************' 
             RETURN 
        ENDIF
   ENDIF ELSE help = 0 
   pa = n_elements(path) 
   v = !version.os_family
   CASE v OF
        'vms' : root = '[]'
        'windows' : root = 'c:\'
        ELSE : root = '/'
   ENDCASE
   CASE pa OF 
        0 : BEGIN
             CASE v OF
                  'vms' : root = '[]'
                  'windows' : root = 'c:\'
                  ELSE : root = '/'
             ENDCASE
             cd, root, current=pat
             cd, pat
             path = [pat, pat]
        END
        1 : path = [path, path]
        else : path = path[0:1]
   ENDCASE
   IF n_elements(instrum) NE 1 THEN instrum = 0
   CASE instrum OF 
        1 : instrum = 'APOGEE'
        2 : instrum = 'NSFCAM'
        ELSE : instrum = 'MIRSI'
   ENDCASE 
   IF apert[2] LT 2 THEN BEGIN
        print, ' '
        print, 'Number of increments in aperture increased to 2'
        print, ' '
        apert[2] = 2
   ENDIF 
   IF n_elements(jy) NE 1 THEN jy = 0
   IF jy LT 0 THEN jy = ABS(jy)

; INITIALIZE VALUES
   fluxx = lonarr(apert[2]) 
   fluxxer = lonarr(apert[2])
   a = intarr(apert[2])
   mag = fltarr(apert[2])
   mager = fltarr(apert[2])
   sky = 0.0 
   skyer = 0.0 
   uncer = 0.0 
   scater = 0.0 
   skymod = 0.0 
   skysig = 0.0 
   skyskw = 0.0 
   incr = (apert[1]-apert[0])/(apert[2]-1)
   radii = fltarr(apert[2])
   FOR ri = 0, (apert[2]-1) DO radii[ri] = apert[0] + ri*incr

; LOAD IMAGE
   getim, infile, image, path[0], hdr=hdr
   IF n_elements(image) EQ 1 THEN RETURN
   siz = size(image, /dimensions)
   IF n_elements(siz) GT 2 THEN image = image[*, *, 0]

; GET CENTROID AND FWHM
   IF n_elements(peak) NE 2 THEN BEGIN 
        inf = IM_STATS(image, /get_fwhm)
        center = inf.peak_xy 
        fw = inf.fwhm
   ENDIF ELSE BEGIN 
        IF peak[0] NE -1 THEN BEGIN  
             center = peak
             fwm1 = center[0]-apert[1]
             fwm2 = center[0]+apert[1]
             fwm3 = center[1]-apert[1]
             fwm4 = center[1]+apert[1]
             fw = FULLWID_HALFMAX(image[fwm1:fwm2, fwm3:fwm4], /average)
        ENDIF ELSE BEGIN 
             center = strsplit(fxpar(hdr, 'LOCATION'), /extract)
             IF n_elements(center) NE 2 THEN BEGIN 
                  print, '  '
                  print, ' LOCATION KEYWORD NOT PRESENT '
                  print, '  '
                  RETURN 
                  fwm1 = center[0]-apert[1]
                  fwm2 = center[0]+apert[1]
                  fwm3 = center[1]-apert[1]
                  fwm4 = center[1]+apert[1]
                  fw = FULLWID_HALFMAX(image[fwm1:fwm2, fwm3:fwm4], $ 
                       /average)
             ENDIF 
        ENDELSE
   ENDELSE 

; CALCULATE SKY VALUES
   skyrad = float(skyrad)
   rinsq = skyrad[0]^2 
   routsq = skyrad[1]^2 

; Check for sky passing image boundary
   lmt1 = center[0] + skyrad[1]
   lmt2 = center[0] - skyrad[1]
   lmt3 = center[1] + skyrad[1]
   lmt4 = center[1] - skyrad[1]
   qt = 0 
   IF lmt1 GT siz[0] THEN qt = 1 
   IF lmt2 LT 0 THEN qt = 1 
   IF lmt3 GT siz[1] THEN qt = 1 
   IF lmt4 LT 0 THEN qt = 1 
   IF qt EQ 1 THEN BEGIN
        print, '**********************************'
        print, '*** Sky is outside image edges ***'
        print, '**********************************'
        RETURN
   ENDIF

; Get subimage
   smx = fix(center[0]-skyrad[1])     
   lgx = fix(center[0]+skyrad[1])    
   nx = lgx-smx+1                    
   smy = fix(center[1]-skyrad[1])     
   lgy = fix(center[1]+skyrad[1])   
   ny = lgy-smy+1                   
   dx = center[0]-smx              
   dy = center[1]-smy     
   subimg = image[smx:lgx, smy:lgy]

; Get distance and square of distance from center
   dxsq = (findgen(nx) - dx)^2
   rsq = fltarr(nx,ny, /nozero)
   FOR ix = 0, (ny-1) DO rsq[0,ix] = dxsq + (ix-dy)^2

; Get arrays with distance along axis from center
   nbox = lindgen(nx*ny)
   xx = reform( (nbox mod nx), nx, ny)
   yy = reform( (nbox/nx),nx,ny)
   x1 = abs(xx-dx) 
   y1 = abs(yy-dy)  

; Select pixels within sky annulus
   sindex =  where((rsq GE rinsq) and (rsq LE routsq))
   nsky = n_elements(sindex)
   skyimg = subimg[ sindex[0:nsky-1] ]     

; Obtain sky statistics
   mmm, skyimg, skymod, skysig, skyskw
   skyvar = skysig^2   
   sigsq = skyvar/nsky 


; CREATE OUTPUT FILE SUBTITLES
   st1 = 'OBSERVER:                 '
   st2 = 'INSTRUMENT:               '
   st3 = 'LOCATION:                 '
   st4 = 'OBJECT NAME:              ' 
   st5 = 'REDUCED IMAGE FILENAME:   '
   st6 = 'OBSERVATION DATE:         ' 
   st7 = 'OBSERVATION TIME:         ' 
   st8 = 'FILTER WAVELENGTH:        ' 
   st9 = 'AIRMASS:                  ' 
   st10 = 'INNER SKY RADIUS:         ' 
   st11 = 'OUTER SKY RADIUS:         ' 
   st12 = 'PIXELS IN SKY ANNULUS:    ' 
   st13 = 'SKY MODE:                 ' 
   st14 = 'SKY STANDARD DEVIATION:   ' 
   st15 = 'SKY SKEW:                 '
   st16 = 'TOTAL INTEGRATION TIME:  ' 
   st17 = 'FWHM:                     '
   tt1 = 'Aperture   Aperture               c/s     '
   tt2 = ' radius     pixels     CTS/sec    Error   ' 
   tt3 = 'Instrument  Magnitude'
   tt4 = 'Magnitude     Error'
   usc = '---------|----------|----------|'
   usc = usc + usc
   tta = tt1 + tt3 
   ttb = tt2 + tt4 

; GET HEADER INFORMATION
   CASE instrum OF
        'MIRSI' : BEGIN 
             mx = 4
             zflux = 1e5
             tii = strtrim(fxpar(hdr, 'TIME-STR'), 2)
             it = float(fxpar(hdr, 'EXPTIME'))
             wv = float(fxpar(hdr, 'LAMBDA'))
             dob = strtrim(fxpar(hdr, 'DATE-OBS'), 2)
             aim = float(fxpar(hdr, 'AIRMASS'))
        END 
        'APOGEE' : BEGIN 
             mx = 1
             zflux = 1e10
             tii = strtrim(fxpar(hdr, 'TIME-OBS'), 2)
             it = float(fxpar(hdr, 'EXPTIME'))
             wv = float(fxpar(hdr, 'LAMBDA'))
             dob = strtrim(fxpar(hdr, 'DATE-OBS'), 2)
             aim = float(fxpar(hdr, 'SECZ'))
        END
        'NSFCAM' : BEGIN 
             mx = 1
             zflux = 1e8
             tii = strtrim(fxpar(hdr, 'TIME_OBS'), 2)
             it = float(fxpar(hdr, 'ITIME'))
             wv = float(fxpar(hdr, 'CVFWLEN'))
             dob = strtrim(fxpar(hdr, 'DATE_OBS'), 2)
             aim = float(fxpar(hdr, 'AIRMASS'))
             itmult = float(fxpar(hdr, 'DIVISOR'))
             it = it*itmult
        END
   ENDCASE
   obser = strtrim(fxpar(hdr, 'OBSERVER'), 2)
   loc = strtrim(fxpar(hdr, 'TELESCOP'), 2)
   ob = strtrim(fxpar(hdr, 'OBJECT'), 2)
   nf1 = fxpar(hdr, 'ORIGINAL')
   nf2 = numchar(nf1, '/')
   IF nf2 EQ 0 THEN nf2 = 1 
   nf = mx*nf2
   ti = strtrim(fxpar(hdr, 'MEANTIME'), 2)
   IF ti EQ '' THEN ti = tii
   ttime = nf*it
   IF n_elements(time) EQ 1 THEN BEGIN 
         IF time NE 0 THEN ttime = time
   ENDIF 

; LOOP THROUGH RADII
   FOR ri = 0, (apert[2]-1) DO BEGIN
                      
; CALCULATE FLUX
; Calculate radial values
        radius = radii[ri]
        area = !pi*(radius^2)
        bigrad = radius + 0.5 
        smallrad = radius/sqrt(2) - 0.5 
        cts = 0.0 
        ctser = 0.0 

; Find pixels within aperture
        mask = fltarr(nx,ny)
        good = where((x1 LT smallrad) AND $ 
             (y1 LT smallrad), ngood)
        IF ngood GT 0 THEN mask(good) = 1.0
        bad = where(  (x1 GT bigrad) OR (y1 GT bigrad ))
        mask(bad) = -1

; Evaluate pixels on border
        gfract = where(mask EQ 0.0, nfract) 
        IF nfract GT 0 THEN $ 
             mask = pixwt(dx,dy,radius,xx,yy)
        thisap = where(mask GT 0.0)
        thisapd = subimg[thisap]
        fractn = mask[thisap]

; Calculate total counts
        cts = total(thisapd*fractn)
        cts = cts - skymod*area  

; Calculate magnitude
        flux = cts/ttime
        mg = 2.5*alog10(zflux/flux)

; Calculate errors
        scater = area*skyvar       
        uncer = sigsq*area^2       
        ctser = sqrt(uncer + cts + scater)
        fluxer = ctser/ttime
        mger = 2.5*fluxer/flux

; Convert to Janskys (if desired)
        IF jy NE 0 THEN BEGIN 
             mg = JFLUX(mg, jy, uncer = mger)
             mger = mg[1]
             mg = mg[0]
        ENDIF 

; Put table information in arrays
        flux = long(flux)
        IF fluxer LT 30000 THEN $ 
             fluxer = round(fluxer) ELSE $ 
             fluxer = long(fluxer)
        fluxx[ri] = flux
        fluxxer[ri] = fluxer   
        a[ri] = round(area)
        mag[ri] = mg
        mager[ri] = mger

   ENDFOR   ; radii loop
   cd, '/'
   cd, path[1]

; PLOT RADIAL PROFILE 
   xc = fix(center[0]) 
   yc = fix(center[1])
   r = skyrad[1]
   subim = extrac(image,(xc-r),(yc-r),(2*r),(2*r))
   dist_circle, rarr, (2*r), (center[0]-xc+r), (center[1]-yc+r)   
   yr = max(subim, min=yrr)
   window, 3, title= infile
   ttl = 'RADIAL PROFILE: '+outfile
   plot, rarr, subim, xtitle='Radial pixel distance', title=ttl, $ 
         psym=7, yrange=[yrr, yr], xminor=2, $ 
         font=10, xmargin=[10,4], ytitle='stnuoC', $ 
         yminor=2, ymargin=[6,4], xrange=[0, skyrad[1]]
   wait, 2

; FORMAT HEADER OUTPUT
   wv = strtrim(string(wv), 2)+' microns'
   aim = strtrim(string(aim), 2)
   sk0 = strtrim(string(skyrad[0]), 2)+' pixels'
   sk1 = strtrim(string(skyrad[1]), 2)+' pixels'
   nsky = strtrim(string(nsky), 2)
   skymod = strtrim(string(skymod), 2) 
   skysig = strtrim(string(skysig), 2) 
   skyskw = strtrim(string(skyskw), 2) 
   ttime = strtrim(string(ttime), 2)+' sec' 
   fw = strtrim(string(fw), 2)+' pixels' 

; SAVE IMAGE RESULTS
;  Header
   openw, logun, outfile, /get_lun
   printf, logun, st1, obser
   printf, logun, st2, instrum
   printf, logun, st3, loc
   printf, logun, st4, ob 
   printf, logun, st5, infile
   printf, logun, st6, dob
   printf, logun, st7, tii
   printf, logun, st8, wv 
   printf, logun, st9, aim 
   printf, logun, st10, sk0 
   printf, logun, st11, sk1
   printf, logun, st12, nsky 
   printf, logun, st13, skymod 
   printf, logun, st14, skysig 
   printf, logun, st15, skyskw 
   printf, logun, st16, ttime
   printf, logun, st17, fw
   printf, logun, '                  '
   printf, logun, tta
   printf, logun, ttb 
   printf, logun, usc
   spc = '     '

;  Table 
   FOR ri = 0, (apert[2]-1) DO BEGIN
        rri = strtrim(string(radii[ri]), 2)
        rri = strmid(rri, 0, 5)
        ari = strtrim(string(a[ri]), 2)
        aril = strlen(ari)
        CASE aril OF 
             1: a_sp = '    '
             2: a_sp = '   '
             3: a_sp = '  '
             else: a_sp = ' '
        ENDCASE
        ari = a_sp+ari+'  '
        cri = strtrim(string(fluxx[ri]), 2)
        cril = strlen(strcompress(fluxx, /remove_all))
        cril = max(cril)
        cril2 = strlen(cri)
        IF cril NE cril2 THEN BEGIN
             dcril = cril-cril2
             CASE dcril OF
                  1: cri = ' ' + cri
                  2: cri = '  ' + cri
                  3: cri = '   ' + cri
                  else: cri = '    ' + cri
             ENDCASE
        ENDIF
        cei = strtrim(string(fluxxer[ri]), 2)
        ceil = strlen(cei)
        CASE ceil OF 
             1: b_sp = '   '
             2: b_sp = '  '
             3: b_sp = ' '
             else: b_sp = ''
        ENDCASE
        cei = b_sp+cei+' '
        mri = strtrim(string(mag[ri]), 2)
        mri = strmid(mri, 0, 7)
        mei = strtrim(string(mager[ri]), 2)
        meil = strlen(mei)
        IF meil GT 7 THEN mei = strmid(mei, 0, 7)
        tbout = ' '+rri+spc+ari+spc+cri+spc+cei
        tbout = tbout+spc+mri+spc+mei 
        printf, logun, tbout
   ENDFOR   ; table element loop
   free_lun, logun

END   
