; NAME: 
;      GET_ORDERS
; 
; VERSION: 1
; 
; PURPOSE: 
;      Separate orders of cross-dispersed spectra.
; 
; CALLING SEQUENCE: 
;      get_orders, infile, outfile, paths, [flt=flt, row=row, cols=cols, 
;                  wid=wid, ord=ord, dv=dv, help=help]
; 
; INPUTS: 
;      INFILE - Filename of spectrum
;      PATHS - Input and output paths
;      
; OPTIONAL KEYWORD INPUTS: 
;      FLT - Set to scalar if input file is a flat or to the filename 
;            of a reduced flat if input file is other than a flat.
;      ROW - Row of bottom of order to extract 
;            Default is based on order number   
;      COLS - [min, max] columns to extract  
;             Default is based on order number     
;      WID - Number of rows to extract in order (default is 101)
;            must be odd integer
;      ORD - Order number to put in comment in header (default is 1st)
;      DV - Divisor for line number to get rough slope of order
;           Default is based on order number (falls between 3 and 7).
;      PLT - Set to plot results
;      HELP - Set to nonzero keyword to have get_orders print syntax
; 
; OUTPUT: 
;      OUTFILE - Filename where result should be saved
; 
; EXTERNAL CALLS: 
;      GETIM
; 
; COMMON BLOCKS: 
;      NONE
; 
; MODIFICATION HISTORY: 
;      - Written by Eric Volquardsen, NASA IRTF, May 2003
; 
; =======================================================================
PRO get_orders, infile, outfile, paths, flt=flt, row=row, cols=cols, $ 
                wid=wid, ord=ord, dv=dv, plt = plt, help=help

; CHECK INPUTS
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN help = 1 
     ENDIF ELSE help = 0 
     IF n_elements(infile) NE 1 THEN help = 1 
     IF n_elements(outfile) NE 1 THEN help = 1 
     np = n_elements(paths) 
     CASE np OF 
          0 : BEGIN
               v = !version.os_family
               CASE v OF
                    'vms' : root = '[]'
                    'windows' : root = 'c:\'
                    ELSE : root = '/'
               ENDCASE
               cd, root, current=nowp
               paths = [nowp, nowp]
               cd, nowp
          END 
          1 : paths = [paths, paths]
          ELSE : paths = paths[0:1]
     ENDCASE
     IF n_elements(wid) NE 1 THEN wid = 101 
     wid = fix(wid*1.45)
     IF help EQ 1 THEN BEGIN
          print, '******************************************************'
          print, '***              GET_ORDERS SYNTAX                 ***' 
          print, '*** > get_orders, infile, outfile, paths, flt=flt, ***'
          print, '***               row=row, cols=cols, wid=wid,     ***' 
          print, '***               ord=ord, dv=dv, plt=plt          ***' 
          print, '***                                                ***'
          print, '*** INFILE - Filename of spectrum                  ***' 
          print, '*** OUTFILE - Filename where results               ***' 
          print, '*** PATHS - Input and output paths                 ***' 
          print, '*** FLT - Name of file with column shift vectors   ***' 
          print, '*** ROW - Row of bottom of order to extract        ***' 
          print, '*** COLS - [min, max] columns to extract           ***' 
          print, '*** WID - Number of rows to extract in order       ***' 
          print, '*** ORD - Order # to put in comment in header      ***' 
          print, '*** DV - Divisor of line number to get slope       ***' 
          print, '*** PLT - Set to plot results                      ***'
          print, '******************************************************' 
          RETURN
     ENDIF 
     IF n_elements(ord) NE 1 THEN ord = 1
     IF n_elements(flt) NE 1 THEN flt = 1 
     IF n_elements(plt) NE 1 THEN plt = 0

; LOAD FILE
     getim, infile, spect, paths[0], hdr=hdr
     IF n_elements(spect) EQ 1 THEN RETURN

; SCALE SHIFTING BASED ON ORDER NUMBER
     IF n_elements(row) EQ 1 THEN rw = row
     IF n_elements(cols) EQ 2 THEN cl = cols
     IF n_elements(dv) EQ 1 THEN dvtemp = dv
     CASE ord OF 

          2 : BEGIN
               dv = 5.2
               row = 600
               cols = [25, 990]
          END

          3 : BEGIN 
               dv = 4.9
               row = 435 
               cols = [25, 990]
          END

          4 : BEGIN 
               dv = 4.4 
               row = 266
               cols = [25, 990]
          END 

          5 : BEGIN 
               dv = 3.6 
               row = 78 
               cols = [25, 990] 
          END 

          6 : BEGIN 
               dv = 3.4
               row = 60 
               cols = [440, 840]
          END 

          ELSE : BEGIN 
               dv = 6.4
               row = 780 
               cols = [10, 700]
          END 

     ENDCASE
     IF n_elements(rw) EQ 1 THEN row = rw
     IF n_elements(cl) EQ 2 THEN cols = cl
     IF n_elements(dvtemp) EQ 1 THEN dv = dvtemp

; INITIALIZE SUBIMAGE VALUES
     s = size(spect, /dimensions)
     s[0] = (cols[1]-cols[0]+1) 
     spect = spect[cols[0]:cols[1], *]
     IF (wid/2) EQ (wid/2.0) THEN wid = wid + 1 
     xspec = fltarr(s[0], wid)
     tp = (row +fix(2*wid)) < (s[1]-1)
     bt = row 
     ll = fltarr(2, s[0])

; EXTRACT ORDER SUBIMAGE
     FOR i = 0, (s[0]-1) DO BEGIN
          ll[0,i] = (bt+i/dv) <(s[1]-wid-1)  
          ll[1,i] = (tp+i/dv) <(s[1]-1)
          line = transpose(spect[i, ll[0,i]:ll[1,i]])
          xspec[i, *] = line[0:(wid-1)]
     ENDFOR

; REMOVE NON-ORDER PIXELS
     ctr = fltarr(s[0])
     ctr1 = ctr
     sf = size(flt)
     sfn = n_elements(sf)

; Non-flats
     IF sf[sfn-2] EQ 7 THEN BEGIN
          chk = filex(flt, paths[1])
          IF chk EQ 0 THEN BEGIN
               msg = flt + ' DOES NOT EXIST'
               print, msg
               RETURN
          ENDIF
          wid = fix(wid/1.45)
          IF wid/2 EQ wid/2.0 THEN wid = wid + 1 
          getim, flt, fspect, paths[1]
          shft = fspect[*, wid]
          FOR k = 0, (s[0]-1) DO BEGIN 
               line3 = transpose(xspec[k, *])
               line3 = shift(line3, shft[k])
               xspec[k, *] = line3
          ENDFOR
          xospec = fltarr(s[0], wid)
          xospec[*,0:(wid-1)] = xspec[*,0:(wid-1)]
          lng = s[0]

; Flats
     ENDIF ELSE BEGIN
          FOR j = 0, (s[0]-1) DO BEGIN
               line2 = transpose(xspec[j, *])
               mx = max(line2, min=mn)
               lm = mn + (mx-mn)*0.5
               w = where(line2 GT lm)
               ctr[j] = (max(w) + min(w))/2.0
          ENDFOR
          xx = uindgen(s[0])
          junk1 = poly_fit(xx, ctr, 4, yfit=ctr1)
          FOR k = 0, (s[0]-1) DO BEGIN 
               line3 = transpose(xspec[k, *])
               line3 = shift(line3, (ctr1[0]-ctr1[k]))
               xspec[k, *] = line3
          ENDFOR
          wid = fix(wid/1.45)
          IF wid/2 EQ wid/2.0 THEN wid = wid + 1 
          xospec = fltarr(s[0], (wid+1))
          xospec[*,0:(wid-1)] = xspec[*,0:(wid-1)]
          xospec[*,wid] = ctr1[0] - ctr1
          lng = s[0]+1
     ENDELSE


; MODIFY HEADER
     fxaddpar, hdr, 'NAXIS1', lng
     fxaddpar, hdr, 'NAXIS2', wid
     sord = strtrim(string(ord), 2)
     com = '= Order # ' + sord + ' extracted from ' + infile
     fxaddpar, hdr, 'COMMENT', com

; DISPLAY RESULTS
     IF plt EQ 1 THEN BEGIN 
          window, xsize=s[0], ysize=wid
          tvscl, xspec
     ENDIF 

; SAVE RESULT
     cd, paths[1], current=nowp
     fits_write, outfile, xospec, hdr
     fits_close, fcb
     cd, nowp

END
