; NAME: 
;      GETIM
; 
; VERSION: 1
; 
; PURPOSE: 
;      Gets image from FITS file
; 
; CALLING SEQUENCE: 
;      getim, fname, image, [path, extend=extend, layer=layer, hdr=hdr, 
;             nos=nos, help=help]
; 
; INPUTS: 
;      fname- Name of FITS file
; 
; OPTIONAL INPUTS: 
;      path- Location of file (home directory is default)
; 
; OPTIONAL KEYWORD INPUTS: 
;      extend- FITS extension of desired image
;      layer- 3rd dimension for image cube
;      nos- Set to nonzero value to not scale images by BSCALE keyword
;      help- Set to nonzero scalar to print syntax
; 
; OUTPUT: 
;      image- FITS image data (-1 returned if file doesnt exist)
;   
; OPTIONAL KEYWORD OUTPUT: 
;      hdr- PDU header
; 
; EXTERNAL CALLS: 
;      FILEX, FITS_CLOSE, FITS_OPEN, FITS_READ
; 
; COMMON BLOCKS: 
;      NONE 
; 
; MODIFICATION HISTORY:
;      - Written by Eric Volquardsen, NASA IRTF, Jan. 2003
; 
; ==================================================================
PRO getim, fname, image, path, extend=extend, layer=layer, hdr=hdr, $
           nos=nos, help=help

; SYSTEM VARIABLES
     v = !version.os_family
     CASE v OF
          'vms' : root = '[]'
          'windows' : root = 'c:\'
          ELSE : root = '/'
     ENDCASE

; CHECK INPUTS
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN help = 1
     ENDIF ELSE help = 0 
     IF n_elements(fname) NE 1 THEN help = 1 
     IF help EQ 1 THEN BEGIN 
          print, '***********************************************'
          print, '***              GETIM SYNTAX               ***' 
          print, '*** > getim, fname, image, path, extend=ex  ***'
          print, '***         layer=layer, hdr=hdr            ***' 
          print, '***                                         ***' 
          print, '*** fname- Name of FITS file                ***' 
          print, '*** image- Name of obtained image           ***' 
          print, '*** path- Location of file                  ***' 
          print, '***       (current directory is default)    ***' 
          print, '*** extend- FITS extension of desired image ***' 
          print, '*** layer- 3rd dimension for image cube     ***'
          print, '*** hdr- PDU header output (optional)       ***' 
          print, '***********************************************'
          image = -1
          RETURN 
     ENDIF 
     IF n_elements(path) NE 1 THEN BEGIN
          cd, root, current=path
          cd, path
     ENDIF
     IF n_elements(extend) EQ 0 THEN extend = 0 
     IF n_elements(layer) EQ 0 THEN layer = (-1) 
     fchk = filex(fname, path)
     IF fchk EQ 0 THEN BEGIN
          image = -1
          print, strtrim(fname, 2), ' DOES NOT EXIST'
          RETURN
     ENDIF
     IF n_elements(nos) EQ 1 THEN BEGIN
          IF nos NE 0 THEN nos = 1 
     ENDIF ELSE nos = 0 

; OPEN FILE 
     cd, path, current=nowp
     fits_open, fname, fcb

; CHECK FOR DESIRED IMAGE
     IF extend GT fcb.nextend THEN BEGIN
          print,  fname
          print, '****************************************'
          print, '***              GETIM               ***'
          print, '*** Desired extension does not exist ***'
          print, '****************************************'
          image = -1
          fits_close, fcb
          RETURN          
     ENDIF
     lyrchk = fcb.axis
     slc = size(lyrchk, /dimensions)
     IF layer NE (-1) THEN BEGIN
          IF lyrchk[2, extend] LT layer THEN BEGIN
               print,  fname
               print, '*************************************'
               print, '***            GETIM              ***'
               print, '**** Desired image does not exist ***'
               print, '*************************************'
               image = -1
               fits_close, fcb
               RETURN
          ENDIF
     ENDIF

; READ IMAGE
     fits_read, fcb, image, hdr, exten_no=extend, noscale=nos, /no_pdu
     fits_close, fcb
     cd, nowp

END
