; NAME: 
;      GETPOS
; 
; PURPOSE:
;      Finds centroid positions of the two positive image elements produced
;      by coadding four exposures from nodding and chopping of telescope.  
; 
; PROCEDURE: 
;      Two methods are used to find peaks in each image quadrant.  One 
;      looks at the slope of pixel values and the other fits a gaussian
;      to the data.  The two quadrants with the greatest pixel values at their 
;      peaks are selected.  Designed to be called by NASA IRTF mid-IR image 
;      registration programs.
; 
; CALLING SEQUENCE:
;      getpos, image, zcx, zcy, [gs1, gs2, help=help]
; 
; INPUTS: 
;      image- Name of image for which centroids will be found
;             (Minimum image size of 100 x 100 pixels)
; 
; OPTIONAL INPUTS: 
;      gs1- Two element vector with x and y coordinates of estimated 
;           position of first positive image element
;      gs2- Two element vector with x and y coordinates of estimated 
;           position of second positive image element
; 
; OPTIONAL KEYWORD INPUTS: 
;      help- If set to nonzero scalar then GETPOS prints syntax help 
;            and returns.
; 
; OUTPUTS: 
;      zcx- x centroid values for two positive subimages rounded to hundredths
;      zcy- y centroid values for two positive subimages rounded to hundredths
; 
; EXTERNAL CALLS: 
;      IM_STATS, IRCNTRD, ROUND_OFF
; 
; COMMON BLOCKS: 
;      NONE
; 
; MODIFICATION HISTORY:
;     -Written by Eric Volquardsen, NASA IRTF, Nov. 2002 
; 
; =========================================================================
PRO getpos, image, zcx, zcy, gs1, gs2, help=help

; CHECK INPUTS
  IF n_elements(image) LT 10000 THEN help = 1 
  IF n_elements(help) EQ 1 THEN BEGIN
       IF help NE 0 THEN BEGIN
            print, '*****************************************************'
            print, '***                  GETPOS SYNTAX                ***'
            print, '*** > getpos, image, zcx, zcy, gs1, gs2           ***'
            print, '***                                               ***' 
            print, '*** image- Name of image for which centroids will ***' 
            print, '***        be found (100 x 100 pixel minimum).    ***'
            print, '*** zcx- X-coordinates calculated by GETPOS for   ***' 
            print, '***      two positive images.                     ***' 
            print, '*** zcy- Y-coordinates calculated by GETPOS for   ***' 
            print, '***      two positive images.                     ***' 
            print, '*** gs1- Two element vector with x and y coords   ***' 
            print, '***      of estimated position of first positive  ***' 
            print, '***      image element (Optional) .               ***' 
            print, '*** gs2- Two element vector with x and y coords   ***' 
            print, '***      of estimated position of second positive ***' 
            print, '***      image element (Optional).                ***' 
            print, '*****************************************************'
            RETURN
       ENDIF
  ENDIF
  sz = size(image, /dimensions)
  ngs = 0 
  IF n_elements(gs1) NE 2 THEN BEGIN 
       gs1 = [fix(sz[0]/4), fix(sz[1]/3)]
       ngs = 1 
  ENDIF
  IF n_elements(gs2) NE 2 THEN BEGIN 
       gs2 = [fix(sz[0]*.66), fix(sz[1]*.66)]
       ngs = 1 
  ENDIF

; GET QUADRANT CENTROIDS
; Get various sizes
  halfx = round(0.5 * sz[0])
  halfy = round(0.5 * sz[1])

; Cut image into quadrants
  q1 = image[0:(halfx-1), 0:(halfy-1)]
  q2 = image[0:(halfx-1), halfy:(sz[1]-1)]
  q3 = image[halfx:(sz[0]-1), halfy:(sz[1]-1)]
  q4 = image[halfx:(sz[0]-1), 0:(halfy-1)] 

; FIND PEAKS IN QUADRANTS
; get structure from im_stats
  inf1 = im_stats(q1, /get_fwhm)
  inf2 = im_stats(q2, /get_fwhm)
  inf3 = im_stats(q3, /get_fwhm)
  inf4 = im_stats(q4, /get_fwhm)

; Return if im_stat cant get values
  infchk = 0 
  IF n_elements(inf1) NE 1 THEN infchk = 1 
  IF n_elements(inf2) NE 1 THEN infchk = 1 
  IF n_elements(inf3) NE 1 THEN infchk = 1 
  IF n_elements(inf4) NE 1 THEN infchk = 1 
  IF infchk EQ 1 THEN BEGIN
       zcx = [gs1[0], gs2[0]]
       zcy = [gs1[1], gs2[1]]   
  ENDIF ELSE BEGIN

; get integer pixel centroids
       centx1 = inf1.peak_xy[0]>10
       centy1 = inf1.peak_xy[1]>10
       centx2 = inf2.peak_xy[0]>10
       centy2 = inf2.peak_xy[1]>10
       centx3 = inf3.peak_xy[0]>10
       centy3 = inf3.peak_xy[1]>10
       centx4 = inf4.peak_xy[0]>10
       centy4 = inf4.peak_xy[1]>10

; Get Pixel Values at Centroids       
       v1 = inf1.max
       v2 = inf2.max
       v3 = inf3.max
       v4 = inf4.max

; DETERMINE POSITIVE QUADRANT CENTROID
; adjust coordinates  
       v = [v1, v2, v3, v4]
       vx = [centx1, centx2, (centx3+halfx), (centx4+halfx)]
       vy = [centy1, (centy2+halfy), (centy3+halfy), centy4]

; get fwhm values
       fw = [inf1.fwhm, inf2.fwhm, inf3.fwhm, inf4.fwhm]

; sort by pixel value
       vo = reverse(sort(v))
       zcx = [vx[vo[0]], vx[vo[1]]]
       zcy = [vy[vo[0]], vy[vo[1]]]
       fwh = [0.0, 0.0]
       fwh[0] = fw[vo[0]]>0.5 
       fwh[1] = fw[vo[1]]>0.5
       fwh = max(fwh)
  ENDELSE

; GET MORE ACCURATE CENTROID
; get decimal centroid value
  zcxx = [gs1[0], gs2[0]]
  zcyy = [gs1[1], gs2[1]]
  ircntrd, image, zcx, zcy, acx, acy, fwh
  ircntrd, image, zcxx, zcyy, bcx, bcy, fwh

; Check new centroid values
  kp = 0
  IF bcx[0] LE 0 THEN ngs = 1 
  IF bcx[1] LE 0 THEN ngs = 1 
  IF bcy[0] LE 0 THEN ngs = 1 
  IF bcy[1] LE 0 THEN ngs = 1 
  IF ngs EQ 0 THEN BEGIN
       zcx = bcx
       zcy = bcy 
  ENDIF ELSE BEGIN
       IF acx[0] LE 0 THEN kp = 1
       IF acy[0] LE 0 THEN kp = 1
       IF acx[1] LE 0 THEN kp = 1
       IF acy[1] LE 0 THEN kp = 1
       IF kp NE 1 THEN BEGIN
            zcx = acx
            zcy = acy
       ENDIF
  ENDELSE

; Round answers to two decimals
  IF n_elements(zcx) EQ 2 THEN BEGIN
       zcx[0] = round_off(zcx[0], 0.01)
       zcx[1] = round_off(zcx[1], 0.01)
       zcy[0] = round_off(zcy[0], 0.01)
       zcy[1] = round_off(zcy[1], 0.01)
  ENDIF

END  


