; NAME: 
;      IM_FLAT
; 
; VERSION: 1
; 
; PURPOSE: 
;      Creates a flat from a group of images by taking the median
;      or mean value of each pixel.  This method assumes that an object 
;      is not at the same location in each image.  A min-max 
;      rejection is applied and then background is normalized to 1.
;      A floor value of 0.5 is then set in the flat.
;      
; 
; CALLING SEQUENCE: 
;      im_flat, infiles, outfile, paths, [methd=methd, mask=mask, 
;               help=help]
; 
; INPUTS: 
;      infiles - String vector of filenames 
;      outfile - Filename where flat should be saved
;      paths - Three element string vector of infile, outfile, and 
;              mask paths
; 
; OPTIONAL KEYWORD INPUTS: 
;      methd - Set for desired pixel averaging method
;                   0 : median (default)
;                   1 : mean
;      mask - Set to [box size, st. dev. limit] to calculate bad 
;             pixels from flat.  Set to -1 to not remove bad pixels.
;             Default is to use [3,3].
;      help - Set to nonzero scalar to print syntax
; 
; OUTPUT: 
;      Flat is created and saved at outfile with header similar to 
;      first infile
; 
; EXTERNAL CALLS: 
;      ACRE, FXADDPAR, FXDELPAR, FXWRITE, GETIM, MEDARR
; 
; COMMON BLOCKS: 
;      NONE
; 
; MODIFICATION HISTORY: 
;      - Written by Eric Volquardsen, NASA IRTF, January 2004
; 
; =======================================================================
PRO im_flat, infiles, outfile, paths, methd = methd, mask = mask, $ 
             help = help

; CHECK INPUTS
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN help = 1 
     ENDIF ELSE help = 0 
     nin = n_elements(infiles)
     IF nin LT 3 THEN IF nin NE 1 THEN help = 1
     IF n_elements(outfile) NE 1 THEN help = 1
     IF help EQ 1 THEN BEGIN
          print, '*****************************************************'
          print, '***              IM_FLAT  SYNTAX                  ***'  
          print, '*** IDL> im_flat, infiles, outfile, paths,        ***' 
          print, '***              methd=methd, mask=mask           ***' 
          print, '***                                               ***'
          print, '*** infiles - String vector of filenames          ***'
          print, '*** outfile - Filename where flat should be saved ***'
          print, '*** paths - Three element string vector of infile ***' 
          print, '***         outfile and mask paths                ***'
          print, '*** methd - Method used for averaging pixels      ***'
          print, '***         (0=median, 1=mean)                    ***'
          print, '*** mask - Set to [box size, st. dev. limit] to   ***'
          print, '***        calculate bad pixels from flat.        ***'
          print, '*****************************************************'   
          RETURN
     ENDIF
     np = n_elements(paths)
     CASE np OF 
          0 : BEGIN
               v = !version.os_family
               CASE v OF
                    'vms' : root = '[]'
                    'windows' : root = 'c:\'
                    ELSE : root = '/'
                ENDCASE
                cd, root, current=pat
                cd, pat
                paths = [pat, pat, path]
          END
          1 : paths = [paths, paths, paths]
          2 : paths = [paths[0], paths[1], paths[0]]
          ELSE : paths = paths[0:2]
     ENDCASE
     IF n_elements(methd) NE 1 THEN methd = 0
     n_msk = n_elements(mask)
     IF n_msk NE 2 THEN BEGIN 
          IF n_msk NE 1 THEN mask = 1
          IF mask[0] NE -1 THEN BEGIN
               n_msk = 2
               mask = [3, 3]
          ENDIF ELSE mask = -1
     ENDIF ELSE IF mask[0] LT 3 THEN mask[0] = 3
     note = 'FLAT'

; LOAD FILES
     names =  ''
     n_msk = n_elements(mask)
     FOR i = 0, (nin-1) DO BEGIN
          IF i EQ 0 THEN BEGIN 
               GETIM, infiles[i], im, paths[0], hdr = h
               sz = size(im, /dimensions)
               nsz = n_elements(sz) 
               CASE nsz OF 
                    2 : 
                    3 : BEGIN 
                         im = im[*, *, 0]
                         FXDELPAR, h, 'NAXIS3'
                    END 
                    ELSE : BEGIN 
                         print, infiles[i]
                         print, 'INCORRECT FILE SIZE'
                         RETURN
                    END 
               ENDCASE 
               ims =  fltarr(sz[0], sz[1], nin)
          ENDIF ELSE GETIM, infiles[i], im, paths[0]

; CREATE IMAGE CUBE 
          ims[*, *, i] = im
          names = names + ' ' + infiles[i]
     ENDFOR 

; CREATE FLAT 
     IF nin NE 1 THEN BEGIN 
          IF methd EQ 1 THEN BEGIN 
               flat = total(ims, 3)/nin
          ENDIF ELSE MEDARR, ims, flat 
     ENDIF ELSE flat = ims

; CALCULATE BAD PIXEL MASK (if desired)
     IF n_msk EQ 2 THEN BEGIN 
          note = 'FLAT WITH BAD PIXELS REMOVED'
          ACRE, flat, flat, mask[1], mask[0]
     ENDIF 

; NORMALIZE FLAT
     medf = median(flat)
     flat = (flat / medf) > 0.5
     medf = median(flat)
     flat = (flat / medf)

; PRINTED OUTPUT 
     print, '  '
     print, note
     print, mask
     print, max(flat), min(flat)
     print, '  '

; ALTER HEADER
     FXADDPAR, h, 'BITPIX', -32
     FXADDPAR, h, 'ORIGINAL', names
     FXADDPAR, h, 'OBJECT', note

; SAVE RESULTS
     cd, paths[1], current=nowp
     FXWRITE, outfile, h, flat
     cd, nowp

END
