; NAME: 
;      IMAGE_MATH
; 
; VERSION: 1
; 
; PURPOSE: 
;      Can add, subtract, multiply, divide, average, and matrix 
;      mulitply two images.  Results can be scaled to a desired
;      value.
; 
; CALLING SEQUENCE: 
;      image_math, im1, im2, outim, op, paths, [scale=scale, help=help]
; 
; INPUTS: 
;      im1 - Filename of first image or a quantity 
;      im2 - Filename of second image or a quantity
;      imout - Filename of resulting image
;      op - Operation to be performed: 
;                0 : Add
;                1 : Subtract
;                2 : Multiply
;                3 : Divide
;                4 : Average
;                5 : Matrix multiplication
;      paths - String vector with paths to im1, im2, and imout
; 
; OPTIONAL KEYWORD INPUTS: 
;      scale - Background image value desired (0 for no scaling)
;      help - Set to nonzero scalar to print syntax
; 
; OUTPUT: 
;      Results are saved to imout
; 
; EXTERNAL CALLS: 
;      FXADDPAR, FXHMAKE, FXWRITE, GETIM
; 
; COMMON BLOCKS: 
;      NONE
; 
; MODIFICATION HISTORY: 
;      - Written by Eric Volquardsen, NASA IRTF, March 2004
; 
; =======================================================================
PRO image_math, im1, im2, outim, op, paths, scale = scale, help=help

; CHECK INPUTS
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN help = 1 
     ENDIF ELSE help = 0 
     n = n_params()
     IF n LT 3 THEN help = 1 
     IF help EQ 1 THEN BEGIN
          print, '***************************************************'
          print, '***             IMAGE_MATH SYNTAX               ***'  
          print, '*** IDL> image_math, im1, im2, outim, op,       ***'
          print, '***                  paths, scale=scale         ***' 
          print, '***                                             ***' 
          print, '*** im1 - Filename of first image               ***'
          print, '*** im2 - Filename of second image              ***'
          print, '*** imout - Filename of resulting image         ***' 
          print, '*** op - Operation to be performed              ***'
          print, '***         0 : Add                             ***'
          print, '***         1 : Subtract                        ***'
          print, '***         2 : Multiply                        ***'
          print, '***         3 : Divide                          ***'
          print, '***         4 : Average                         ***'
          print, '***         5 : Matrix multiplication           ***'
          print, '*** paths - String vector with paths            ***'
          print, '*** scale - Set to desired background value for ***'
          print, '***         results (default is no scaling)     ***'
          print, '***************************************************'   
          RETURN
     ENDIF
     np = n_elements(paths)
     CASE np OF 
          0 : BEGIN
               v = !version.os_family
               CASE v OF
                    'vms' : root = '[]'
                    'windows' : root = 'c:\'
                    ELSE : root = '/'
                ENDCASE
                cd, root, current=pat
                cd, pat
                paths = [pat, pat, pat]
          END
          1 : paths = [paths, paths, paths]
          2 : paths = [paths[0], paths[0], paths[1]]
          ELSE : paths = paths[0:2]
     ENDCASE
     outim = outim[0]
     IF n_elements(op) NE 1 THEN op = 0
     IF n_elements(scale) NE 1 THEN scale = 0

; READ IMAGE FILES OR SET CONSTANT VALUES
     qt = 0
     s1 = size(im1)
     s2 = size(im2)
     IF n_elements(s1) EQ 3 THEN BEGIN 
          IF s1[1] EQ 7 THEN BEGIN 
               GETIM, im1, img1, paths[0], hdr = h
               sz1 = size(img1, /dimension)
               IF n_elements(sz1) NE 2 THEN qt = 1 
          ENDIF ELSE img1 = im1
     ENDIF ELSE qt = 1
     IF n_elements(s2) EQ 3 THEN BEGIN 
          IF s2[1] EQ 7 THEN BEGIN 
               GETIM, im2, img2, paths[1], hdr = h1
               IF n_elements(h) EQ 0 THEN h = h1
               sz2 = size(img2, /dimension)
               IF n_elements(sz2) NE 2 THEN qt = 1
               IF n_elements(sz1) EQ 2 THEN BEGIN
                    IF sz1[0] NE sz2[0] THEN qt = 1
               ENDIF 
          ENDIF ELSE img2 = im2
     ENDIF ELSE qt = 1
     
; Check image existence and size
     IF qt EQ 1 THEN BEGIN 
          IF op NE 5 THEN BEGIN 
               print, '  '
               print, 'IMAGES NOT OF EQUAL SIZES'
               print, '  '
               RETURN 
          ENDIF 
     ENDIF 

; PERFORM OPERATION
     CASE op OF 
          1 : BEGIN 
               img = img1 - img2
               commnt = strtrim(im1, 2) + ' - ' + strtrim(im2, 2)
          END 
          2 : BEGIN 
               img = img1 * img2
               commnt = strtrim(im1, 2) + ' * ' + strtrim(im2, 2)
          END
          3 : BEGIN 
               img = img1 / img2 
               commnt = strtrim(im1, 2) + ' / ' + strtrim(im2, 2)
          END 
          4 : BEGIN 
               img = (img1 + img2) / 2.0
               commnt = 'Average of ' + strtrim(im1, 2) + strtrim(im2, 2)
          END 
          5 : BEGIN 
               img = img1 # img2
               commnt = strtrim(im1, 2) + ' # ' + strtrim(im2, 2)
          END 
          ELSE : BEGIN 
               img = img1 + img2
               commnt = strtrim(im1, 2) + ' + ' + strtrim(im2, 2)
          END 
     ENDCASE 

; SCALING (if desired)
     img = float(img)
     IF scale NE 0 THEN BEGIN 
          commnt = commnt + ' (Scaled)'
          immed = median(img)
          IF immed GT 0 THEN img = img*scale/immed $ 
               ELSE img = (img - immed + 1)*scale
     ENDIF 

; CREATE HEADER
     IF n_elements(h) EQ 0 THEN FXHMAKE, h
     FXADDPAR, h, 'OBJECT', commnt
     FXADDPAR, h, 'BITPIX', -32

; SAVE RESULTS
     cd, paths[2], current=nowp
     FXWRITE, outim, h, img
     cd, nowp

END
