; NAME: 
;      LN_TO_WAVE
; 
; VERSION: 1
; 
; PURPOSE: 
;      Converts line number to wavelength in spectrum file given the
;      input file and the polynomial coefficients.
; 
; CALLING SEQUENCE: 
;      ln_to_wave, infile, coeffs, [paths=paths, outfile=outfile, help=help]
; 
; INPUTS: 
;      INFILE - Filename of spectrum- should be 3 column ascii file with
;               line numbers in 1st column, reflectance in 2nd column, 
;               and reflectance error in 3rd column.
;      COEFFS - Vector of polynomial coefficients to use in converting 
;               line numbers to wavelengths.
;      
; OPTIONAL KEYWORD INPUTS: 
;      PATHS - Input and output file paths (default is current)
;      HELP - Set to nonzero scalar to print syntax
; 
; OUTPUT: 
;      OUTFILE - Filename of output file (default is infile)
; 
; EXTERNAL CALLS: 
;      READ_SPEC
; 
; COMMON BLOCKS: 
;      NONE 
; 
; MODIFICATION HISTORY: 
;      - Written by Eric Volquardsen, NASA IRTF, April 2003
; 
; ========================================================================
PRO ln_to_wave, infile, coeffs, paths=paths, outfile=outfile, help=help

; CHECK INPUTS
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN help = 1 
     ENDIF ELSE help = 0 
     IF n_elements(infile) LT 1 THEN help = 1 
     IF n_elements(coeffs) LT 2 THEN help = 1 
     IF help EQ 1 THEN BEGIN
          print, '**************************************************'
          print, '***               LN_TO_WAVE SYNTAX            ***' 
          print, '*** > ln_to_wave, infile, coeffs, paths=paths, ***' 
          print, '***               outfile=outfile              ***' 
          print, '***                                            ***' 
          print, '*** INFILE - Filename of spectrum- should be 3 ***' 
          print, '***          column ascii file with line #s in ***' 
          print, '***          1st column, reflectance in 2nd    ***'  
          print, '***          and reflectance error in 3rd col. ***' 
          print, '*** COEFFS - Vector of polynomial coefficients ***' 
          print, '***          to use in converting line numbers ***' 
          print, '***          to wavelengths.                   ***' 
          print, '*** PATHS - Input and output file paths        ***' 
          print, '***         (default is current)               ***' 
          print, '*** OUTFILE - Filename of output file          ***' 
          print, '***           (default is infile)              ***' 
          print, '**************************************************' 
          RETURN
     ENDIF
     nep = n_elements(paths)
     CASE nep OF
          1 : paths = [paths, paths]
          2 : 
          else : BEGIN
               v = !version.os_family
               CASE v OF
                    'vms' : root = '[]'
                    'windows' : root = 'c:\'
                    ELSE : root = '/'
               ENDCASE
               cd, root, current=nowp
               paths = [nowp, nowp]
               cd, nowp
          END
     ENDCASE
     n = n_elements(infile) 
     IF n_elements(outfile) NE n THEN outfile = infile

; DETERMINE POLYNOMIAL SIZE
     np = n_elements(coeffs)

; LOOP THROUGH FILES
     FOR i = 0, (n-1) DO BEGIN

          data = READ_SPEC(infile[i], paths[0])

; Calculate wavelength
           wavel = fltarr(n_elements(data[0, *]))
           FOR j = 0, (np-1) DO wavel = wavel + coeffs[j]*(data[0, *]^j)
           data[0, *] = wavel
      
; Save output file
           cd, paths[1], current=nowp
           openw, logun, outfile[i], /get_lun
           printf, logun, data
           free_lun, logun
           cd, nowp

     ENDFOR

END 
