; NAME: 
;      MAKE_FLAT
; 
; VERSION: 1
; 
; PURPOSE: 
;      Combines flats by median of each pixel value, removes 'bad' pixels, 
;      and can be set to normalize by line or surface.  Resulting image 
;      and bad pixel mask are saved.
; 
; CALLING SEQUENCE: 
;      make_flat, fnames, [flatnm, boxw=boxw, nsig=nsig, nor=nor,  
;                 pa=pa, itr=itr, help=help]
; 
; INPUTS: 
;      FNAMES - Vector of input filenames of flats
; 
; OPTIONAL KEYWORD INPUTS:
;      BOXW - Dimension of moving box used to find bad pixels 
;             (default is 3).
;      NSIG - Standard deviation limit indicating bad pixel
;             (default is 2)
;      NOR - Set to value for desired normalization and fit method: 
;                 default : linear
;                       1 : surface
;                      -1 : none
;      PA - One or two element vector giving paths where fnames and 
;           flatnm should be located (default is current directory)
;      ITR - Number of times to run bad pixel removal (default is 2) 
;      HELP - Set to nonzero scalar to have make_flat print syntax
; 
; OUTPUT: 
;      FLATNM - File name of returned flat (default name is superflat.fits)
; 
; EXTERNAL CALLS: 
;      ACRE, BOOST_ARRAY, GETIM, SDEV
; 
; COMMON BLOCKS: 
;      NONE 
; 
; MODIFICATION HISTORY: 
;      - Written by Eric Volquardsen, NASA IRTF, April 2003
; 
; =========================================================================
PRO make_flat, fnames, flatnm, boxw=boxw, nsig=nsig, nor=nor, pa=pa, $ 
               itr=itr, help=help

; CHECK INPUTS
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN help = 1 
     ENDIF ELSE help = 0 
     nf = n_elements(fnames)
     IF nf LT 2 THEN help = 1 ELSE BEGIN
          IF fnames[0] EQ '' THEN help = 1 
          IF fnames[1] EQ '' THEN help = 1 
          nsif = n_elements(size(fnames, /dimensions))
          IF nsif GT 1 THEN inames = reform(fnames, nf)
     ENDELSE
     IF help EQ 1 THEN BEGIN
          print, '*********************************************************'
          print, '***                  MAKE_FLAT SYNTAX                 ***'
          print, '*** > make_flat, fnames, flatnm, boxw=boxw, nsig=nsig,***'
          print, '***              nor=nor, pa=pa, itr=itr              ***'
          print, '***                                                   ***'
          print, '*** FNAMES - Vector of input filenames of flats       ***'
          print, '*** FLATNM - Returned flat(default is superflat.fits) ***'
          print, '*** BOXW - Side length of moving box used to find     ***'
          print, '***        bad pixels (default is 7)                  ***' 
          print, '*** NSIG - Standard deviation limit indicating bad    ***'
          print, '***        pixel (default is 3)                       ***'
          print, '*** NOR - Set value for desired normalization and fit ***' 
          print, '***            default : linear                       ***' 
          print, '***                  1 : surface                      ***' 
          print, '***                 -1 : none                         ***'
          print, '*** PA - Vector of path(s) to fnames and flatnm       ***' 
          print, '***      (default is current directory)               ***'
          print, '*** ITR - Number of times to run bad pixel removal    ***'
          print, '***       (default is twice)                          ***'
          print, '*********************************************************'
          RETURN
     ENDIF
     IF n_elements(boxw) NE 1 THEN boxw = 7
     IF n_elements(nsig) NE 1 THEN nsig = 3
     IF n_elements(flatnm) NE 1 THEN flatnm = 'superflat.fits'
     IF n_elements(nor) EQ 1 THEN BEGIN
          IF nor LT 0 THEN nor = -1
          IF nor GT 0 THEN nor = 1 
     ENDIF ELSE nor = 0
     nep = n_elements(pa)
     CASE nep OF
          1 : pa = [pa,pa]
          2 : 
          else : BEGIN
               v = !version.os_family
               CASE v OF
                    'vms' : root = '[]'
                    'windows' : root = 'c:\'
                    ELSE : root = '/'
               ENDCASE
               cd, root, current=nowp
               pa = [nowp, nowp]
               cd, nowp
          END
     ENDCASE
     IF n_elements(itr) NE 1 THEN itr = 2
     itr = abs(itr)

; CREATE IMAGE CUBE
     FOR i = 0, (nf-1) DO BEGIN
          getim, fnames[i], im, pa[0], hdr=h
          IF n_elements(im) EQ 1 THEN RETURN
          sim = size(im, /dimensions)
          IF sim[1]/2 EQ sim[1]/2.0 THEN im = im[*,0:(sim[1]-2)]
          IF i EQ 0 THEN image = im ELSE boost_array, image, im
          IF i LT (nf-1) THEN BEGIN
               IF fnames[(i+1)] EQ '' THEN i = nf
          ENDIF
     ENDFOR

; COMBINE FLATS BY MEDIAN OF EACH PIXEL
     medarr, image, sflat
     sflat = float(sflat)

; REMOVE BAD PIXELS
     IF itr NE 0 THEN BEGIN
          FOR j = 1, itr DO BEGIN
               ACRE, sflat, sflat, nsig, boxw, mask = mask1
               IF j NE 1 THEN mask = mask + mask1 ELSE mask = mask1
               mask = mask < 1
          ENDFOR   
     ENDIF

; NORMALIZE
     CASE nor OF

          1 : BEGIN
               sflat = sfit(sflat, 2)
               mnn = min(sflat)
               sflat = sflat - mnn + 1
               mdn = median(sflat, /even)
               sflat = sflat / mdn
               com6 = '   Surface Normalization and fit'
          END   ; surface normalization

          -1 :  com6 = '   Not Normalized or fit'

          ELSE : BEGIN
               sz = size(sflat, /dimensions)
               ss = sz[1]
               sz = sz[0]
               x = indgen(ss)
               line = fltarr(ss)
               FOR ll = 0, (sz-1) DO BEGIN
                    line[*] = sflat[ll,*]
                    reslt = poly_fit(x, line, 2, yfit=line)
                    mdn = median(line, /even)
                    sflat[ll,*] = line[*]/mdn
               ENDFOR
               com6 = '  Line Normalization and fit'
          END   ; line normalization

     ENDCASE   ; normalization

; SAVE FLAT AND MASK
     fxaddpar, h, 'BITPIX', -32
     fxaddpar, h, 'IRAFNAME', flatnm
     fxaddpar, h, 'EXTEND', 'T'
     com1 = strtrim(string(boxw[0]), 2) + 'x'
     com2 = strtrim(string(boxw[0]), 2) + ' box /'
     com3 = strtrim(string(nsig), 2) + ' sig /'
     com4 = strtrim(string(itr), 2) + ' iter'
     com5 = com1 + com2 + com3 + com4
     fxaddpar, h, 'FIXPIX', com5
     fxaddpar, h, 'NORMED', com6
     fxaddpar, h, 'COMMENT', '  FLAT with bad pixel mask'
     cd, pa[1], current=nowp
     fits_open, flatnm, fcb, /write
     fits_write, flatnm, sflat, h
     fits_close, fcb
     fits_open, flatnm, fcb, /append
     fits_write, fcb, mask, extname='MASK', extver=1
     fits_close, fcb
     cd, nowp

END
