; NAME: 
;      MAKE_LOG
; 
; VERSION : 1
; 
; PURPOSE: 
;      Creates observing log from a list of FITS files.
; 
; CALLING SEQUENCE: 
;      make_log, infiles, outfile, path, [inst=inst, help=help]
; 
; INPUTS: 
;      infiles - Vector of FITS file names to check
;      path - Directory where files are located
; 
; OPTIONAL KEYWORD INPUTS: 
;      inst - Instrument used
;              0 for MIRSI (default)
;              1 for SpeX
;              2 for NSFCam
;              3 for Apogee Camera
;      help - Set to nonzero scalar to print syntax
; 
; OUTPUT: 
;      outfile - Filename where log should be saved, including:
;                     - FITS filenames
;                     - Object name
;                     - Exposure time
;                     - Observing mode
;                     - Coadds
;                     - Cycles
;                     - Airmass
;                     - Wavelength
;                     - Observation start time
;                     - Observation date, telescope, instrument and 
;                       observer name (above table)
; 
; EXTERNAL CALLS: 
;      FXPAR, GETIM
; 
; COMMON BLOCKS: 
;      NONE
; 
; MODIFICATION HISTORY:
;      - Written by Eric Volquardsen, NASA IRTF, July 2003
; 
; ====================================================================
PRO make_log, infiles, outfile, path, inst=inst, help=help

; CHECK INPUTS
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN help = 1 
     ENDIF ELSE help = 0 
     IF n_elements(infiles) LT 1 THEN help = 1 
     IF n_elements(path) LT 1 THEN help = 1 
     IF help EQ 1 THEN BEGIN
          print, '*******************************************'
          print, '***           MAKE_LOG SYNTAX           ***'
          print, '*** > make_log, infiles, outfile, path, ***' 
          print, '***             inst=inst               ***' 
          print, '***                                     ***' 
          print, '*** infiles - Vector of FITS file names ***' 
          print, '*** outfile - Filename where log should ***' 
          print, '***           be saved                  ***' 
          print, '*** path - Path to FITS files           ***' 
          print, '*** inst - Instrument used              ***' 
          print, '***          0 for MIRSI (default)      ***' 
          print, '***          1 for SpeX                 ***' 
          print, '***          2 for NSFCam               ***' 
          print, '***          3 for Apogee Camera        ***' 
          print, '*******************************************'
          RETURN
     ENDIF 
     IF n_elements(outfile) NE 1 THEN outfile = 'observation.log'
     IF n_elements(inst) NE 1 THEN inst = 0 
   
; DESIGNATE HEADER KEYWORDS USED BY INSTRUMENT
     CASE inst OF 

          1 : BEGIN
               lbl = 'Object   Grating    ITime   Coadds  '
               lbl = lbl + 'Cycles   Airmass   Filter   Start tm'
               ins_name = 'SpeX'
               kwd_obsrv = 'OBSERVER'
               kwd_date = 'DATE_OBS'
               kwd_tel = 'TELESCOP'
               kwd_obj = 'OBJECT'
               kwd_time = 'ITIME'
               kwd_mode = 'GRAT'
               kwd_coad = 'CO_ADDS'
               kwd_cycl = 'CYCLES'
               kwd_airm = 'AIRMASS'
               kwd_wl = 'GFLT'
               kwd_start = 'TIME_OBS'
          END

          2 : BEGIN
               lbl = 'Object   Mode    ITime    Coadds '
               lbl = lbl + 'Cycles   Airmass  Wavelength   Start tm'
               ins_name = 'NSFCam'
               kwd_obsrv = 'OBSERVER'
               kwd_date = 'DATE_OBS'
               kwd_tel = 'TELESCOP'
               kwd_obj = 'OBJECT'
               kwd_time = 'ITIME'
               kwd_mode = 'OBSMODE'
               kwd_coad = 'CO_ADDS'
               kwd_cycl = 'CYCLES'
               kwd_airm = 'AIRMASS'
               kwd_wl = 'CVFWLEN'
               kwd_start = 'TIME_OBS'
          END

          3 : BEGIN
               lbl = 'Object  Img type    Exposure   Gain '
               lbl = lbl + 'Airmass   Filter   Start tm'
               ins_name = 'Apogee'
               kwd_obsrv = 'OBSERVER'
               kwd_date = 'DATE-OBS'
               kwd_tel = 'TELESCOP'
               kwd_obj = 'OBJECT'
               kwd_time = 'EXPTIME'
               kwd_mode = 'IMGTYPE'
               kwd_coad = 'GAIN'
               kwd_cycl = ''
               kwd_airm = 'SECZ'
               kwd_wl = 'FILTER'
               kwd_start = 'TIME-OBS'
          END

          ELSE : BEGIN
               lbl = 'Object      Mode      Exposure   Coad  '
               lbl = lbl + 'Airmass  Wavelength  Start tm'
               ins_name = 'MIRSI'
               kwd_obsrv = 'OBSERVER'
               kwd_date = 'DATE-OBS'
               kwd_tel = 'TELESCOP'
               kwd_obj = 'OBJECT'
               kwd_time = 'EXPTIME'
               kwd_mode = 'OBS-MODE'
               kwd_coad = 'NCOADS'
               kwd_cycl = ''
               kwd_airm = 'AIRMASS'
               kwd_wl = 'LAMBDA'
               kwd_start = 'TIME-STR'
          END

     ENDCASE 

; LOOP THROUGH FILES
     n = n_elements(infiles)
     obj = strarr(n)
     tm = strarr(n)
     md = strarr(n)
     coad = strarr(n)
     airm = strarr(n)
     wl = strarr(n)
     stt = strarr(n)
     cycl = strarr(n)
     FOR i = 0, (n-1) DO BEGIN
          h = ''
          getim, infiles[i], im, path, hdr=h
          IF n_elements(h) GT 1 THEN BEGIN
               IF i EQ 0 THEN BEGIN
                    obsrvr = strtrim(fxpar(h, kwd_obsrv))
                    dt = strtrim(fxpar(h, kwd_date))
                    tel = strtrim(fxpar(h, kwd_tel))
               ENDIF 
               obj[i] = strtrim(fxpar(h, kwd_obj), 2)
               tm[i] = strtrim(fxpar(h, kwd_time), 2)
               md[i] = strtrim(fxpar(h, kwd_mode), 2)
               coad[i] = strtrim(fxpar(h, kwd_coad), 2)
               airm[i] = strtrim(fxpar(h, kwd_airm), 2)
               wl[i] = strtrim(fxpar(h, kwd_wl), 2)
               stt[i] = strtrim(fxpar(h, kwd_start), 2)
               IF kwd_cycl NE '' THEN $ 
                    cycl[i] = strtrim(fxpar(h, kwd_cycl), 2)
          ENDIF ELSE BEGIN
               IF i EQ 0 THEN RETURN
          ENDELSE 
     ENDFOR

; SAVE RESULTS
     IF kwd_cycl EQ '' THEN BEGIN
          tbl = strarr(8,n)
          tbl[0,*] = infiles
          tbl[1,*] = obj
          tbl[2,*] = md
          tbl[3,*] = tm
          tbl[4,*] = coad
          tbl[5,*] = airm
          tbl[6,*] = wl
          tbl[7,*] = stt
          nn = 7
     ENDIF ELSE BEGIN
          tbl = strarr(9,n)
          tbl[0,*] = infiles
          tbl[1,*] = obj
          tbl[2,*] = md
          tbl[3,*] = tm
          tbl[4,*] = coad
          tbl[5,*] = cycl
          tbl[6,*] = airm
          tbl[7,*] = wl
          tbl[8,*] = stt
          nn = 8 
     ENDELSE
     cd, path, current=nowp
     openw, logun, outfile, /get_lun
     printf, logun, '                         OBSERVING LOG           '
     printf, logun, '      '
     printf, logun, 'OBSERVER: ', obsrvr
     printf, logun, 'TELESCOPE: ', tel
     printf, logun, 'INSTRUMENT: ', ins_name
     printf, logun, 'DATE: ', dt
     printf, logun, '      '
     printf, logun, lbl
     FOR j = 0, (n-1) DO BEGIN
          printf, logun, tbl[0, j]
          IF nn EQ 7 THEN BEGIN
               printf, logun, tbl[1, j], '   ', tbl[2, j], '   ', tbl[3, j], $ 
                       '   ', tbl[4, j], '   ', tbl[5, j], '   ', tbl[6, j], $ 
                       '   ', tbl[7, j]
          ENDIF ELSE $ 
               printf, logun, tbl[1, j], '  ', tbl[2, j], '   ', tbl[3, j], $ 
                       '  ', tbl[4, j], '  ', tbl[5, j], '  ', tbl[6, j], $ 
                       '  ', tbl[7, j], '  ', tbl[8, j]
     ENDFOR
     close, logun
     free_lun, logun
     cd, nowp

END
