; NAME: 
;      MCSHIFT
; 
; VERSION: 1
; 
; PURPOSE:
;      A Monte Carlo-based procedure that coregisters images in an image cube.
; 
; PROCEDURE:   
;      The standard deviation of the difference between a reference and
;      registration image is minimized by random shifts of the registration
;      image.  Random displacements are normally distributed with a mean of 
;      zero and a standard deviation of one-tenth.  X and Y displacements are 
;      taken from the same distribution.
; 
; CALLING SEQUENCE:
;      mcshift, image, imad, [stsh=stsh, rsd=rsd, help=help]
; 
; INPUTS: 
;      image- Image cube with reference image followed by registration
;             images.  Succesive images are placed in z-direction.
; 
; OPTIONAL KEYWORD INPUTS:  
;      stsh- Optional initial guess for shift of 2 x n numeric array
;            for (n+1) images.
;      rsd - Standard deviation of normally-distributed numbers 
;            (default is 0.1)
;      help- If set to nonzero scalar, MCSHIFT prints syntax help 
;            and returns to caller.
; 
; OUTPUT: 
;      imad- 2 x n floating point array with x and y direction 
;            shifts for (n+1) images in image.
; 
; EXTERNAL CALLS: 
;      ROUND_OFF
; 
; COMMON BLOCKS: 
;      NONE
; 
; MODIFICATION HISTORY:
;      -Written by Eric Volquardsen, NASA IRTF, Nov. 2002
; 
; =========================================================================
PRO mcshift, image, imad, stsh=stsh, rsd = rsd, help=help

; CHECK INPUTS
  IF n_elements(image) EQ 0 THEN help = 1 ELSE BEGIN
       sz = size(image)
       IF n_elements(sz) NE 6 THEN help = 1 
       IF sz[4] EQ 7 THEN help = 1 
  ENDELSE
  IF n_elements(help) EQ 1 THEN BEGIN
       IF help NE 0 THEN BEGIN
            print, '***********************************************'
            print, '***              MCSHIFT SYNTAX             ***'
            print, '*** > mcshift, image, imad, stsh=stsh       ***'
            print, '***                                         ***'
            print, '*** image- Image cube with reference image  ***'
            print, '***        followed by registration images. ***'
            print, '***        Succesive images are placed in   ***'
            print, '***        z-direction.                     ***' 
            print, '*** imad- 2 x n floating point array with   ***' 
            print, '***       x and y shifts calculated by      ***'
            print, '***       MCSHIFT for (n+1) images in image.***'
            print, '*** stsh- Optional initial guess for shift  ***' 
            print, '***       of 2 x n numeric array for (n+1)  ***' 
            print, '***       images.                           ***'
            print, '*** rsd - Standard deviation of normally-   ***' 
            print, '***       distributed numbers               ***' 
            print, '***********************************************'
            RETURN
       ENDIF
  ENDIF
  IF n_elements(stsh) LT 2*(sz[3]-1) THEN stsh = fltarr(2, (sz[3]-1))
  IF n_elements(rsd) NE 1 THEN rsd = 0.1

; INITIALIZE VALUES 
  imad = fltarr(2, (sz[3]-1))              ; output shifts
  n = n_elements(image[*,*,0])             ; # of pixels
  ref = image[*,*,0]                       ; reference
  img = fltarr(sz[1], sz[2])               ; current registration
  seed = (systime(1) - 1.0e+9)/(1.0e+6)    ; random # seed
  rnd = rsd*(randomn(seed, 10000))         ; random numbers
  xi = findgen(sz[0])                      ; x-indicies
  yi = findgen(sz[1])                      ; y-indicies
  resid = 0                                ; sum of standard dev. 
  preresid = sqrt(total((image[*,*,0])^2)) ; previous resid
  i = 0                                    ; rejection index
  r = 0                                    ; random # index
  prreset = preresid

; LOOP TO COREGISTER EACH PAIR OF IMAGES
  FOR p = 1, (sz[3]-1) DO BEGIN
       dx = stsh[0, (p-1)]
       dy = stsh[1, (p-1)]
       img = image[*,*,p]
       i = 0 
       preresid = prreset

; SEARCH FOR STOPPING CONDITIONS
       WHILE (i LT 10) DO BEGIN

; GET SUM OF ST. DEV. OF DIFFERENCE BETWEEN REFERENCE IMAGE
;  AND REGISTERED IMAGE
            xind = xi + dx 
            yind = yi + dy 
            ims = interpolate(img, xind, yind, /grid)
            sum = ref - ims
            mean = total(sum)/n
            sd = sqrt(total((sum-mean)^2)/(n-1))
            resid = sd

; CHECK TO SEE IF SHIFT SHOULD BE ACCEPTED OR REJECTED
            IF resid LT preresid THEN BEGIN
                 i = 0 
                 preresid = resid
                 imad[0,(p-1)] = round_off(dx, 0.01)
                 imad[1,(p-1)] = round_off(dy, 0.01)

            ENDIF ELSE i = i + 1

; SET VALUES FOR NEXT LOOP
            dx = imad[0,(p-1)] + rnd[r]
            dy = imad[1,(p-1)] + rnd[r+1]
            r = r + 2

; RESET RANDOM NUMBERS IF NEEDED
            IF r GT 9996 THEN BEGIN
                 r = 1
                 rnd = (-0.5)*rnd
            ENDIF

       ENDWHILE   ; i while loop

  ENDFOR   ; p loop

END   ; mcshift
