; NAME: 
;    MIRSI_COAD
; 
; PURPOSE:
;    Procedure for reading MIRSI FITS images, coadding them, and 
;    saving the results as a FITS file.  Some noise artifacts
;    particular to MIRSI are removed.
; 
; CALLING SEQUENCE:
;    mirsi_coad, infile, [outfile, path, ext=ext, help=help]
; 
; INPUTS:
;    infile- Filename of images to coadd in FITS format with extensions.  
;            Four chop/nod images saved as extensions 0-3.
; 
; OPTIONAL INPUTS: 
;    ext - Set to nonzero scalar if image is of an extended object.
;    path- One or two element string vector with input and output paths. 
;          Default is current directory.
;    outfile- Filename of resulting coadds.  Default is coad.dat
; 
; OPTIONAL KEYWORD INPUTS: 
;    help- Set to nonzero scalar to have MIRSI_COAD print syntax.
;
; OUTPUT: 
;    An image cube with two images is saved to outfile.  They are
;    formed by (ext0-ext1)-(ext2-ext3) and (ext1-ext0)-(ext3-ext2)
;    respectively.
;
; EXTERNAL CALLS: 
;      CGMEDF, FXADDPAR, FXPAR, FXWRITE, GETIM, GETWRD, SDEV, SXDELPAR
; 
; COMMON BLOCKS: 
;      NONE
; 
; MODIFICATION HISTORY:
;      - Written by Eric Volquardsen, NASA IRTF, November 2002
;      - Extended object capability added by ELV, October 2003
;      - Changed MEANTIME to 3rd header start time, ELV, November 2004
; 
; =========================================================================
PRO mirsi_coad, infile, outfile, path, ext=ext, help=help

; CHECK INPUTS
   IF n_elements(infile) EQ 0 THEN help = 1 
   IF n_elements(help) EQ 1 THEN BEGIN
        IF help NE 0 THEN BEGIN
             print, '*********************************************'
             print, '***           MIRSI_COAD SYNTAX           ***' 
             print, '*** > mirsi_coad, infile, outfile, path   ***'
             print, '***               ext=ext                 ***' 
             print, '***                                       ***'
             print, '*** infile- filename of images to coadd   ***'
             print, '*** path- One or two element string       ***' 
             print, '***       vector with input and output    ***' 
             print, '***       paths.  List only one if both   ***' 
             print, '***       are the same. Default is home   ***' 
             print, '***       directory.                      ***' 
             print, '*** outfile- Filename of resulting coadds ***' 
             print, '*** ext- Set for extended objects         ***' 
             print, '*********************************************'
             RETURN 
        ENDIF
   ENDIF
   IF n_elements(outfile) EQ 0 THEN outfile='coad.dat'
   np = n_elements(path)
   CASE np OF 
        2 : 
        1 : path = [path, path]
        ELSE : BEGIN
             v = !version.os_family
             CASE v OF
                  'vms' : root = '[]'
                  'windows' : root = 'c:\'
                  ELSE : root = '/'
             ENDCASE
             cd, root, current=nowp
             cd, nowp
             path = [nowp, nowp]
        END
   ENDCASE
   IF n_elements(ext) EQ 1 THEN BEGIN
        IF ext NE 0 THEN ext = 1 
   ENDIF ELSE ext = 0 

; INITIALIZE NEEDED VALUES
   im = lonarr(320, 240, 4)
   co = lonarr(320, 240, 4)
   co1 = lonarr(320, 240, 2)
   hdr = bytarr(80, 55, /nozero)
   hdr1 = bytarr(80, 55, /nozero)
   hdr2 = bytarr(80, 55, /nozero)
   hdr3 = bytarr(80, 55, /nozero)
   tttt = fltarr(4)
   meantm = ''

; Set size of subimage if object is extended
; (size of region without object)
   IF n_elements(ext) EQ 1 THEN BEGIN
        IF ext NE 0 THEN nc = 200 ELSE nc = 320
   ENDIF ELSE nc = 320

; LOAD IMAGES
   GETIM, infile, im1, path[0], extend=0, hdr=hdr, nos=1
   GETIM, infile, im2, path[0], extend=1, hdr=hdr1, nos=1
   GETIM, infile, im3, path[0], extend=2, hdr=hdr2, nos=1
   GETIM, infile, im4, path[0], extend=3, hdr=hdr3, nos=1

; Check for file completeness
   IF n_elements(im1) LT 100 THEN RETURN
   IF n_elements(im2) LT 100 THEN RETURN
   IF n_elements(im3) LT 100 THEN RETURN
   IF n_elements(im4) LT 100 THEN RETURN
   IF n_elements(hdr) LT 1 THEN RETURN
   IF n_elements(hdr1) LT 1 THEN RETURN
   IF n_elements(hdr2) LT 1 THEN RETURN
   IF n_elements(hdr3) LT 1 THEN RETURN

; Create image cube
   im[*,*,0] = im1[*,*]
   im[*,*,1] = im2[*,*]
   im[*,*,2] = im3[*,*]
   im[*,*,3] = im4[*,*]

; Get observation times
;   thist = strtrim(FXPAR(hdr, 'TIME-STR'), 2)
;   hr = float(GETWRD(thist, 0, delimiter=':'))
;   minit = float(GETWRD(thist, 1, delimiter=':'))
;   secnd = float(GETWRD(thist, 2, delimiter=':'))
;   tttt[0] = 3600*hr+60*minit+secnd
;   thist = strtrim(FXPAR(hdr1, 'TIME-STR'), 2)
;   hr = float(GETWRD(thist, 0, delimiter=':'))
;   minit = float(GETWRD(thist, 1, delimiter=':'))
;   secnd = float(GETWRD(thist, 2, delimiter=':'))
;   tttt[1] = 3600*hr+60*minit+secnd
   meantm = strtrim(FXPAR(hdr2, 'TIME-STR'), 2)
;   hr = float(GETWRD(thist, 0, delimiter=':'))
;   minit = float(GETWRD(thist, 1, delimiter=':'))
;   secnd = float(GETWRD(thist, 2, delimiter=':'))
;   tttt[2] = 3600*hr+60*minit+secnd
;   thist = strtrim(FXPAR(hdr3, 'TIME-STR'), 2)
;   hr = float(GETWRD(thist, 0, delimiter=':'))
;   minit = float(GETWRD(thist, 1, delimiter=':'))
;   secnd = float(GETWRD(thist, 2, delimiter=':'))
;   tttt[3] = 3600*hr+60*minit+secnd

; COADD IMAGES
   co[*,*,0] = (im[*,*,0] - im[*,*,1])
   co[*,*,1] = (im[*,*,2] - im[*,*,3])
   co[*,*,2] = (im[*,*,1] - im[*,*,0])
   co[*,*,3] = (im[*,*,3] - im[*,*,2])
   i_mx = 3

; Non-extended case
   IF ext EQ 0 THEN BEGIN
        co[*,*,0] = co[*,*,0] - co[*,*,1]
        co[*,*,1] = co[*,*,2] - co[*,*,3]
        i_mx = 1
   ENDIF

; REMOVE NOISE
   arra = fltarr(320, 240)

; Loop through positive and negative coadds
   FOR i = 0, i_mx DO BEGIN
        arra[*,*] = co[*,*,i]

; Loop through groups of columns
        FOR j = 0, 14 DO BEGIN   ; 1st j loop

; Find standard deviation
             k = 15 - j
             sd = SDEV(arra[(k*20):(k*20+19),*])
             sd = sd[1]


; Stop check if standard deviation is reasonable
;  This is to check last several column groups for noise artifiacts
;  sometimes found in MIRSI images

             IF sd LT 10000 THEN j = 15
        ENDFOR   ; j loop

; Check for excessive column group deletion
        IF k LT 11 THEN BEGIN
             print, k
             print, 'Excessive column group deletion'
             print, 'due to noise in ', infile
             RETURN
        ENDIF

; Get rid of high noise column groups

        IF k LT 15 THEN BEGIN
             kk = k+1
             co[(kk*20):319, *, i] = fltarr((320-kk*20), 240)
        ENDIF ELSE kk = k+1
        arrb = fltarr((kk*20), 240)
        arrb = arra[0:(kk*20-1), *]
        IF kk*20 LT nc THEN nc = kk*20-60

; Determine which column groups to use in filter
; (removes those with bright extended objects in them)
        IF ext EQ 1 THEN BEGIN 
             sd1 = SDEV(arrb[0:nc-1, *])
             sd2 = SDEV(arrb[(kk*20-nc):(kk*20-1), *])
             IF sd1[1] GT sd2[1] THEN BEGIN
                  mnw = kk*20-nc
                  mxw = kk*20-1
             ENDIF ELSE BEGIN
                  mnw = 0 
                  mxw = nc-1
             ENDELSE	
         ENDIF ELSE BEGIN
             mnw = 0 
             mxw = nc-1
         ENDELSE 

; Filter
        arrc = fltarr((kk*20), 240)
        arrc = CGMEDF(arrb, 20, ss=[mnw,mxw], ext=ext)
        IF n_elements(arrc) EQ 1 THEN BEGIN
             arrc = arrb
             print, 'UNABLE TO FILTER COLUMN GROUP NOISE'
        ENDIF

; Adjust coadded image
        co[*,*,i] = fltarr(320, 240)
        co[0:(kk*20-1), *, i] = arrc

   ENDFOR   ; i loop

   IF ext EQ 1 THEN BEGIN   
        co1[*,*,0] = co[*,*,0] - co[*,*,1]
        co1[*,*,1] = co[*,*,2] - co[*,*,3]
   ENDIF ELSE BEGIN
        co1[*,*,0] = co[*,*,0]
        co1[*,*,1] = co[*,*,1]
   ENDELSE

; SAVE COADDS AND HEADER INFO
; Change to output directory
    cd, path[1], current=nowp

; Get mean time of observation
;    mntime = total(tttt)/4.0
;    mhr = strtrim(fix(mntime/3600), 2)
;    mminit = strtrim(fix((mntime-3600.0*mhr)/60), 2)
;    msecnd = strtrim(round(mntime-3600.0*mhr-60.0*mminit), 2)
;    IF strlen(mhr) EQ 1 THEN mhr = '0'+mhr
;    IF strlen(mminit) EQ 1 THEN mminit = '0'+mminit
;    IF strlen(msecnd) EQ 1 THEN msecnd = '0'+msecnd
;    meantm = string(mhr)+':'+string(mminit)+':'+string(msecnd)

; Alter header as needed
    msg = 'Coadded by mirsi_coad.pro'
    SXDELPAR, hdr, 'EXTEND'
    FXADDPAR, hdr, 'MEANTIME', meantm
    FXADDPAR, hdr, 'CCOMMENT', msg
    FXADDPAR, hdr, 'NAXIS', 3
    FXADDPAR, hdr, 'ORIGINAL', infile
    FXADDPAR, hdr, 'NAXIS3', 2, after='NAXIS2'

; save header and coadded images in fits file
    FXWRITE, outfile, hdr, co1
    cd, nowp

END 
