; NAME: 
;      NSFCAM_REG
;
; VERSION: 1.1
; 
; PURPOSE: 
;      IRTF NSFCAM reduction pipeline subroutine that registers
;      images.  Registration can be by finding the peak in each 
;      image, or by cross-correlation of images.  The header of 
;      each image is also searched for the LOCATION keyword, which 
;      has the x and y pixel coordinates of the object.  The 
;      returned image is a robust average of the registered images.
; 
; CALLING SEQUENCE: 
;      nsfcam_reg, infiles, outfile, path, [dither=dither, 
;                      plot=plot, mshft=mshft, help=help]
; 
; INPUTS: 
;      infiles - Filenames of images to register. 
;      outfile - Filename where registered images should be saved
; 
; OPTIONAL INPUTS:
;      path- 2-element string vector listing paths 
;
; OPTIONAL KEYWORD INPUTS: 
;      dither - 2n element vector with x and y displacements for an 
;               n point dither pattern (in pixels).  Dither overrides
;               position keyword in image header.
;      plot - Set to nonzero scalar to display images saved to
;             outfile. 
;      mshft - Maximum allowable shift in pixels (default is 10) 
;      ccorel - Set to use cross-correlation, rather than peak
;               location to register images.
;      help- Set to nonzero scalar to print syntax. 
; 
; OUTPUT:
;      Registered image is saved to outfile
; 
; EXTERNAL CALLS: 
;      ADDARR, CORREL_IMAGES, CORRMAT_ANALYZE, FULLWID_HALFMAX, 
;      FXADDPAR, FXPAR, FXWRITE, GETIM, IM_STATS, SXDELPAR
; 
; COMMON BLOCK: 
;      NONE 
; 
; MODIFICATION HISTORY: 
;      - Written by Eric Volquardsen, NASA IRTF, January 2004
;      - Return robust average of registered images, ELV, May 2004
; 
; =========================================================================
PRO nsfcam_reg, infiles, outfile, path, dither = dither, plot = plot, $ 
                    mshft = mshft, ccorel = ccorel, help = help

; CHECK INPUTS 
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN help = 1 
     ENDIF ELSE help = 0 
     IF n_elements(infiles) LT 2 THEN help = 1
     IF n_elements(outfile) NE 1 THEN help = 1 
     IF help EQ 1 THEN BEGIN
          print, '************************************************'
          print, '***            NSFCAM_REG SYNTAX             ***' 
          print, '*** nsfcam_reg, infiles, outfile, path,      ***'
          print, '***                 dither=dither,           ***'
          print, '***                 plot=plot, mshft=mshft,  ***' 
          print, '***                 ccorel=ccorel            ***'
          print, '***                                          ***'
          print, '*** infiles - Names of files to register     ***' 
          print, '*** outfile - Filename where registered      ***' 
          print, '***           image will be saved            ***'  
          print, '*** path- 2-element vector listing paths     ***'  
          print, '***       for infiles and outfile            ***'
          print, '*** dither - 2n element vector with x and y  ***' 
          print, '***          displacements for an n-point    ***' 
          print, '***          dither pattern (in pixels)      ***' 
          print, '*** plot - Set to display results            ***'   
          print, '*** mshft - Maximum allowable shift          ***'
          print, '*** ccorel - Set to use cross-correlation    ***'
          print, '***          rather than peak location to    ***' 
          print, '***          register images                 ***' 
          print, '************************************************' 
          RETURN 
     ENDIF  
     np = n_elements(path)
     CASE np OF  
          1 : path = [path, path]
          0 : BEGIN
               v = !version.os_family
               CASE v OF
                    'vms' : root = '[]'
                    'windows' : root = 'c:\'
                    ELSE : root = '/'
               ENDCASE
               cd, root, current=nowp
               cd, nowp
               path = [nowp, nowp]
          END
          ELSE : path = path[0:1]
     ENDCASE
     n = n_elements(infiles)
     IF n_elements(dither) LT 2 THEN dither = -1
     IF n_elements(plot) EQ 1 THEN BEGIN
          IF plot NE 0 THEN plot = 1 
     ENDIF ELSE plot = 0 
     IF n_elements(mshft) NE 1 THEN mshft = 10
     IF n_elements(ccorel) NE 1 THEN ccorel = 0
;     keywd = 'UT'         ; QUIRC
     keywd = 'TIME_OBS'   ; NSFCAM

; LOAD IMAGES
     cofn = ''
     cpos = fltarr(2, n) - 1.0
     tttt = fltarr(n)
     FOR i = 0, (n-1) DO BEGIN
          GETIM, infiles[i], img, path[0], hdr = hdr
          IF n_elements(img) EQ 1 THEN RETURN
          position = FXPAR(hdr, 'LOCATION')
          posit = strsplit(position, /extract)
          nposit = n_elements(posit)
          IF nposit EQ 2 THEN cpos[*, i] = posit
          IF i EQ 0 THEN BEGIN 
               sz = size(img, /dimension)
               IF n_elements(sz) NE 2 THEN BEGIN 
                    sz =  sz[0:1]
                    img = img[*, *, 0]
               ENDIF 
               images = fltarr(sz[0], sz[1], n)
          ENDIF 
          images[*, *, i] = img[*, *, 0]

; Specify original files registered
; (looks for previous registration)
          orig1 = FXPAR(hdr, 'ORIGINAL')
          norig = strpos(orig1, '/')
          IF norig EQ -1 THEN BEGIN 
               cofn = cofn+' / '+infiles[i]
          ENDIF ELSE BEGIN 
               IF i EQ 0 THEN cofn = orig1 $ 
                    ELSE cofn = cofn+' / '+orig1
          ENDELSE

; Get observation time
          thist = strtrim(FXPAR(hdr, keywd), 2)
          hr = float(GETWRD(thist, 0, delimiter = ':'))
          minit = float(GETWRD(thist, 1, delimiter = ':'))
          secnd = float(GETWRD(thist, 2, delimiter = ':'))
          tttt[i] = 3600*hr+60*minit+secnd  
     ENDFOR

; PERFORM DITHERING SHIFT (if needed)
     IF dither[0] NE -1 THEN BEGIN
          j = 0 
          FOR i = 0, (n-1) DO BEGIN
               IF i GT (n_elements(dither)/2-1) THEN i = 0 
               d_shift = dither[(2*i):(2*i+1)]
               images[*, *, j] = shift(images[*, *, j], d_shift[0], d_shift[1])
               j = j + 1 
               IF j GT (n-1) THEN GOTO, skp1
          ENDFOR
          skp1: 
     ENDIF

; CALCULATE MEAN OBSERVATION TIME OF REGISTERED IMAGE
     mntime = total(tttt)/float(n)
     mhr = strtrim(fix(mntime/3600), 2)
     mminit = strtrim(fix((mntime-3600.0*mhr)/60), 2) 
     msecnd = strtrim(round(mntime-3600.0*mhr-60*mminit), 2)
     IF strlen(mhr) EQ 1 THEN mhr = '0'+mhr
     IF strlen(mminit) EQ 1 THEN mminit = '0'+mminit
     IF strlen(msecnd) EQ 1 THEN msecnd = '0'+msecnd
     meantm = mhr+':'+mminit+':'+msecnd

; CALCULATE SHIFT AMOUNTS
     mcs = fltarr(2, (n-1))
     cnt = fltarr(2, n)
     FOR i =  0,  (n-1) DO BEGIN 
; By finding peak
          IF ccorel EQ 0 THEN BEGIN 
               ss = IM_STATS(images[*, *, i])
               IF cpos[0, i] EQ -1 THEN cnt[*, i] = ss.peak_xy $ 
                    ELSE cnt[*, i] = cpos[*, i]
               IF i GT 0 THEN BEGIN 
                    IF abs(cnt[0, i] - cnt[0, 0]) GT mshft THEN $ 
                         cnt[*, i] = cnt[*, 0]
                    IF abs(cnt[1, i] - cnt[1, 0]) GT mshft THEN $ 
                         cnt[*, i] = cnt[*, 0]
                    mcs[*, (i-1)] = cnt[*, i] - cnt[*, 0]
               ENDIF 
               scnt = strtrim(string(cnt[0, 0]), 2) + $
                      '     ' + strtrim(string(cnt[1, 0]), 2)
               FXADDPAR, hdr, 'LOCATION', scnt
          ENDIF ELSE BEGIN 
; By cross correlation
               IF i NE 0 THEN BEGIN 
                     cmat = correl_images(images[*, *, 0], $ 
                            images[*, *, i], xshift = mshft, yshift = mshft)
                     corrmat_analyze, cmat, mcs[0, (i-1)], mcs[1, (i-1)]
                ENDIF 
          ENDELSE      
     ENDFOR 

; CREATE OUTPUT
     ADDARR, images, mcs, cor, /robust

; Make adjustments to header
          SXDELPAR, hdr, 'MEANTIME'
          SXDELPAR, hdr, 'ORIGINAL'
          FXADDPAR, hdr, 'ORIGINAL', cofn
          FXADDPAR, hdr, 'RCOMMENT', 'Registered by nsfcam_reg'
          FXADDPAR, hdr, 'MEANTIME', meantm
          FXADDPAR, hdr, 'BITPIX', -32

; Plot (if set)
          IF plot EQ 1 THEN BEGIN
               ttl = 'REGISTRATION RESULTS FOR '+ strtrim(outfile, 2)
               window, 1, title = ttl, xsize = sz[0], ysize = sz[1]
               wset, 1
               tvscl, cor[*, *]
               print, 'Registration Shifts'
               print, mcs
          ENDIF  

; Add data to file
     cd, path[1], current = nowp
     FXWRITE, outfile, hdr, cor
     cd, nowp

END 







