; NAME: 
;      OVERSUB
; 
; VERSION: 1
; 
; PURPOSE:
;      Return image with overscan removed and overscan value
;      subtracted from original.  Up to four regions may be designated.
; 
; CALLING SEQUENCE:
;      oversub, fin, fout, [path=path, r1=r1, r2=r2, r3=r3, r4=r4, 
;           bsub=bsub, nosub=nosub, help=help]
; 
; INPUTS:
;      fin- Filename of image to be trimmed
; 
; OPTIONAL KEYWORD INPUTS: 
;      path - String vector with fin and fout paths
;      r1- First overscan region (lowx, hix, lowy, hiy)
;          Default is last 16 columns of image.  If only one number
;          is passed (n), then region is assumed to be ther last n col.
;      r2- Second overscan region
;      r3- Third overscan region
;      r4- Fourth overscan region
;      bsub- How overscan subtraction value should be calculated.
;            (0) for median of all pixel values in first region
;            (1) for 2nd order polynomial fit to median of each row
;      nosub- If set, overscan value is not subtracted from image
;      help- Set to nonzero scalar to print syntax
; 
; OUTPUTS:
;      fout- Trimmed image saved here
; 
; COMMON BLOCKS:
;      NONE 
; 
; EXTERNAL CALLS: 
;      FXADDPAR, FXWRITE, GETIM
; 
; MODIFICATION HISTORY:
;      -Written by Eric Volquardsen, NASA IRTF, February 2003
;
; ====================================================================
PRO oversub, fin, fout, path=path, r1=r1, r2=r2, r3=r3, r4=r4, $ 
             bsub=bsub, nosub=nosub, help=help

; CHECK INPUTS 
     IF n_elements(fin) NE 1 THEN help = 1 
     IF n_elements(fout) NE 1 THEN help = 1  
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN BEGIN 
               print, '***********************************************'
               print, '***            OVERSUB SYNTAX               ***'
               print, '*** oversub, fin, fout, path=path, rn=rn,   ***'
               print, '***      bsub=bsub, nosub=nosub             ***'
               print, '***                                         ***'
               print, '*** fin- Input array filename               ***'
               print, '*** fout- Output array filename             ***'
               print, '*** rn- n is numbers 1 through 4            ***'
               print, '***     4 element numerical vector with     ***'
               print, '***     boundries of overscan (x1,x2,y1,y2) ***'
               print, '***     r1=n deletes last n columns         ***' 
               print, '*** bsub- Sets how average is computed      ***'
               print, '***       0 for total median                ***'
               print, '***       1 for 2nd order fit to rows       ***' 
               print, '*** nosub- set if average not to be         ***'
               print, '***        subtracted from image            ***' 
               print, '***********************************************' 
               RETURN 
          ENDIF 
     ENDIF
     IF n_elements(bsub) NE 1 THEN bsub = 0
     IF bsub GT 1 THEN bsub = 1 
     IF bsub LT 0 THEN bsub = 1 
     np = n_elements(path)
     CASE np OF 
          0 : BEGIN
               v = !version.os_family
               CASE v OF
                    'vms' : root = '[]'
                    'windows' : root = 'c:\'
                    ELSE : root = '/'
                ENDCASE
                cd, root, current=pat
                cd, pat
                path = [pat, pat]
          END
          1 : path = [path, path]
          ELSE : path = path[0:1]
     ENDCASE
     chk = FILEX(fin, path[0])
     IF chk EQ 0 THEN BEGIN
          print, fin, ' DOES NOT EXIST'
          RETURN
     ENDIF

; LOAD IMAGE
     GETIM, fin, arrin, path[0], hdr=h

; DETERMINE REGIONS TO REMOVE
     sz = size(arrin, /dimensions)
     IF n_elements(r1) EQ 1 THEN r1 = [(sz[0]-r1), (sz[0]-1), 0, (sz[1]-1)]
     IF n_elements(r1) LT 4 THEN r1 = [(sz[0]-16), (sz[0]-1), 0, (sz[1]-1)]
     IF n_elements(r2) LT 4 THEN r2 = [(-1)] 
     IF n_elements(r3) LT 4 THEN r3 = [(-1)]
     IF n_elements(r4) LT 4 THEN r4 = [(-1)]
     IF n_elements(nosub) NE 1 THEN nosub = 0 
     IF nosub NE 0 THEN nosub = 1 
     sr1 = (r1[1]-r1[0]+1)*(r1[3]-r1[2]+1) 

; CREATE IMAGE WITHOUT OVERSCAN REGION(S)
; Get size and starting points for first region 
     szz = [0.0, 0.0]
     IF (r1[1]-r1[0]+1) LT sz[0] THEN BEGIN 
          szz[0] = sz[0] - r1[1] + r1[0] - 1
          szz[1] = sz[1] 
          IF r1[1] LE (sz[0]-1) THEN nx = 0 ELSE nx = (r1[1]+1)
          ny = 0 
          av = fltarr(sz[1]-1)
          si = (sz[1]-1)
          cl = 1 
     ENDIF ELSE BEGIN
          szz[0] = sz[0] 
          szz[1] = sz[1] - r1[3] + r1[2] - 1 
          nx = 0 
          IF r1[3] LT (sz[1]-1) THEN ny = 0 ELSE ny = (r1[3]+1)
          av = fltarr(sz[0]-1)
          si = (sz[0]-1) 
          cl = (-1) 
     ENDELSE 

; Adjust size and starting points for subsequent regions 
     IF r2[0] NE (-1) THEN BEGIN
          IF (r2[1]-r2[0]+1) LT sz[0] THEN BEGIN 
               szz[0] = szz[0] - r2[1] + r2[0] - 1
               IF r2[1] EQ (sz[0]-1) THEN nx = (r2[1]+1+nx) 
          ENDIF ELSE BEGIN
               szz[1] = szz[1] - r2[3] + r2[2] - 1 
               IF r2[3] EQ (sz[1]-1) THEN ny = (r2[3]+1+ny)
          ENDELSE 
     ENDIF
     IF r3[0] NE (-1) THEN BEGIN
          IF (r3[1]-r3[0]+1) LT sz[0] THEN BEGIN 
               szz[0] = szz[0] - r3[1] + r3[0] - 1
               IF r3[1] EQ (sz[0]-1) THEN nx = (r3[1]+1+nx) 
          ENDIF ELSE BEGIN
               szz[1] = szz[1] - r3[3] + r3[2] - 1 
               IF r3[3] EQ (sz[1]-1) THEN ny = (r3[3]+1+ny)
          ENDELSE 
     ENDIF
     IF r4[0] NE (-1) THEN BEGIN
          IF (r4[1]-r4[0]+1) LT sz[0] THEN BEGIN 
               szz[0] = szz[0] - r4[1] + r4[0] - 1
               IF r4[1] EQ (sz[0]-1) THEN nx = (r4[1]+1+nx) 
          ENDIF ELSE BEGIN
               szz[1] = szz[1] - r4[3] + r4[2] - 1 
               IF r4[3] EQ (sz[1]-1) THEN ny = (r4[3]+1+ny)
          ENDELSE 
     ENDIF

; Create subimage
     arrout = fltarr(szz[0], szz[1]) 
     arrout[*,*] = arrin[nx:(nx+szz[0]-1), ny:(ny+szz[1]-1)]

; CALCULATE OVERSCAN SUBTRACTION VALUE
     IF nosub EQ 0 THEN BEGIN

          IF bsub EQ 1 THEN BEGIN
               FOR i = 0, si DO BEGIN
                    IF cl EQ 1 THEN BEGIN 
                         av[i] = median(arrin[r1[0]:r1[1], i], /even) 
                    ENDIF ELSE BEGIN 
                         av[i] = median(arrin[i, r1[2]:r1[3]], /even) 
                    ENDELSE 
               ENDFOR
               x = indgen(si+1)
               coef = poly_fit(x, av, 2)
               IF cl EQ 1 THEN BEGIN
                    FOR j = 0, szz[1] DO BEGIN 
                         arrout[*,j] = arrout[*,j] - coef[0] - $ 
                              coef[1]*(j+ny) - coef[2]*((j+ny)^2)
                    ENDFOR 
               ENDIF ELSE BEGIN
                    FOR j = 0, szz[0] DO BEGIN 
                         arrout[j,*] = arrout[j,*] - coef[0] - $ 
                              coef[1]*(j+nx) - coef[2]*((j+nx)^2)
                    ENDFOR 
               ENDELSE 
          ENDIF ELSE BEGIN 
               aval = median(arrin[r1[0]:r1[1], r1[2]:r1[3]], /even)
               arrout = arrout - aval 
          ENDELSE 

     ENDIF  

; MODIFY HEADER
     comment = '= Overscan subtraction performed on ' + strtrim(fin, 2)
     IF nosub EQ 0 THEN $ 
          comment = '= Overscan area removed from ' + strtrim(fin, 2)
     FXADDPAR, h, 'COMMENT', comment
     FXADDPAR, h, 'NAXIS1', szz[0]
     FXADDPAR, h, 'NAXIS2', szz[1]

; SAVE RESULT
     cd, path[1], current=nowp
     FXWRITE, fout, h, arrout
     cd, nowp

END   ; oversub 
