; NAME: 
;      PAIR_SUB
; 
; VERSION: 1
; 
; PURPOSE:
;      Performs A-B and B-A subtraction of spectra and removes 
;      pixels that are more that 2 sigma from median of a 5 x 5 
;      moving box.
; 
; CALLING SEQUENCE: 
;      pair_sub, ifa, ifb, suba, subb, [paths=paths, help=help]
; 
; INPUTS: 
;      IFA - File name of first image (a)
;      IFB - File name of second image (b)
;      
; OPTIONAL KEYWORD INPUTS: 
;      PATHS - One or two element vector containing paths for input 
;              and output files
;      HELP - Set to nonzero scalar to have PAIR_SUB print syntax
; 
; OUTPUT: 
;      SUBA - File name of A - B image
;      SUBB - File name of B - A image 
; 
; EXTERNAL CALLS: 
;      FITS_CLOSE, FITS_WRITE, FXADDPAR, GETIM, OUTLIER_FILTER
; 
; COMMON BLOCKS: 
;      NONE 
; 
; MODIFICATION HISTORY: 
;      - Written by Eric Volquardsen, NASA IRTF, April 2003
; 
; =======================================================================
PRO pair_sub, ifa, ifb, suba, subb, paths=paths, help=help

; CHECK INPUTS
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN help = 1
     ENDIF ELSE help = 0 

; Required inputs
     ian = n_elements(ifa)
     ibn = n_elements(ifb)
     IF ian LT 1 THEN help = 1 
     IF ian NE ibn THEN help = 1 
     IF help EQ 1 THEN BEGIN
          print, '*****************************************************'
          print, '***                 PAIR_SUB SYNTAX               ***'
          print, '*** > pair_sub, ifa, ifb, suba, subb, paths=paths ***'
          print, '***                                               ***' 
          print, '*** IFA - File name(s) of first image (A)         ***' 
          print, '*** IFB - File name(s) of second image (B)        ***' 
          print, '*** SUBA - Output file name(s) of A - B image     ***' 
          print, '*** SUBB - Output file name(s) of B - A image     ***' 
          print, '*** PATHS - One or two element vector containing  ***' 
          print, '***         paths for input and output files      ***'
          print, '***         (default is current directory)        ***' 
          print, '*****************************************************' 
          RETURN
     ENDIF 

; Optional inputs
     san = n_elements(suba)
     sbn = n_elements(subb)
     IF san NE ian THEN BEGIN
          suba = ifa
          subb = ifb
     ENDIF ELSE BEGIN
          IF sbn NE ibn THEN BEGIN
               suba = ifa
               subb = ifb 
          ENDIF ELSE BEGIN
               ifa = reform(ifa, ian)
               ifb = reform(ifb, ian)
               suba = reform(suba, ian)
               subb = reform(subb, ian)
          ENDELSE
     ENDELSE
     nep = n_elements(paths)
     CASE nep OF 
          0 : BEGIN
               v = !version.os_family
               CASE v OF
                    'vms' : root = '[]'
                    'windows' : root = 'c:\'
                    ELSE : root = '/'
               ENDCASE
               cd, root, current=nowp
               pa = [nowp, nowp]
               cd, nowp
          END
          1 : paths = [paths, paths]
          ELSE : paths = paths[0:1]
     ENDCASE   

; LOOP THROUGH PAIRS
     FOR i = 0, (ian-1) DO BEGIN

; Get files
          getim, ifa[i], imga, paths[0], hdr=hda
          getim, ifb[i], imgb, paths[0], hdr=hdb
          IF n_elements(imga) EQ 1 THEN RETURN
          IF n_elements(imgb) EQ 1 THEN RETURN

; Perform pair subtractions 
          mimga = imga - imgb
          mimgb = imgb - imga

; Remove outliers
          outlier_filter, mimga, 5, n_sigma=2, /median
          outlier_filter, mimgb, 5, n_sigma=2, /median

; Create subtracted pair headers
          fxaddpar, hda, 'IRAFNAME', suba[i]
          fxaddpar, hdb, 'IRAFNAME', subb[i]
          notea = ifa[i] + ' - ' + ifb[i]
          noteb = ifb[i] + ' - ' + ifa[i]
          fxaddpar, hda, 'PAIRSUB', notea
          fxaddpar, hdb, 'PAIRSUB', noteb

; Save subtracted pairs
          cd, paths[1], current=nowp
          fits_write, suba[i], mimga, hda
          fits_close, fcb
          fits_write, subb[i], mimgb, hdb
          fits_close, fcb
          cd, nowp

     ENDFOR   ; image pair loop

END 
