; NAME: 
;      PHOT_RESULTS
; 
; VERSION: 1.3
; 
; PURPOSE: 
;      Reads output of aperture photometry programs at selected 
;      aperture and creates an output file for a nights run.
; 
; CALLING SEQUENCE: 
;      phot_results, infiles, outfile, aper, [path=path, amc=amc, 
;                    am1=am1, time=time, jy=jy, help=help]
; 
; INPUTS: 
;      infiles - String vector of input filenames 
;      outfile - Filename of output file
;      aper - Aperture size in pixels
; 
; OPTIONAL KEYWORD INPUTS: 
;      path - Paths for infiles and outfile
;      amc - Set to correct instrumental magnitude for airmass
;            [zero point, extinction].  If only one value is given, 
;            then it is taken as the zero point correction.
;      am1 - Set to correct to 1 airmass (default is 0 airmass).
;            An error message is returned if am1 is set without 
;            a two elements vector being set in amc.
;      time - Set to print UT time in the output table, rather than 
;             cts/sec 
;      jy - Set to filter wavelength to convert magnitude to Janskys 
;      help - Set to nonzero scalar to print syntax
; 
; OUTPUT: 
;      Observer, instrument, location, observation date, and aperture
;      are saved.  This is followed by a table of object name, airmass,
;      counts/second, counts/second error, magnitude, and error. 
;      Results are appended to a file.
; 
; EXTERNAL CALLS: 
;      FILEX, JFLUX
; 
; COMMON BLOCKS: 
;      NONE
; 
; MODIFICATION HISTORY: 
;      - Written by Eric Volquardsen, NASA IRTF, October 2003
;      - AM1/TIME keywords added by ELV, NASA IRTF, September 2004
;      - JY keyword added by ELV, NASA IRTF, December 2004
; 
; =======================================================================
PRO phot_results, infiles, outfile, aper, path=path, amc=amc, am1 = am1, $
                  time = time, jy = jy, help=help

; CHECK INPUTS
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN help = 1 
     ENDIF ELSE help = 0 
     nf = n_elements(infiles)
     IF nf LT 1 THEN help = 1 
     IF n_elements(outfile) NE 1 THEN help = 1 
     IF n_elements(aper) NE 1 THEN help = 1 
     IF help EQ 1 THEN BEGIN
          print, '***************************************************'
          print, '***             PHOT_RESULTS SYNTAX             ***'  
          print, '*** IDL> phot_results, infiles, outfile, aper,  ***' 
          print, '***                    path=path, amc=amc,      ***' 
          print, '***                    am1=am1, time=time       ***'
          print, '***                                             ***' 
          print, '*** infiles - String vector of input filenames  ***' 
          print, '*** outfile - Filename of output file           ***' 
          print, '*** aper - Aperture size in pixels              ***' 
          print, '*** path - Paths for infiles and outfile        ***' 
          print, '*** amc - Set to correct instrumental magnitude ***' 
          print, '***       for airmass [zero point, extinction]  ***' 
          print, '*** am1 - Set to correct to 1 airmass           ***' 
          print, '***       (default is to correct to 0 airmass)  ***'
          print, '*** time - Set to print UT time in the output   ***' 
          print, '***        table, rather than cts/sec           ***'
          print, '*** jy - Set to filter wavelength to convert    ***' 
          print, '***      magnitudes to Janskys                  ***' 
          print, '***************************************************'   
          RETURN
     ENDIF
     np = n_elements(path)
     CASE np OF 
          0 : BEGIN
               v = !version.os_family
               CASE v OF
                    'vms' : root = '[]'
                    'windows' : root = 'c:\'
                    ELSE : root = '/'
                ENDCASE
                cd, root, current=pat
                cd, pat
                path = [pat, pat]
          END
          1 : path = [path, path]
          ELSE : path = path[0:1]
     ENDCASE
     IF n_elements(amc) LT 1 THEN amc = 0
     IF n_elements(am1) EQ 1 THEN BEGIN 
          IF n_elements(amc) LT 2 THEN BEGIN 
               print, ' '
               print, 'CANNOT CORRECT FOR AIRMASS'
               print, 'WITHOUT EXTINCTION COEFFICIENT'
               print, ' '
               RETURN
          ENDIF 
          IF am1 NE 0 THEN am1 = 1
     ENDIF ELSE am1 = 0
     IF n_elements(time) NE 1 THEN time = 0
     IF n_elements(jy) NE 1 THEN jy = 0
     jy = ABS(jy)

; INITALIZE VALUES FROM FIRST INPUT FILE
     data1 = strarr(20)
     chk = FILEX(infiles[0], path[0])
     IF chk EQ 0 THEN BEGIN
          print, infiles[0], ' DOES NOT EXIST'
          RETURN
     ENDIF
     cd, path[0], current=nowp
     openr, logun, infiles[0], /get_lun
     readf, logun, data1
     inst = strsplit(data1[1], /extract)
     IF n_elements(inst) GT 1 THEN inst = inst[1] ELSE inst = ''
     IF inst EQ 'Apogee' THEN skp = 20 ELSE skp = 21
     obs = strtrim(strmid(data1[0], 9), 2)
     loc = strtrim(strmid(data1[2], 9), 2)
     odt = strtrim(strmid(data1[5], 19), 2)
     sr1 = strsplit(data1[skp-12], /extract)
     sr1 = sr1[3]
     sr2 = strsplit(data1[skp-11], /extract)
     sr2 = sr2[3]
     sr = sr1 + ' to ' + sr2 + ' pixels'
     free_lun, logun

; CREATE TABLE
     ss = 0 
     f = 'a,a,a,a,a,a'
     table = strarr(6,nf)
     table[1:5, *] = '0'
     FOR i = 0, (nf-1) DO BEGIN
          nofile: 
          chk = FILEX(infiles[i], path[0])
          IF chk EQ 0 THEN BEGIN 
               print, infiles[i], ' DOES NOT EXIST'
               i = i + 1 
               IF i LE (nf-1) THEN GOTO, nofile ELSE GOTO, nofile2
          ENDIF
          IF skp EQ 21 THEN BEGIN
               openr, logun, infiles[i], /get_lun
               readf, logun, data1
               free_lun, logun 
               table[1, i] = strtrim(strmid(data1[8], 19), 2)
          ENDIF
          fst = strpos(infiles[i], '.')
          IF fst GT 0 THEN table[0,i] = strmid(infiles[i], 0, fst) $ 
               ELSE table[0,i] = infiles[i]
          READCOL, infiles[i], ar, ap, cs, cse, im, ime, format=f, $ 
                   skipline=skp
          w = where(float(ar) EQ aper)
          IF w EQ -1 THEN BEGIN
               print, infiles[i], ' DOES NOT HAVE APERTURE ', string(aper)
               i = i + 1 
               IF i LE (nf-1) THEN GOTO, nofile ELSE GOTO, nofile2
          ENDIF   
          IF time EQ 0 THEN BEGIN     
               table[2, i] = cs[w]
               table[3, i] = cse[w]
          ENDIF ELSE BEGIN 
               table[2, i] = strmid(data1[6], 26, 8)
               table[3, i] = ''
          ENDELSE 
          table[4, i] = im[w]
          table[5, i] = ime[w]
          s = strlen(table[0, i])
          IF s GT ss THEN ss = s 
     ENDFOR
     nofile2: 

; CORRECT FOR AIRMASS (if set)
     amc1 = '0.0'
     amc2 = '0.0'
     IF n_elements(amc) EQ 1 THEN BEGIN
          amc1 = strtrim(string(amc), 2)
          t1 = float(table[1,*])
          t4 = float(table[4,*])
          ww = where(t1 GE 1)
          table[4,ww] = string(t4[ww] - amc[0])
          table[4,*] = strtrim(table[4,*], 2)
     ENDIF ELSE BEGIN 
          amc1 = strtrim(string(amc[0]), 2)
          amc2 = strtrim(string(amc[1]), 2)
          t1 = float(table[1,*])
          t4 = float(table[4,*])
          ww = where(t1 GE 1)
          IF ww[0] EQ -1 THEN BEGIN 
               cc = (t4-t4*amc[1]-amc[0]) > 1.0
               table[4,*] = string(cc)
               print, ' '
               print, 'AIRMASS VALUES UNAVAILABLE'
               print, ' '
          ENDIF ELSE BEGIN 
               IF am1 NE 0 THEN tt1 = t1[ww] - 1 ELSE tt1 = t1[ww]
               table[4,ww] = string(t4[ww] - tt1*amc[1] - amc[0])
          ENDELSE 
          table[4,*] = strtrim(table[4,*], 2)
     ENDELSE

; SAVE RESULTS
     l1 = 'OBSERVER:     '
     l2 = 'INSTRUMENT:   '
     l3 = 'LOCATION:     '
     l4 = 'DATE:         '
     l5 = 'APERTURE:     '
     l6 = 'SKY RADII:    '
     l7 = 'ZERO POINT:   '
     l8 = 'EXTINCTION:   ' 
     IF time EQ 0 THEN BEGIN 
          tbl0 = '   Object    Airmass    CTS/sec      ' + $ 
                   'Error   Magnitude    Error'
          tbl1 = '|----------|----------|----------|' + $ 
                 '----------|---------|----------|'
     ENDIF ELSE BEGIN 
          tbl0 = '   Object    Airmass    Time_UT      Magnitude    Error'
          tbl1 = '|----------|----------|----------|' + $ 
                 '----------|---------|'
     ENDELSE 
     cd, path[1]
     openw, logun, outfile, /get_lun, /append
     printf, logun, '  '
     printf, logun, l1, obs
     printf, logun, l2, inst
     printf, logun, l3, loc
     printf, logun, l4, odt
     printf, logun, l5, strtrim(string(aper), 2)
     printf, logun, l6, sr
     printf, logun, l7, amc1
     printf, logun, l8, amc2
     IF am1 EQ 1 THEN printf, logun, 'CORRECTED TO 1 AIRMASS'
     printf, logun, '   '
     printf, logun, tbl0
     printf, logun, tbl1
     IF skp EQ 20 THEN spc = '          ' ELSE spc = '   '

     nct = max(strlen(table[2, *]))
     FOR j = 0, (nf-1) DO BEGIN 
          s = ss - strlen(table[0,j])
          CASE s OF 
               0 : spc1 = ' '
               1 : spc1 = '  '
               2 : spc1 = '   '
               3 : spc1 = '    '
               4 : spc1 = '     '
               ELSE : spc1 = '      '
          ENDCASE
          nc = nct - strlen(table[2, j])
          CASE nc OF 
               0 : nc1 = ' '
               1 : nc1 = '  '
               2 : nc1 = '   '
               3 : nc1 = '    '
               4 : nc1 = '     '
               ELSE : nc1 = '      '
          ENDCASE
          IF time EQ 0 THEN spc2 = '       ' ELSE spc2 = ''

; Convert to Janskys (if desired)
          IF jy NE 0 THEN BEGIN 
               junk = JFLUX(table[4,j], jy, uncer = table[5,j])
               table[4,j] = junk[0]
               table[5,j] = junk[1]
          ENDIF 
          line = spc1 + table[0,j] + spc + table[1,j] + nc1 + $ 
                 spc + table[2,j] + spc2 + table[3,j] + $ 
                 '      '  + table[4,j] + '   ' + table[5,j]
          printf, logun, line
     ENDFOR 
     free_lun, logun
     cd, nowp
END
