; NAME: 
;      PHOTCHECK
; 
; VERSION: 1
; 
; PURPOSE:
;      To load and display a FITS image.  Program will also return cursor
;      coordinates and surrounding pixel values when marked.
;      Individual object locations can be selected and saved to the
;      FITS header.
; 
; CALLING SEQUENCE: 
;      photcheck, filename, [path, extend=extend, layer=layer, box=box, 
;           notscl=notscl, subim=subim, stat=stat, help=help]
; 
; INPUTS:
;      FILENAME - name of FITS file
; 
; OPTIONAL INPUTS: 
;      PATH - Directory where filename is located (default is current)
; 
; OPTIONAL KEYWORD INPUTS: 
;      EXTEND - Set equal to FITS file extension number (default is PDU)
;      LAYER - Set to 3rd dimension value for image cubes
;      BOX - Two element integer vector with x and y dimensions of the box 
;            surrounding the cursor.  (default is 5 x 5)  
;            Values need to be odd or photcheck will add 1 to value.
;      NOTSCL - Does not scale image display (tv used, rather than tvscl)
;      SUBIM - Limits of subimage to display [min_x, max_x, min_y,
;              max_y]
;      STAT - Set to print image statistics (sky level, noise
;             st. dev., rms SNR, and max SNR)
;      SAVE - Set to save image with cursor location in header
;             Can set to 1 to overwrite original or set to new filename.
;      HELP - Set to nonzero scalar to have photcheck print syntax
; 
; OUTPUT: 
;      Image values in a box surrounding the cursor are printed to the
;      IDL command line along with the cursor position.  If /stat then
;      image statistics will be printed.
; 
; EXTERNAL CALLS: 
;      GETIM, IM_STATS
; 
; COMMON BLOCKS: 
;      NONE
; 
; MODIFICATION HISTORY: 
;      - Written by Eric Volquardsen, NASA IRTF, Dec. 2002
;      - Added SAVE keyword, ELV, April 2004
; 
; ===================================================================
;                         EVENT HANDLER MODULE
;                        ----------------------
PRO photcheck_event, event

; SEPARATE EVENTS
     widget_control, event.id, get_uvalue=val
     widget_control, event.top, get_uvalue=pcom
     CASE val OF 

; -----CURSOR ACTION-----
     
          'cur' : BEGIN

; GET CURSOR LOCATION AND VALUES
               IF event.press EQ 1 THEN BEGIN
                    bc = pcom.bc
                    pimage = pcom.pimage
                    box = pcom.box
                    bx = box[0]/2
                    by = box[1]/2
                    isz = pcom.isz
                    x1 = (event.x - bx + bc[0]) >0 
                    x2 = (event.x + bx + bc[0]) <(isz[0]-1)
                    y1 = (event.y - by + bc[1]) >0 
                    y2 = (event.y + by + bc[1]) <(isz[1]-1)
                    cxy = string((event.x+bc[0]), (event.y+bc[1]))

; PRINT LOCATION AND VALUES
                    print, 'CURSOR LOCATION ', cxy
                    print, pimage[x1:x2, y1:y2]
                    IF pcom.stat EQ 1 THEN BEGIN
                         print, '  '

; PRINT STATS (if desired)
                         ss = IM_STATS(pimage, /print_stats)
                    ENDIF

; SAVE LOCATION (if desired)
                    save = pcom.save
                    old = pcom.old
                    path = ''
                    IF save NE '0' THEN BEGIN 
                         GETIM, old, img, path, hdr = hdr
                         FXADDPAR, hdr, 'LOCATION', cxy
                         FXWRITE, save, hdr, img
                    ENDIF 

               ENDIF
               widget_control, event.top, set_uvalue=pcom

          END   ; cursor events


; -----EXIT BUTTON-----

          'ext' : BEGIN

               widget_control, event.top, /destroy

          END   ; exit button

     ENDCASE   ; event separation

END

; =====================================================================
;                WIDGET DEFINITION/IMAGE DISPLAY MODULE
;               ----------------------------------------
PRO photcheck, filename, path, extend=extend, layer=layer, box=box, $ 
     notscl=notscl, subim=subim, stat=stat, save = save, help=help

; CHECK INPUT
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN help = 1 
     ENDIF ELSE help = 0 
     IF n_elements(filename) EQ 0 THEN help = 1 
     IF help EQ 1 THEN BEGIN
          print, '***************************************************'
          print, '***              PHOTCHECK SYNTAX               ***' 
          print, '*** > photcheck, filename, path, extend=extend, ***' 
          print, '***     layer=layer, box=box, notscl=notscl,    ***' 
          print, '***     subim=subim, stat=stat                  ***' 
          print, '***                                             ***' 
          print, '*** filename- name of FITS file                 ***' 
          print, '*** path- directory where filename is located   ***' 
          print, '***       default is current directory          ***'
          print, '*** extend- set equal to FITS file extension #  ***' 
          print, '***         default is primary data unit        ***' 
          print, '*** layer- Set to 3rd dimension for image cubes ***' 
          print, '*** box- two element integer vector with x and  ***' 
          print, '***      y dimensions of the box surrounding    ***' 
          print, '***      the cursor. ( default is 5 x 5 )       ***' 
          print, '***      Even values will have 1 added to them  ***' 
          print, '*** notscl- does not scale image display        ***' 
          print, '***         (tv used, rather than tvscl)        ***' 
          print, '*** subim- Limits of subimage to display        ***' 
          print, '***        [min_x, max_x, min_y, max_y]         ***'
          print, '*** stat- Set to print image statistics         ***' 
          print, '*** save- Set to save cursor position to hdr    ***'
          print, '***************************************************' 
          RETURN
     ENDIF 
     IF n_elements(path) NE 1 THEN BEGIN
          cd, '/', current=path
          cd, path
     ENDIF ELSE cd, path
     IF n_elements(notscl) NE 1 THEN notscl = 0 
     IF notscl NE 0 THEN notscl = 1 
     IF n_elements(extend) EQ 0 THEN extend = 0 
     IF n_elements(layer) NE 1 THEN layer = 0 
     IF n_elements(box) LT 1 THEN box = [5, 5]
     IF n_elements(box) EQ 1 THEN BEGIN
          box = [box, box]     
     ENDIF ELSE box = box[0:1]
     IF n_elements(stat) EQ 1 THEN BEGIN
          IF stat NE 0 THEN stat = 1 
     ENDIF ELSE STAT = 0 
     IF n_elements(save) EQ 1 THEN BEGIN 
          svsz = size(save)
          IF svsz[1] EQ 2 THEN BEGIN 
               IF save NE 0 THEN BEGIN 
                    save = path + filename
               ENDIF ELSE save = '0'
          ENDIF 
     ENDIF ELSE save = '0'
     old = path + filename

; LOAD IMAGE
     GETIM, filename, pimage, path, extend=extend, layer=layer
     IF n_elements(pimage) EQ 1 THEN RETURN

; CHECK IMAGE PARAMETERS
     pimage = float(pimage)
     isz = size(pimage)
     IF isz[0] EQ 3 THEN pimage = pimage[*,*,layer]
     isz = size(pimage, /dimensions)
     IF n_elements(subim) NE 4 THEN subim = [0,(isz[0]-1),0,(isz[1]-1)]

; BASE WIDGETS
     IF isz[0] GT 190 THEN xss = (subim[1]-subim[0]+1) ELSE xss = 190
     checkbs = widget_base(title=filename, xsize=xss, $  
          ysize=(subim[3]-subim[2]+35), /column, /align_center)
     dispbas = widget_base(checkbs, /frame, /align_center)
     exitbas = widget_base(checkbs, /align_center)

; OTHER WIDGETS
     dispwin = widget_draw(dispbas, xsize=(subim[1]-subim[0]+1), $ 
          ysize=(subim[3]-subim[2]), uvalue='cur', /button_events)
     exitbtn = widget_button(exitbas, uvalue='ext', $ 
          value='EXIT') 

; COMMON BLOCK
     bc = [subim[0], subim[2]]
     pcom = { pimage:pimage, isz:isz, box:box, bc:bc, stat:stat, $ 
              save:save, old:old }

; ACTIVATE WIDGETS
     widget_control, checkbs, set_uvalue=pcom, /no_copy
     widget_control, checkbs, /realize
     widget_control, dispwin, get_value=wid1
     wset, wid1
     IF notscl EQ 1 THEN tv, pimage[subim[0]:subim[1], subim[2]:subim[3]] $ 
          ELSE tvscl, pimage[subim[0]:subim[1], subim[2]:subim[3]] 
     xmanager, 'photcheck', checkbs

END
