; NAME: 
;      QUICKEX
; 
; PURPOSE: 
;      Quick and dirty 1D spectrum extraction.
; 
; CALLING SEQUENCE: 
;      quickex, infile, outfile, paths, [plt=plt, ap=ap, flp=flp, help=help]
; 
; INPUTS: 
;      infile - Filename(s) of 2D spectrum
;      paths - Two element string vector with input and output paths
; 
; OPTIONAL KEYWORD INPUTS: 
;      plt - Set to nonzero scalar to print results to screen
;      ap - Two element vector with minimum and maximum aperture values
;      flp - Set to nonzero scalar for b-beam case
;      help - Set to nonzero scalar to print syntax
; 
; OUTPUT: 
;      outfile - Filename of extracted spectrum
; 
; EXTERNAL CALLS: 
;      FILEX, GETIM, SDEV
; 
; COMMON BLOCKS: 
;      NONE 
; 
; MODIFICATION HISTORY: 
;      - Written by Eric Volquardsen, NASA IRTF, August 2003
; 
; ====================================================================
PRO quickex, infile, outfile, paths, plt=plt, ap=ap, flp=flp, help=help

; CHECK INPUTS
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN help = 1 
     ENDIF ELSE help = 0 
     IF n_elements(infile) LT 1 THEN help = 1 
     IF n_elements(outfile) NE 1 THEN help = 1 
     IF help EQ 1 THEN BEGIN
          print, '************************************************'
          print, '***              QUICKEX SYNTAX              ***'
          print, '*** > quickex, infile, outfile, paths,       ***'
          print, '***            plt=plt, ap=ap, flp=flp       ***'  
          print, '***                                          ***' 
          print, '*** infile - Filename of 2D spectrum         ***' 
          print, '*** outfile - Filename of extracted spectrum ***'
          print, '*** paths - Two element string vector with   ***' 
          print, '***         input and output paths           ***'
          print, '*** plt - Set to nonzero scalar to print     ***' 
          print, '***       results to screen                  ***'
          print, '*** ap - Two element vector with minimum and ***' 
          print, '***      maximum aperture values             ***' 
          print, '*** flp - Set to nonzero scalar for B-beam   ***'
          print, '************************************************' 
          RETURN
     ENDIF
     nep = n_elements(paths)
     CASE nep OF 
          0 : BEGIN
               v = !version.os_family
               CASE v OF
                    'vms' : root = '[]'
                    'windows' : root = 'c:\'
                    ELSE : root = '/'
               ENDCASE
               cd, root, current=nowp
               paths = [nowp, nowp]
               cd, nowp
          END 
          1 : paths = [paths, paths] 
          ELSE : paths = paths[0:1]
     ENDCASE 
     IF n_elements(plt) EQ 1 THEN BEGIN
          IF plt NE 0 THEN plt = 1 
     ENDIF ELSE plt = 0 
     IF n_elements(ap) NE 2 THEN ap = [45, 55]
     IF n_elements(flp) EQ 1 THEN BEGIN
          IF flp NE 0 THEN flp = 1 
     ENDIF ELSE flp = 0 

; GET INPUT SPECTRUM
     nei = n_elements(infile)
     FOR j = 0, (nei-1) DO BEGIN
          ck = filex(infile[j], paths[0])
          IF ck EQ 0 THEN BEGIN
               print, infile[j], 'DOES NOT EXIST'
               RETURN
          ENDIF
          getim, infile[j], f, paths[0]
          IF j EQ 0 THEN f1 = f ELSE f1 = f1 + f
     ENDFOR

; EXTRACT 1D SPECTRUM
     s1 = total(f1[*, ap[0]:ap[1]], 2)
     sz = size(f1, /dimensions)
     linn = indgen(sz[0])
     IF flp EQ 1 THEN s1 = (-1)*s1

; CALCULATE ERROR
     er = fltarr(sz[0])
     FOR i = 0, (sz[0]-1) DO BEGIN
          sd = sdev(f1[i, 0:ap[0]])
          ss = sd[1]*sd[1]
          er[i] = sqrt(abs(s1[i]))
     ENDFOR

; SAVE RESULTS
     cd, paths[1], current=nowp
     openw, logun, outfile, /get_lun
     result = fltarr(3, sz[0])
     result[0, *] = linn
     result[1, *] = s1
     result[2, *] = er
     printf, logun, result
     free_lun, logun

; PLOT RESULTS (if desired)
     IF plt EQ 1 THEN plot, linn, s1, title='EXTRACTED SPECTRUM', $
                            charsize=1.4

END
 
