; NAME: 
;      REBIN_SPEC
; 
; PURPOSE: 
;      Creates rebinned spectrum from 3 or 4 column spectral table.
; 
; CALLING SEQUENCE: 
;      new_spec = rebin_spec(spec, factor, [offset, flags=flags, help=help])
; 
; INPUTS: 
;      spec - 3 x n array with wavelength, reflectance, and error
;             (n musy be at least 10 and greater than factor*2)
;      factor - Number of lines to combine
; 
; OPTIONAL INPUTS: 
;      offset - Enter shift in microns with positive shift for 
;               increasing wavelength
; 
; OPTIONAL KEYWORD INPUTS: 
;      flags - Set if there is a fourth column of flags.  If one or
;              more of the flags being rebinned are not 1 then the
;              resulting rebinned value is -1.
;      help - Set to nonzero scalar to print syntax
; 
; OUTPUT: 
;      new_spec - 3 (or 4) x n/factor array similar to spec
; 
; EXTERNAL CALLS: 
;      SDEV
; 
; COMMON BLOCKS: 
;      NONE
; 
; MODIFICATION HISTORY: 
;      - Written by Eric Volquardsen, NASA IRTF, April 2004
; 
; =======================================================================
FUNCTION rebin_spec, spec, factor, offset, flags = flags, help = help

; CHECK INPUTS
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN help = 1 
     ENDIF ELSE help = 0 
     IF n_params() LT 2 THEN help = 1
     IF n_elements(factor) NE 1 THEN help = 1
     IF n_elements(spec) LT 30 THEN help = 1 ELSE BEGIN 
          ss = size(spec, /dimension)
          IF n_elements(ss) NE 2 THEN help = 1
     ENDELSE 
     IF help EQ 1 THEN BEGIN
          print, '**********************************************************'
          print, '***                REBIN_SPEC SYNTAX                   ***'  
          print, '*** IDL> new_spec = rebin_spec(spec, factor, offset, $ ***' 
          print, '***                 flags=flags)                       ***' 
          print, '*** spec - Original 3 or 4 x n spectrum                ***'
          print, '*** factor - Number of lines to combine (integer)      ***' 
          print, '*** offset - Enter shift in microns with positive      ***' 
          print, '***          shift for increasing wavelength           ***'
          print, '*** flags - Set if spectrum has a fourth column        ***'
          print, '**********************************************************' 
          RETURN, -1
     ENDIF
     IF n_elements(flags) EQ 1 THEN BEGIN 
          IF flags EQ 0 THEN cols = 3 ELSE cols = 4
     ENDIF ELSE cols = 3
     factor = fix(factor)

; REBIN 
     n = fix(n_elements(spec[0, *])/factor)
     IF n LT 2 THEN RETURN, -1
     new_spec = fltarr(cols, n)
     FOR i = 0, (n-1) DO BEGIN 
          new_spec[0, i] = mean(spec[0, (i*factor):((i+1)*factor-1)])
          region = spec[*, (i*factor):((i+1)*factor-1)]
          IF flags NE 0 THEN BEGIN 
               wf = where(region[3, *] GE 1, good)
               IF good GT 0 THEN BEGIN 
                    region = region[*, wf]
                    wts = 1.0/(region[2, *]^2)
                    new_spec[1, i] = total(wts*region[1, *])/total(wts)
                    new_spec[2, i] = sqrt(1/total(wts)) 
                    new_spec[3, i] = float(fix(mean(region[3, *])))
               ENDIF ELSE BEGIN 
                    new_spec[1:3, i] = [0, 0, -1]
               ENDELSE 
          ENDIF ELSE BEGIN 
               wts = 1.0/(region[2, *]^2)
               new_spec[1, i] = total(wts*region[1, *])/total(wts)
               new_spec[2, i] = sqrt(1/total(wts))
          ENDELSE 
     ENDFOR 

; SHIFT
     IF n_elements(offset) EQ 1 THEN BEGIN 
          wl = new_spec[0, *]
          new_spec[0, *] = new_spec[0, *] + offset
          new_spec[1, *] = interpol(new_spec[1, *], wl, new_spec[0, *], $ 
                                    /spline)
          new_spec[2, *] = interpol(new_spec[2, *], wl, new_spec[0, *], $ 
                                    /spline)
     ENDIF 

     RETURN, new_spec

END
