; NAME: 
;      SPEC_PLOT
; 
; VERSION: 1
; 
; PURPOSE: 
;      Reads ascii file consisting of a spectrum and plots it
; 
; CALLING SEQUENCE: 
;      spec_plot, file, [path=path, print=print, datao=datao, logsc=logsc, 
;                 flag=flag, help=help]
; 
; INPUTS: 
;      FILE - Filename where spectrum is stored.  There should be 3 columns:
;                  1) Line number or wavelength
;                  2) Reflectance or counts
;                  3) Error
; 
; OPTIONAL KEYWORD INPUTS: 
;      PATH - Directory containing FILE (default is current).
;      PRINT - Set to string needed to send plot to printer.
;      LOGSC - Set to nonzero scalar to plot in log scale
;                   1 - x axis in log scale
;                   2 - y axis in log scale
;                   3 - log-log plot
;      FLAG - Set to read fourth column where negative numbers
;             indicate that the value is not to be plotted 
;      HELP- Set to nonzero scalar to have SPEC_PLOT print syntax.
; 
; OUTPUT: 
;      Plot of spectrum to screen (and printer if print is set)
; 
; OPTIONAL KEYWORD OUTPUT:
;      DATAO - Variable name for 3xn (4xn if /flag) array of data.
; 
; EXTERNAL CALLS: 
;      FILEX, READCOL, TVLASER
; 
; COMMON BLOCKS: 
;      NONE
; 
; MODIFICATION HISTORY: 
;      - Written by Eric Volquardsen, NASA IRTF, April 2003
; 
; =========================================================================
PRO spec_plot, file, path=path, print=print, datao=datao, logsc=logsc, $
               flag=flag, help=help

; CHECK INPUTS
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN help = 1 
     ENDIF ELSE help = 0 
     IF n_elements(file) NE 1 THEN help = 1 
     IF n_elements(path) NE 1 THEN BEGIN
          v = !version.os_family
          CASE v OF
               'vms' : root = '[]' 
               'windows' : root = 'c:\' 
               ELSE : root = '/' 
          ENDCASE
          cd, root, current=path
          cd, path
     ENDIF
     IF help NE 1 THEN BEGIN 
          fchk = filex(file, path)
          IF fchk EQ 0 THEN help = 1 
     ENDIF 
     sp = size(path)
     IF sp[0] EQ 1 THEN BEGIN 
          path = path[0]
          IF sp[2] NE 7 THEN help = 1 
     ENDIF ELSE IF sp[1] NE 7 THEN help = 1 
     IF n_elements(print) EQ 1 THEN BEGIN
          spp = size(print)
          IF  spp[0] EQ 0 THEN BEGIN
               IF spp[1] NE 7 THEN print = ''
          ENDIF ELSE print = ''
     ENDIF ELSE print = ''
     IF help EQ 1 THEN BEGIN
          print, '******************************************************'
          print, '***               SPEC_PLOT SYNTAX                 ***'
          print, '*** > spec_plot, file, path=path, datao=datao,     ***' 
          print, '***              print=print, logsc=logsc,         ***' 
          print, '***              flag=flag                         ***'
          print, '***                                                ***' 
          print, '*** FILE - Filename where spectrum is stored.      ***' 
          print, '***        There should be 3 columns:              ***' 
          print, '***                  1) Line or wavelength         ***'
          print, '***                  2) Reflectance or counts      ***' 
          print, '***                  3) Error                      ***' 
          print, '*** PATH - Directory containing FILE               ***' 
          print, '***        (default is current)                    ***' 
          print, '*** PRINT - Set to string to send plot to printer. ***'
          print, '*** DATAO - Variable name for output of data       ***' 
          print, '*** LOGSC - Set to plot in log scale:              ***' 
          print, '***              1 - x axis in log scale           ***' 
          print, '***              2 - y axis in log scale           ***' 
          print, '***              3 - log-log plot                  ***' 
          print, '*** FLAG - Set to have fourth column of flags      ***'
          print, '******************************************************'
          RETURN
     ENDIF
     IF n_elements(logsc) NE 1 THEN logsc = 0 
     
; READ DATA
     cd, path, current=nowp
     IF n_elements(flag) EQ 1 THEN BEGIN
         IF flag NE 0 THEN BEGIN 
              readcol, file, wl, ct, cte, fl
              w = WHERE(fl GE 0)
              wl = wl[w]
              ct = ct[w]
              cte = cte[w]
         ENDIF ELSE BEGIN
              readcol, file, wl, ct, cte
         ENDELSE 
     ENDIF ELSE readcol, file, wl, ct, cte
     datao = [transpose(wl), transpose(ct), transpose(cte)]
     cd, nowp
     mx = MAX(ct, min=mn)

; PLOT DATA
     CASE logsc OF 
          3 : plot, wl, ct, title='Spectrum for '+file, $ 
                    charsize=1.5, /xlog, /ylog, yrange=[mn, mx]
          2 : plot, wl, ct, title='Spectrum for '+file, $ 
                    charsize=1.5, /xlog, yrange=[mn, mx]
          1 : plot, wl, ct, title='Spectrum for '+file, $ 
                    charsize=1.5, /ylog, yrange=[mn, mx]
          ELSE : plot, wl, ct, title='Spectrum for '+file, $ 
                    charsize=1.5, yrange=[mn, mx]
     ENDCASE
     IF print NE '' THEN tvlaser, printer=print, /reverse

END
