; NAME: 
;      SPEX_TO_TABLE
; 
; PURPOSE: 
;      Converts Spextool ascii output file to a file for each 
;      order and aperture (beam).  
; 
; CALLING SEQUENCE: 
;      spex_to_table, infile, outfile, paths, [n_orders, dc=dc, help=help]
; 
; INPUTS: 
;      infile - Filename of spextool output file
;      outfile - Filename root for output
;      paths - Two element string vector with input and output paths
; 
; OPTIONAL INPUTS: 
;      n_orders - Number of orders present in infile
;                 (default is 6)
; 
; OPTIONAL KEYWORD INPUTS: 
;      help - Set to nonzero scalar to print syntax
;      dc - Set to desired separation between lines (in microns) 
;           to perform dispersion correction on all orders.
;           Value must be greater than the existing separation 
;           between wavelengths.  Input spectra should already 
;           have been linearized.
; 
; OUTPUT: 
;      Wavelength, reflectance, and error columns are saved for each
;      aperture and order ( to outfile.a.o1.dat for first order 
;      a-beam, for example).
; 
; EXTERNAL CALLS: 
;      READCOL, WL_LIN
; 
; COMMON BLOCKS: 
;      NONE
; 
; MODIFICATION HISTORY: 
;      - Written by Eric Volquardsen, NASA IRTF, August 2004
; 
; =======================================================================
PRO spex_to_table, infile, outfile, paths, n_orders, dc = dc, help=help

; CHECK INPUTS
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN help = 1 
     ENDIF ELSE help = 0 
     IF n_elements(infile) NE 1 THEN help = 1
     IF n_elements(outfile) NE 1 THEN help = 1
     IF n_elements(paths) NE 2 THEN help = 1
     IF help EQ 1 THEN BEGIN
          print, '***************************************************'
          print, '***           SPEX_TO_TABLE SYNTAX              ***'  
          print, '*** IDL> spex_to_table, infile, outfile, paths, ***'
          print, '***                     n_orders, dc=dc         ***'
          print, '***                                             ***' 
          print, '*** infile - Filename of spextool output file   ***'
          print, '*** outfile - Filename root for output          ***'
          print, '*** paths - Two element string vector with      ***' 
          print, '***         input and output paths              ***'
          print, '*** n_orders - Number of orders present in      ***' 
          print, '***           infile (default is 6)             ***'
          print, '*** dc - Set to desired separation between      ***' 
          print, '***      lines (in microns) to perform          ***' 
          print, '***      dispersion correction on all orders.   ***'
          print, '***************************************************'   
          RETURN
     ENDIF
     IF n_elements(n_orders) NE 1 THEN n_orders = 6
     IF n_elements(dc) EQ 1 THEN BEGIN 
          IF dc LT 0 THEN dc = 0
     ENDIF ELSE dc = 0

; CHECK FOR PROPER FILE
     cd, paths[0], current = nowp
     READCOL, infile, v1, /silent
     IF n_elements(v1) EQ 0 THEN BEGIN
          print, '  '
          print, 'FILE DOES NOT EXIST'
          print, '  '
          RETURN 
     ENDIF 
     n = n_elements(v1)
     
; READ FILE
     junk = ' '
     table = fltarr(6*n_orders, n)
     OPENR, logun, infile, /get_lun
     FOR j = 0, 83 DO BEGIN 
          READF, logun, junk
          junk = strtrim(junk, 2)
          IF junk EQ 'END' THEN j = 82
     ENDFOR 
     READF, logun, table
     free_lun, logun

; LOOP THROUGH ORDERS
     cd, paths[1]
     FOR i = 0, (n_orders-1) DO BEGIN 

; Remove non-finite numbers
          w = finite(table[6*i+1, *], /NaN)
          w = where(w NE 1)
          nw = n_elements(w) - 1

; Perform dispersion correction (if desired)
          tbl_a = table[6*i:6*i+2, w]
          tbl_b = table[6*i+3:6*i+5, w]
          IF dc NE 0 THEN BEGIN 
               result = WL_LIN(tbl_a, tbl_b)
               IF n_elements(result) EQ 1 THEN RETURN
               tbl_a = [result[0, *], result[1, *], result[2, *]]
               tbl_b = [result[0, *], result[3, *], result[4, *]]
          ENDIF 

; Save files
          fa = outfile + '.o' + strtrim(string(i+1), 2) + '.a.dat'
          fb = outfile + '.o' + strtrim(string(i+1), 2) + '.b.dat'
          OPENW, logun, fa, /get_lun
          PRINTF, logun, tbl_a
          free_lun, logun
          OPENW, logun, fb, /get_lun
          PRINTF, logun, tbl_b
          free_lun, logun
     ENDFOR 

END
