; NAME: 
;      STRIKE_OUT
; 
; VERSION: 1
; 
; PURPOSE: 
;      Removes cosmic ray strikes from CCD images
; 
; CALLING SEQUENCE: 
;      strike_out, in_fn, out_fn, thresh, box, [path=path, blink=blink,
;                  exclude=exclude, help=help]
; 
; INPUTS: 
;      in_fn - Filename of image to clean
;      out_fn - filename of cleaned image
;      thresh - Standard deviation threshold to determine bad pixels
;      box - Width of box for determining thresh
; 
; OPTIONAL KEYWORD INPUTS: 
;      path - Input and output paths
;      blink - Set to value for following results
;                   0 : no blinking (default)
;                   1 : blink between original image and removel mask
;                   2 : blink between original and cleaned image
;      exclude - 5 x n array for scanning star locations with
;                different box and thresh parameters  
;                [x, y, radius, thresh, box]    
;      help - Set to nonzero scalar to print syntax
; 
; OUTPUT: 
;      Cleaned image and removal mask are saved in out_fn.  Input and 
;      cleaned images or input and removal mask can be blinked.
; 
; EXTERNAL CALLS: 
;      ACRE, FXADDPAR, FXPAR, FXWRITE, GETIM
; 
; COMMON BLOCKS: 
;      NONE
; 
; MODIFICATION HISTORY: 
;      - Written by Eric Volquardsen, NASA IRTF, October 2003
; 
; =======================================================================
PRO strike_out, in_fn, out_fn, thresh, box, path=path, blink=blink, $ 
                 exclude = exclude, help=help

; CHECK INPUTS
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN help = 1 
     ENDIF ELSE help = 0 
     IF n_elements(in_fn) NE 1 THEN help = 1 
     IF n_elements(out_fn) NE 1 THEN help = 1 
     IF n_elements(thresh) NE 1 THEN help = 1 
     IF n_elements(box) NE 1 THEN help = 1 
     np = n_elements(path)
     CASE np OF 
          0 : BEGIN
               v = !version.os_family
               CASE v OF
                    'vms' : root = '[]'
                    'windows' : root = 'c:\'
                    ELSE : root = '/'
                ENDCASE
                cd, root, current=pat
                cd, pat
                path = [pat, pat]
          END
          1 : path = [path, path]
          ELSE : path = path[0:1]
     ENDCASE
     chk = FILEX(in_fn, path[0])
     IF chk EQ 0 THEN help = 1
     IF help EQ 1 THEN BEGIN
          print, '****************************************************'
          print, '***              STRIKE_OUT SYNTAX               ***'  
          print, '*** IDL> strike_out, in_fn, out_fn, thresh, box, ***'
          print, '***                  path=path, blink=blink,     ***'
          print, '***                  exclude=exclude             ***'
          print, '***                                              ***'
          print, '*** in_fn - Filename of image to clean           ***'
          print, '*** out_fn - Filename of cleaned image and       ***'
          print, '*** thresh - Standard deviation threshold to     ***' 
          print, '***          determine bad pixels                ***'
          print, '*** box - Width of box for determining thresh    ***' 
          print, '*** path - Input and output paths                ***' 
          print, '*** blink - Set to value for following results   ***'
          print, '***            0 : no blinking (default)         ***' 
          print, '***            1 : blink between original image  ***' 
          print, '***                and removel mask              ***' 
          print, '***            2 : blink between original and    ***' 
          print, '***                cleaned image                 ***'
          print, '*** exclude - 3 x n array for area to not use in ***' 
          print, '***           filtering (bright star locations)  ***' 
          print, '***           [x, y, radius]                     ***'
          print, '****************************************************'   
          RETURN
     ENDIF
     IF n_elements(blink) NE 1 THEN blink = 0 
     nx =  n_elements(exclude)

; LOAD IMAGE
     GETIM, in_fn, img1, path[0], hdr=h

; REMOVE COSMIC RAYS
     CASE blink OF
          1 : IF nx NE 3 THEN BEGIN 
                 ACRE, img1, img2, thresh, box, mask=mask, /blmask 
              ENDIF ELSE BEGIN 
                 ACRE, img1, img2, thresh, box, mask=mask
                 ex1 = exclude[0]-exclude[2]
                 ex2 = exclude[0]+exclude[2]
                 ex3 = exclude[1]-exclude[2]
                 ex4 = exclude[1]+exclude[2]
                 mask[ex1:ex2, ex3:ex4] = 0
                 img2[ex1:ex2, ex3:ex4] = img1[ex1:ex2, ex3:ex4]
                 tvscl, mask
                 wait, 2
                 wdelete
              ENDELSE 
          2 : IF nx NE 3 THEN BEGIN 
                 ACRE, img1, img2, thresh, box, mask=mask, /blfinal 
              ENDIF ELSE BEGIN 
                 ACRE, img1, img2, thresh, box, mask=mask
                 ex1 = exclude[0]-exclude[2]
                 ex2 = exclude[0]+exclude[2]
                 ex3 = exclude[1]-exclude[2]
                 ex4 = exclude[1]+exclude[2]
                 mask[ex1:ex2, ex3:ex4] = 0
                 img2[ex1:ex2, ex3:ex4] = img1[ex1:ex2, ex3:ex4]
                 tvscl, img2
                 wait, 3
                 wdelete
              ENDELSE 
          ELSE : IF nx NE 3 THEN BEGIN 
                    ACRE, img1, img2, thresh, box, mask=mask  
                 ENDIF ELSE BEGIN 
                    ACRE, img1, img2, thresh, box, mask=mask
                    ex1 = exclude[0]-exclude[2]
                    ex2 = exclude[0]+exclude[2]
                    ex3 = exclude[1]-exclude[2]
                    ex4 = exclude[1]+exclude[2]
                    mask[ex1:ex2, ex3:ex4] = 0
                    img2[ex1:ex2, ex3:ex4] = img1[ex1:ex2, ex3:ex4]
                 ENDELSE 
     ENDCASE

; SAVE RESULTS
     szx = float(FXPAR(h, 'NAXIS1'))
     szy = float(FXPAR(h, 'NAXIS2')) 
     outpt = fltarr(szx, szy, 2)
     outpt[*,*,0] = img2
     outpt[*,*,1] = mask
     comm = 'Cosmic ray removal ' + strtrim(string(thresh), 2) + $ 
            '/' + strtrim(string(box), 2) + ' (sdev/box width)'
     FXADDPAR, h, 'CR_REMV', comm
     FXADDPAR, h, 'NAXIS', 3
     FXADDPAR, h, 'NAXIS3', 2
     cd, path[1], current=nowp
     FXWRITE, out_fn, h, outpt
     cd, nowp

END
