; NAME: 
;      SUBSPEC
; 
; PURPOSE: 
;      Extracts a subregion of a spectrum
; 
; CALLING SEQUENCE: 
;      new_spec = subspec(old_spec, stwl, nl, [center=center, help=help])
; 
; INPUTS: 
;      old_spec - Spectrum from which to extract subregion
;                 (m x n array with m > 1 and n > 3)
;      stwl - Starting wavelength (in microns).  If the listed 
;             wavelength does not exist, then the wavelength 
;             following it will be chosen.
;      nl - Number of lines to extract
; 
; OPTIONAL KEYWORD INPUTS: 
;      center - If this keyword is set, then subspec will extract 
;               nl lines to the right and left of stwl.
;      help - Set to nonzero scalar to print syntax
; 
; OUTPUT: 
;      new_spec - Extracted spectrum
; 
; EXTERNAL CALLS: 
;      NONE 
; 
; COMMON BLOCKS: 
;      NONE
; 
; MODIFICATION HISTORY: 
;      - Written by Eric Volquardsen, NASA IRTF, April 2004
; 
; =======================================================================
FUNCTION subspec, old_spec, stwl, nl, center = center, help = help

; CHECK INPUTS
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN help = 1 
     ENDIF ELSE help = 0 
     IF n_params() LT 3 THEN help = 1
     IF n_elements(stwl) NE 1 THEN help = 1
     IF n_elements(nl) NE 1 THEN help = 1
     IF n_elements(old_spec) GT 12 THEN BEGIN 
          ss = size(old_spec,  /dimension)
          IF n_elements(ss) LT 2 THEN help = 1
     ENDIF ELSE help = 1
     IF help EQ 1 THEN BEGIN
          print, '***************************************************'
          print, '***               SUBSPEC SYNTAX                ***'  
          print, '*** IDL> new_spec = subspec(old_spec, stwl, nl, ***' 
          print, '***                 center=center)              ***' 
          print, '***                                             ***' 
          print, '*** old_spec - Spectrum from which to extract   ***'
          print, '*** stwl - Starting wavelength (in microns)     ***'
          print, '*** nl - Number of lines to extract             ***'
          print, '*** center - Set to go nl lines to the right    ***' 
          print, '***          and left of stwl                   ***' 
          print, '***************************************************'   
          RETURN, -1
     ENDIF
     IF n_elements(center) EQ 1 THEN BEGIN 
          IF center NE 0 THEN center = 1
     ENDIF ELSE center = 0

; FIND STARTING WAVELENGTH
     w = where(old_spec[0, *] GE stwl)
     IF w[0] EQ -1 THEN BEGIN 
          print, 'STARTING WAVELENGTH IS OUTSIDE SPECTRUM'
          RETURN, -1
     ENDIF ELSE w = min(w)

; ADJUST VALUES IF CENTER IS SET
     IF center EQ 1 THEN BEGIN 
          w = (w - nl) > 0
          nl = 2*nl+1
     ENDIF 

; CHECK FOR ENDING WAVELENGTH
     IF (w+nl) GT ss[1] THEN nl = ss[1]-w-1
     el = w + nl - 1
     
; EXTRACT SUB-SPECTRUM
     new_spec = old_spec[*, w:el]

     RETURN, new_spec

END
