; NAME:
;      SUPERSMOOTH
; 
; VERSION: 1.1
;
; PURPOSE:
;      To smooth spectra noise via an interactive smoothing spline.
;
; CALLING SEQUENCE:
;      supersmooth, dcol, help = help
;
; INPUTS:
;      1) Inputs that should be in an input file entered by user are:
;                -Wavelength column
;                -Reflectance column
;                -Reflectance error column
;         If the error column doesnt exist then it is assumed to be 0.02
;      2) Output wavelengths may be in a file, designated by bounds and
;         increments, or input wavelengths above.
;      3) Smoothness parameter is entered in a field.  Zero yields
;         original values.
;
; OPTIONAL INPUTS:
;      dcol- A three element array giving the column numbers for
;            wavelength, reflectance, and reflectance error.
;            (default is [0,1,2]).  Put -1 for error column if it 
;            does not exist.
;      help- Set to nonzero numeric value to have SUPERSMOOTH print syntax.
;
;
; OUTPUT:
;      Wavelength and fitted reflectance are saved to a file designated
;      by the user after call or .ss is added to input file name.
;
; EXTERNAL CALLS:
;      COYOTE_FIELD, ERNOTE, READCOL, SPLINESM
;
; COMMON BLOCKS:
;      NONE
;
; MODIFICATION HISTORY:
;      - Written by Eric Volquardsen, NASA IRTF, March 2003
;      - Added ability to use spectra without errors, ELV, November 2004
;
; ====================================================================
;                       EVENT HANDLER MODULE
;                      ----------------------
PRO supersmooth_event, event

; GET EVENT AND DATA STRUCTURE INFO
     widget_control, event.id, get_uvalue=val
     widget_control, event.top, get_uvalue=dst

; SEPARATE EVENTS
     CASE val OF

; -----LOAD BUTTON-----

          'lbt' : BEGIN

; Get filename and path
               dfile = dst.dfile
               IF dfile EQ '' THEN BEGIN
                    ernote, '      ENTER DATA FILE PATH/NAME     ', $ 
                         event.top
                    RETURN
               ENDIF
               osf = !version.os_family
               CASE osf OF
                    'vms' : delim = ']'
                    'windows' : delim = '\'
                    ELSE : delim = '/'
               ENDCASE ; operating type separation
               slen = strlen(dfile)
               id = rstrpos(dfile, delim)
               file = strmid(dfile, id+1, slen)
               path = strmid(dfile, 0, id+1)

; Initialize values
              dcol = dst.dcol
              imaged = dst.imaged

; Read data 
              cd, path
              format = ''
              IF dcol[0] GT 0 THEN $
                   FOR i = 0, (dcol[0]-1) DO format = format + 'x, '
              format = format + 'f, '
              IF (dcol[1]-dcol[0]) GE 2 THEN $ 
                   FOR i = 0, (dcol[1]-dcol[0]-1) DO format = format + 'x, '
              format = format + 'f'
              IF dcol[2] NE -1 THEN BEGIN 
                   IF (dcol[2]-dcol[1]) GE 2 THEN format = format + ', x'
                   format = format + ', f'
                   READCOL, file, wavel, refl, rerr, format=format
              ENDIF ELSE BEGIN 
                   READCOL, file, wavel, refl, format=format
                   rerr = fltarr(n_elements(wavel))+0.02
              ENDELSE

; Plot data in draw window
             widget_control, imaged, get_value=wid
             wset, wid
             mx = max(refl, min=mn)
             xmx = max(wavel, min=xmn)
             rn = mx - mn
             xrn = xmx - xmn
             ofs = 0.25*rn
             xofs = 0.25 *xrn
             mx = mx + ofs
             mn = mn - ofs
             xmx = xmx + xofs
             xmn = xmn - xofs
             plot, wavel, refl, yrange=[mn, mx], xrange=[xmn, xmx], psym=1

; Return data structure
             IF n_elements(size(wavel, /dimension)) NE 2 THEN BEGIN 
                  wavel = TRANSPOSE(wavel)
                  refl = TRANSPOSE(refl)
                  rerr = TRANSPOSE(rerr)
             ENDIF 
             *dst.wavel = wavel
             *dst.refl = refl
             *dst.rerr = rerr
             widget_control, event.top, set_uvalue=dst, /no_copy

         END ; load button

; -----RUN BUTTON-----

         'rbt' : BEGIN

; Get data structure values
              smooth = dst.smooth
              imaged = dst.imaged
              wavel = *dst.wavel
              refl = *dst.refl
              rerr = *dst.rerr
              wfile = dst.wfile

; Check inputs
              ww = where(wavel NE 0)
              IF n_elements(ww) LT 2 THEN BEGIN
                   msg = '       NO DATA LOADED '
                   ernote, msg, event.top
                   widget_control, event.top, set_uvalue=dst, /no_copy
                   RETURN
              ENDIF

; Determine which wavelengths to feed into spline fit
              IF wfile EQ '' THEN BEGIN
                   swave = dst.swave
                   ewave = dst.ewave
                   incr = dst.incr
                   IF ewave EQ 0.0 THEN wavelfit = wavel ELSE BEGIN
                        szi = ((ewave - swave) / incr) + 1
                        wavelfit = findgen(szi)*incr + swave
                   ENDELSE
              ENDIF ELSE BEGIN
                   wfile = dst.wfile
                   osf = !version.os_family
                   CASE osf OF
                        'vms' : delim = ']'
                        'windows' : delim = '\'
                        ELSE : delim = '/'
                   ENDCASE
                   slen = strlen(wfile)
                   id = rstrpos(wfile, delim)
                   file = strmid(wfile, id+1, slen)
                   path = strmid(wfile, 0, id+1)
                   wavelfit = [0.0]
                   data = [0.0]
                   cd, path
                   openr, logun, file, /get_lun
                   IF EOF(logun) NE 1 THEN readf, logun, data ELSE BEGIN
                        ernote, '      NO DATA IN FILE      ', event.top
                        RETURN
                   ENDELSE
                   wavelfit = data
                   WHILE (NOT EOF(logun)) DO BEGIN
                        readf, logun, data
                        boost_array, wavelfit, data
                   ENDWHILE
                   free_lun, logun
              ENDELSE

; Do spline fit to data
              splinesm, wavel, refl, rerr, smooth, wavelfit, yfit
              IF n_elements(yfit) EQ 1 THEN RETURN

; Plot results
              widget_control, imaged, get_value=wid
              wset, wid
              mx = max(refl, min=mn)
              xmx = max(wavel, min=xmn)
              rn = mx - mn
              xrn = xmx - xmn
              ofs = 0.25*rn
              xofs = 0.1 *xrn
              mx = mx + 4*ofs
              mn = mn - 0.1*ofs
              xmx = xmx + xofs
              xmn = xmn - xofs
              plot, wavel, refl, yrange=[mn, mx], xrange=[xmn, xmx], psym=1
              oplot, wavelfit, (yfit+(1.5)*ofs)
              oplot, wavel, (refl+(3*ofs)), psym=1
              oplot, wavelfit, (yfit+(3*ofs)), color=250, thick=2

; Return data structure
             *dst.yfit = yfit
             *dst.wavelfit = wavelfit
             widget_control, event.top, set_uvalue=dst, /no_copy

         END ; run button

; -----SAVE BUTTON-----

         'sbt' : BEGIN

; Get filename and path
              sfile = dst.sfile
              IF sfile EQ '' THEN BEGIN
                   dfile = dst.dfile
                   osf = !version.os_family
                   CASE osf OF
                        'vms' : delim = ']'
                        'windows' : delim = '\'
                        ELSE : delim = '/'
                   ENDCASE
                   slen = strlen(dfile)
                   id = rstrpos(dfile, delim)
                   file = strmid(dfile, id+1, slen)
                   path = strmid(dfile, 0, id+1)
                   file = file + '.ss'
              ENDIF ELSE BEGIN
                   osf = !version.os_family
                   CASE osf OF
                        'vms' : delim = ']'
                        'windows' : delim = '\'
                        ELSE : delim = '/'
                   ENDCASE
                   slen = strlen(sfile)
                   id = rstrpos(sfile, delim)
                   file = strmid(sfile, id+1, slen)
                   path = strmid(sfile, 0, id+1)
              ENDELSE

; Get data to be saved
              wavelfit = *dst.wavelfit
              yfit = *dst.yfit
              s = max(size(wavelfit, /dimensions))

; Check results to be saved
              ww = where(wavelfit NE 0)
              IF n_elements(ww) LT 2 THEN BEGIN
                   msg = '       NO DATA LOADED '
                   ernote, msg, event.top
                   widget_control, event.top, set_uvalue=dst, /no_copy
                   RETURN
              ENDIF

; Save spline fit results
              cd, path
              openw, logun, file, /get_lun
              FOR i = 0, (s-1) DO printf, logun, wavelfit[i], yfit[i]
              free_lun, logun

              widget_control, event.top, set_uvalue=dst, /no_copy

         END ; save button

; -----EXIT BUTTON-----

         'ext' : BEGIN

               ptr_free, dst.wavel, dst.refl, dst.rerr
               ptr_free, dst.wavelfit, dst.yfit
               widget_control, event.top, /destroy

         END ; exit button

; -----DATA FILENAME FIELD-----

         'dfd' : BEGIN

              dfile = dst.dfile
              widget_control, event.id, get_value=pv
              dfile = (*pv)
              dst.dfile = dfile
              widget_control, event.top, set_uvalue=dst, /no_copy

          END ; data filename field

; -----DATA FILENAME BROWSE BUTTON-----

          'dbw' : BEGIN

; Use native file browser to select path
               br = dialog_pickfile(filter='*', /read)

; Put path in data structure and corresponding field
               dfile = dst.dfile
               dfile = br
               dst.dfile = dfile
               widget_control, event.handler, set_value=br
               widget_control, event.top, set_uvalue=dst, /no_copy

          END ; data browse button

; -----SAVE FILENAME FIELD-----

          'sfd' : BEGIN

               sfile = dst.sfile
               widget_control, event.id, get_value=pv
               sfile = (*pv)
               dst.sfile = sfile
               widget_control, event.top, set_uvalue=dst, /no_copy

          END ; save filename field

; -----SAVE FILENAME BROWSE BUTTON-----

          'sbw' : BEGIN

; Use native file browser to select path
               br = dialog_pickfile(filter='*', /read)

; Put path in data structure and corresponding field
               sfile = dst.sfile
               sfile = br
               dst.sfile = sfile
               widget_control, event.handler, set_value=br
               widget_control, event.top, set_uvalue=dst, /no_copy

          END ; save browse button

; -----WAVE FILENAME FIELD-----

          'wfd' : BEGIN

               wfile = dst.wfile
               widget_control, event.id, get_value=pv
               wfile = (*pv)
               dst.wfile = wfile
               widget_control, event.top, set_uvalue=dst, /no_copy

          END ; wave filename field

; -----WAVE FILENAME BROWSE BUTTON-----

          'wbw' : BEGIN

; Use native file browser to select path
               br = dialog_pickfile(filter='*', /read)

; Put path in data structure and corresponding field
               wfile = dst.wfile
               wfile = br
               dst.wfile = wfile
               widget_control, event.handler, set_value=br
               widget_control, event.top, set_uvalue=dst, /no_copy

          END ; wave browse button

; -----SMOOTHING PARAMETER FIELD-----

          'smp' : BEGIN

              smooth = dst.smooth
              widget_control, event.id, get_value=pv
              smooth = (*pv)
              dst.smooth = smooth
              widget_control, event.top, set_uvalue=dst, /no_copy


          END ; smoothing parameter

; -----STARTING WAVELENGTH FIELD-----

         'stw' : BEGIN

              swave = dst.swave
              widget_control, event.id, get_value=pv
              swave = (*pv)
              dst.swave = swave
              widget_control, event.top, set_uvalue=dst, /no_copy

         END ; starting wavelength field

; -----ENDING WAVELENGTH FIELD-----

         'ewv' : BEGIN

              ewave = dst.ewave
              widget_control, event.id, get_value=pv
              ewave = (*pv)
              dst.ewave = ewave
              widget_control, event.top, set_uvalue=dst, /no_copy


         END ; ending wavelength field

; -----INCREMENT FIELD-----

         'inc' : BEGIN

              incr = dst.incr
              widget_control, event.id, get_value=pv
              incr = (*pv)
              dst.incr = incr
              widget_control, event.top, set_uvalue=dst, /no_copy


         END ; increment field

; -----DISPLAY WIDGET-----

         'dis' : BEGIN

; Get data structure values
              textwd = dst.textwd
              tv = dst.tv

; Get cursor position and report
              IF event.press EQ 1 THEN BEGIN
                   wavel = dst.wavel
                   x = event.x
                   y = event.y
                   coord = convert_coord(x, y, /device, /to_data)
                   cx = strtrim(string(coord[0]), 1)
                   cy = strtrim(string(coord[1]), 1)
                   tv = tv + 'wavelength=' + cx
                   tv = tv + ' / reflectance=' + cy
                   widget_control, textwd, set_value=tv
              ENDIF

         END   ; display widget

     ENDCASE ; event separation

END ; event handler

; ====================================================================
;                     WIDGET DEFINITION MODULE
;                    --------------------------
PRO supersmooth, dcol, help=help

; CHECK INPUTS
     IF n_elements(help) NE 0 THEN BEGIN
          IF help NE 0 THEN help = 1
     ENDIF ELSE help = 0
     IF n_elements(dcol) NE 3 THEN dcol = [0, 1, 2]
     IF help EQ 1 THEN BEGIN
          print, '******************************************************'
          print, '***               SUPERSMOOTH SYNTAX               ***'
          print, '*** > supersmooth, dcol                            ***'
          print, '***                                                ***'
          print, '*** dcol- A three element array giving the column  ***'
          print, '***       numbers for wavelength, reflectance, and ***'
          print, '***       reflectance error. (default is [0,1,2])  ***'
          print, '******************************************************'
          RETURN
     ENDIF

; DATA STRUCTURE
     tv = '  CURSOR DISPLAY:'   ; Display value for text widget
     textwd = 0                 ; Text widget ID
     dfile = ''                 ; Filename and path where data located
     sfile = ''         ; Where to save results
     wfile = ''         ; Where evaluation wavelengths are saved
     smooth = 0.0       ; Smoothness parameter
     swave = 0.0        ; Starting evaluation wavelength
     ewave = 0.0        ; Ending evaluation wavelength
     incr = 0.0         ; Dist. between each eval. wavelength
     imaged = 0         ; Draw window ID
     wavel = [0.0]      ; Wavelength vector (1xN)
     refl = [0.0]       ; Reflectance vector (1xN)
     rerr = [0.0]       ; Reflectance Error vector (1xN)
     wavelfit = [0.0]   ; Wavelengths of smoothed data (1xM)
     yfit = [0.0]       ; Fitted reflectances (1xM)
                        ; N = number of points in original data
                        ; M = number of points in output
     dst = { dfile:dfile, sfile:sfile, smooth:smooth, swave:swave, $
          ewave:ewave, wfile:wfile, incr:incr, imaged:imaged, $
          wavel:ptr_new(wavel), refl:ptr_new(refl), rerr:ptr_new(rerr), $
          wavelfit:ptr_new(wavelfit), yfit:ptr_new(yfit), dcol:dcol, $
          tv:tv, textwd:textwd }

; BASE WIDGETS
     smbase = widget_base(title='S U P E R S M O O T H ', $
          /column)
     butnbs = widget_base(smbase, /row, frame=5)
     subbas = widget_base(smbase, /row)
     subbs1 = widget_base(subbas, /column)
     fldbas = widget_base(subbs1, frame=8, /column)
     parbas = widget_base(subbas, frame=8, /column, /align_right)
     imgbas = widget_base(smbase, frame=8, /column)

; MENU BUTTONS
     loadbt = widget_button(butnbs, value=' LOAD ', uvalue='lbt')
     runbtn = widget_button(butnbs, value=' RUN ', uvalue='rbt')
     savebt = widget_button(butnbs, value=' SAVE ', uvalue='sbt')
     exitbt = widget_button(butnbs, value=' EXIT ', uvalue='ext')

; FILENAME FIELDS
     datafd = coyote_field(fldbas, uvalue='dfd', /cr_only, $
          value=dst.dfile, event_pro='supersmooth_event', $
          title='Data File', scr_xsize=300)
     dtabrw = widget_button(datafd, value='BROWSE', uvalue='dbw')
     savefd = coyote_field(fldbas, uvalue='sfd', /cr_only, $
          value=dst.sfile, event_pro='supersmooth_event', $
          title='Save File', scr_xsize=300)
     svbrws = widget_button(savefd, value='BROWSE', uvalue='sbw')
     wavefd = coyote_field(fldbas, uvalue='wfd', /cr_only, $
          value=dst.wfile, event_pro='supersmooth_event', $
          title='Wave File', scr_xsize=300)
     wvbrws = widget_button(wavefd, value='BROWSE', uvalue='wbw')

; PARAMETER FIELDS
     smparf = coyote_field(parbas, uvalue='smp', /cr_only, $
          value=dst.smooth, title='Smoothness Parameter', $
          event_pro='supersmooth_event', scr_xsize=110)
     stwave = coyote_field(parbas, uvalue='stw', /cr_only, $
          value=dst.swave, title=' Starting Wavelength', $
          event_pro='supersmooth_event', scr_xsize=110)
     endwav = coyote_field(parbas, uvalue='ewv', /cr_only, $
          value=dst.ewave, title='   Ending Wavelength', $
          event_pro='supersmooth_event', scr_xsize=110)
     wincrm = coyote_field(parbas, uvalue='inc', /cr_only, $
          value=dst.incr, title=' Wavelengh Increment', $
          event_pro='supersmooth_event', scr_xsize=110)

; DISPLAY AND TEXT WIDGETS
     imaged = widget_draw(imgbas, /button_events, xsize=700, $
          ysize=600, uvalue='dis')
     textwd = widget_text(subbs1, value=tv, ysize=1)

; ACTIVATE WIDGETS
     dst.textwd = textwd
     dst.imaged = imaged
     widget_control, smbase, set_uvalue=dst, /no_copy
     widget_control, smbase, /realize
     xmanager, 'supersmooth', smbase


END ; widget defintion
