; NAME: 
;      TELLCOR
; 
; PURPOSE: 
;      Performs telluric corrections and combines orders of asteroid 
;      and standard star spectra from SpeX in LXD mode.  The 1st, 2nd, 
;      3rd, and 4th orders are used (3.5-4.2 microns, 3-3.6 microns, 
;      2.6-3.1 microns, and 2.2-2.7 microns respectively). 
; 
; CALLING SEQUENCE: 
;      tellcor, asteroid, star, atran, result, record, paths, 
;              [min_wtr=min_wtr, max_wtr=max_wtr, nodiv=nodiv, 
;               noatran=noatran, plt_lim=plt_lim, help=help]
; 
; INPUTS: 
;      asteroid - 8-element string vector with filenames of 
;                 order 1 beam a, order 1 beam b, etc.  
;      star - Similar string vector for the standard star filenames.
;      atran - 2-element string vector with asteroid and standard star
;              atran model filenames
;      result - Filename where resulting spectrum is saved 
;      record - Filename where input parameters are saved 
;      paths - Three element string vector with input path, 
;              atran path, and output path
; 
; OPTIONAL KEYWORD INPUTS: 
;      min_wtr - Set to minimum water amount to be checked.  Negative
;                numbers will set default value.
;                (default is 0.5 mm, minimum is 0.0 mm)
;      max_wtr - Set to maximum water amount to be checked.  Negative 
;                numbers will set default value.
;                (default is 2.0 mm, maximum is 5.9 mm)
;      nodiv - Controls what spectrum is returned
;                0 : object/star (default)
;                1 : object 
;                2 : star 
;      noatran - Set to not perform telluric correction 
;                (just combines orders)
;      plt_lim - Vector of lower and upper bounds for plotting
;      help - Set to nonzero scalar to print syntax
; 
; OUTPUT: 
;      4xn spectrum array is saved to result.  1st column is
;      wavelength, 2nd column is reflectance, 3rd column is error,
;      and 4th column is for flags (indicating rejection by standard
;      deviation). The inputs used are saved to record.
; 
; EXTERNAL CALLS: 
;      APPLY_ATRAN, READ_ATRN, READ_SPEC, SDEV, WL_LIN
; 
; COMMON BLOCKS: 
;      NONE
; 
; MODIFICATION HISTORY: 
;      - Written by Eric Volquardsen, NASA IRTF, April 2004
;      - Adjusted plotting limits and added cross correlation shift
;        when combining A and B beams, ELV, NASA IRTF, March 2005
; 
; =======================================================================
PRO tellcor, asteroid, star, atran, result, record, paths, $ 
             min_wtr = min_wtr, max_wtr = max_wtr, nodiv = nodiv, $ 
             noatran = noatran, plt_lim = plt_lim, help = help

; CHECK INPUTS
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN help = 1 
     ENDIF ELSE help = 0 
     IF n_elements(asteroid) NE 8 THEN help = 1
     IF n_elements(star) NE 8 THEN help = 1
     IF n_elements(atran) NE 2 THEN help = 1
     IF help EQ 1 THEN BEGIN
          print, '***************************************************'
          print, '***              TELLCOR SYNTAX                 ***'  
          print, '*** IDL> tellcor, asteroid, star, atran,        ***' 
          print, '***               result, record, paths,        ***'
          print, '***               min_wtr=miw, max_wtr=maw,     ***'
          print, '***               nodiv=nodiv, noatran=noatran  ***' 
          print, '***                                             ***' 
          print, '*** asteroid - Filenames of asteroid spectra    ***' 
          print, '*** star - Filenames of standard star spectra   ***' 
          print, '*** atran - Filenames of asteroid and star      ***' 
          print, '***         atran files                         ***'
          print, '*** result - Resulting spectrum filename        ***' 
          print, '*** record - Filename where input parameters    ***' 
          print, '***          are saved                          ***' 
          print, '*** paths - Three element string vector with    ***' 
          print, '***         input, atran, and output paths      ***'
          print, '*** min_wtr - Minimum water to check            ***'
          print, '*** max_wtr - Maximum water to check            ***'
          print, '*** nodiv - Controls what spectrum is returned  ***' 
          print, '***           0 : object/star (default)         ***' 
          print, '***           1 : object                        ***' 
          print, '***           2 : star                          ***'
          print, '*** noatran - Set to just combine orders        ***'
          print, '***************************************************'   
          RETURN
     ENDIF
     IF n_elements(result) NE 1 THEN result = 'results.dat'
     IF n_elements(record) NE 1 THEN record = 'record.dat'
     pa = n_elements(paths) 
     v = !version.os_family
     CASE v OF
          'vms' : root = '[]'
          'windows' : root = 'c:\'
          ELSE : root = '/'
     ENDCASE
     CASE pa OF 
          0 : BEGIN
               CASE v OF
                    'vms' : root = '[]'
                    'windows' : root = 'c:\'
                    ELSE : root = '/'
               ENDCASE
               cd, root, current = pat
               cd, pat
               path = [pat, pat, pat]
          END
          1 : paths = [paths, paths, paths]
          2 : paths = [paths[0], paths[0], paths[1]]
          ELSE : paths = paths[0:2]
     ENDCASE
     IF n_elements(min_wtr) EQ 1 THEN BEGIN 
          IF min_wtr LT 0 THEN min_wtr = 0.5
     ENDIF ELSE min_wtr = 0.5
     IF n_elements(max_wtr) EQ 1 THEN BEGIN 
          IF max_wtr GT 5.9 THEN max_wtr = 5.9
          IF max_wtr LT 0 THEN max_wtr = 2.0
     ENDIF ELSE max_wtr = 2.0
     water = [min_wtr, max_wtr]
     IF n_elements(nodiv) NE 1 THEN nodiv = 0
     IF n_elements(noatran) NE 1 THEN noatran = 0
     IF n_elements(plt_lim) EQ 2 THEN BEGIN 
          IF plt_lim[0] GT plt_lim[1] THEN $ 
            plt_lim = [plt_lim[1], plt_lim[0]]
          ymax = plt_lim[1]
          ymin = plt_lim[0]
     ENDIF ELSE plt_lim = [0, -1]
     thresh = 2.55  
     Window, 1, xpos = 650, ypos = 50, xsize = 600, ysize = 420, $ 
             title = 'TELLURIC CORRECTION'


; ------------------ 2nd ORDER -------------------

; LOAD 2nd ORDER SPECTRA FILES
     print, ' '
     print, 'TELLURIC CORRECTION PROGRAM'
     print, ' '
     print, 'Reading 2nd order files'
     a2a = READ_SPEC(asteroid[2], paths[0])
     a2b = READ_SPEC(asteroid[3], paths[0])
     s2a = READ_SPEC(star[2], paths[0])
     s2b = READ_SPEC(star[3], paths[0])

; ADD A AND B BEAM SPECTRA TOGETHER
     n_a =  n_elements(a2a[0, *])*50.0
     n_b =  n_elements(a2b[0, *])*50.0
     wl_a = min(a2a[0, *]) + findgen(n_a)*(a2a[0, 5] - a2a[0, 4])/50.0
     wl_b = min(a2b[0, *]) + findgen(n_b)*(a2b[0, 5] - a2b[0, 4])/50.0
     tempa = interpol(a2a[1, *], a2a[0, *], wl_a, /spline)
     tempb = interpol(a2b[1, *], a2b[0, *], wl_b, /spline)
     lag = findgen(101) - 50 
     ccor = c_correlate(tempa, tempb, lag)
     junk = max(ccor, ix)
     wl_c = a2b[0, *] + lag[ix]*(wl_b[5] - wl_b[4])
     a2b[1, *] = interpol(a2b[1, *], a2b[0, *], wl_c, /spline)
     a2a[1, *] = a2a[1, *] + a2b[1, *]
     s2a[1, *] = s2a[1, *] + s2b[1, *]
     a2a[2, *] = sqrt((a2a[2, *]^2 + a2b[2, *]^2)/2.0)
     s2a[2, *] = sqrt((s2a[2, *]^2 + s2b[2, *]^2)/2.0)

; PLOT ORDER 
     xmax = max(a2a[0, *], min = xmin)
     CASE nodiv OF 
          1 : BEGIN 
               IF plt_lim[0] GT plt_lim[1] THEN BEGIN 
                    dy = SDEV(a2a[1, *])
                    ymax = dy[0] + 10*dy[1]
                    ymin = dy[0] - 10*dy[1]
               ENDIF 
               plot, a2a[0, *], a2a[1, *], $ 
                     yrange = [ymin, ymax], psym = 1, $ 
                     title = '2nd ORDER WITHOUT TELLURIC CORRECTION', $ 
                     xrange = [xmin, xmax]
          END 
          2 : BEGIN 
               IF plt_lim[0] GT plt_lim[1] THEN BEGIN 
                    dy = SDEV(s2a[1, *])
                    ymax = dy[0] + 10*dy[1]
                    ymin = dy[0] - 10*dy[1]
               ENDIF 
               plot, s2a[0, *], s2a[1, *], $ 
                     yrange = [ymin, ymax], psym = 1, $ 
                     title = '2nd ORDER WITHOUT TELLURIC CORRECTION', $ 
                     xrange = [xmin, xmax]
          END 
          ELSE : BEGIN 
               IF plt_lim[0] GT plt_lim[1] THEN BEGIN 
                    dy = SDEV(a2a[1, *]/s2a[1, *])
                    ymax = dy[0] + 10*dy[1]
                    ymin = dy[0] - 10*dy[1]
               ENDIF 
               plot, a2a[0, *], (a2a[1, *]/s2a[1, *]), $ 
                     yrange = [ymin, ymax], psym = 1, $ 
                     title = '2nd ORDER WITHOUT TELLURIC CORRECTION', $ 
                     xrange = [xmin, xmax]
          END 
     ENDCASE 

; LOAD ATRAN FILES
     aatran = READ_ATRN(atran[0], paths[1])
     satran = READ_ATRN(atran[1], paths[1])

; APPLY ATRAN FILES TO SPECTRA
     CASE nodiv OF 
          1 : strg = 'Output is asteroid spectrum only'
          2 : strg = 'Output is standard star spectrum only'
          ELSE : strg = 'Output is asteroid / standard star'
     ENDCASE
     IF noatran EQ 0 THEN BEGIN 
          a2_result = APPLY_ATRAN(a2a, aatran, 2, water, outp = a2vals)
          s2_result = APPLY_ATRAN(s2a, satran, 2, water, outp = s2vals)
     ENDIF ELSE BEGIN 
          a2_result = a2a
          s2_result = s2a
          a2vals = [0.0, 0.0]
          s2vals = [0.0, 0.0]
     ENDELSE 

; FLAG BAD POINTS
     xa = transpose(a2_result[0, *])
     ya = transpose(a2_result[1, *])
     za = transpose(a2_result[2, *])
     xs = transpose(s2_result[0, *])
     ys = transpose(s2_result[1, *])
     zs = transpose(s2_result[2, *])
     flaga = intarr(1, n_elements(a2_result[0, *])) + 1
     flags = intarr(1, n_elements(s2_result[0, *])) + 1

; For asteroid
     gd = where(flaga EQ 1)
     FOR i = 0, 3 DO BEGIN 
          junk = svdfit(xa[gd], ya[gd], 5, yfit = yy, meas = za[gd])
          dif = (ya[gd] - yy) + mean(yy)
          sd = sdev(dif)
          bad = where(abs(dif - sd[0]) GT thresh*sd[1], bnum)
          IF bnum NE 0 THEN flaga[0, gd[bad]] = -1 ELSE i = 4
          gd = where(flaga GT 0)
     ENDFOR
     a2_result = [a2_result, flaga]
     w2 = where(a2_result[0, *] LE 3.6)
     a2_result = a2_result[*, w2]

; For standard star
     gd = where(flags EQ 1)
     FOR i = 0, 3 DO BEGIN 
          junk = svdfit(xs[gd], ys[gd], 5, yfit = yy, meas = zs[gd])
          dif = (ys[gd] - yy) + mean(yy)
          sd = sdev(dif)
          bad = where(abs(dif - sd[0]) GT thresh*sd[1], bnum)
          IF bnum NE 0 THEN flags[0, gd[bad]] = -1 ELSE i = 4
          gd = where(flags GT 0)
     ENDFOR
     s2_result = [s2_result, flags]
     w2 = where(s2_result[0, *] LE 3.6)
     s2_result = s2_result[*, w2]

; LOOK FOR REGIONS OF NO OVERLAP BETWEEN STANDARD AND ASTEROID
     end1 = min(a2_result[0, *])
     end2 = max(a2_result[0, *])
     end3 = min(s2_result[0, *])
     end4 = max(s2_result[0, *])
     IF end1 GT end3 THEN BEGIN 
          endn = n_elements(s2_result[0, *])-1
          endb = WHERE(s2_result[0, *] EQ end1)
          s2_result = s2_result[*, endb:endn]
     ENDIF ELSE BEGIN 
          endn = n_elements(a2_result[0, *])-1
          endb = WHERE(a2_result[0, *] EQ end3)
          a2_result = a2_result[*, endb:endn]
     ENDELSE 
     IF end2 GT end4 THEN BEGIN 
          endb = WHERE(a2_result[0, *] EQ end4)
          a2_result = a2_result[*, 0:endb]
     ENDIF ELSE BEGIN 
          endb = WHERE(s2_result[0, *] EQ end2)
          s2_result = s2_result[*, 0:endb]
     ENDELSE 

; PLOT 2nd ORDER RESULT
     CASE nodiv OF 
          1 : BEGIN 
               result2 = a2_result
          END 
          2 : BEGIN 
               result2 = s2_result
          END 
          ELSE : BEGIN 
               result2 = a2_result
               result2[1, *] = (a2_result[1, *] / s2_result[1, *]) 
               e1 = (a2_result[2, *])^2/(a2_result[1, *])^2
               e2 = (s2_result[2, *])^2/(s2_result[1, *])^2
               e3 = (a2_result[1, *]/s2_result[1, *])^2
               result2[2, *] = sqrt((e1 + e2) * e3)
               result2[3, *] = (a2_result[3, *] + s2_result[3, *]) 
               w = where(result2[3, *] NE 2)
               IF w[0] NE -1 THEN result2[3, w] = -1  
               result2[3, *] = result2[3, *] < 1
          END 
     ENDCASE 
     gd = where(result2[3, *] EQ 1)
     tbad = n_elements(result2[0, *]) - n_elements(gd)
     IF noatran EQ 0 THEN $ 
       plot, result2[0, gd], result2[1, gd], xrange = [xmin, xmax], $ 
       yrange = [ymin, ymax], psym = 2, $ 
       title = '2nd ORDER WITH TELLURIC CORRECTION'
     print, 'BAD POINTS REMOVED FROM 2nd ORDER: ', tbad



; ------------------- 3rd ORDER --------------------

; LOAD 3rd ORDER SPECTRA FILES
     print, ' '
     print, 'Reading 3rd order files'
     a2a = READ_SPEC(asteroid[4], paths[0])
     a2b = READ_SPEC(asteroid[5], paths[0])
     s2a = READ_SPEC(star[4], paths[0])
     s2b = READ_SPEC(star[5], paths[0])

; ADD A AND B BEAM SPECTRA TOGETHER
     n_a =  n_elements(a2a[0, *])*50.0
     n_b =  n_elements(a2b[0, *])*50.0
     wl_a = min(a2a[0, *]) + findgen(n_a)*(a2a[0, 5] - a2a[0, 4])/50.0
     wl_b = min(a2b[0, *]) + findgen(n_b)*(a2b[0, 5] - a2b[0, 4])/50.0
     tempa = interpol(a2a[1, *], a2a[0, *], wl_a, /spline)
     tempb = interpol(a2b[1, *], a2b[0, *], wl_b, /spline)
     lag = findgen(101) - 50 
     ccor = c_correlate(tempa, tempb, lag)
     junk = max(ccor, ix)
     wl_c = a2b[0, *] + lag[ix]*(wl_b[5] - wl_b[4])
     a2b[1, *] = interpol(a2b[1, *], a2b[0, *], wl_c, /spline)
     a2a[1, *] = a2a[1, *] + a2b[1, *]
     s2a[1, *] = s2a[1, *] + s2b[1, *]
     a2a[2, *] = sqrt((a2a[2, *]^2 + a2b[2, *]^2)/2.0)
     s2a[2, *] = sqrt((s2a[2, *]^2 + s2b[2, *]^2)/2.0)

; PLOT ORDER
     xmax = max(a2a[0, *], min = xmin)
     CASE nodiv OF 
          1 : BEGIN 
               IF plt_lim[0] GT plt_lim[1] THEN BEGIN 
                    dy = SDEV(a2a[1, *])
                    ymax = dy[0] + 10*dy[1]
                    ymin = dy[0] - 10*dy[1]
               ENDIF 
               plot, a2a[0, *], a2a[1, *], $ 
                     yrange = [ymin, ymax], psym = 1, $ 
                     title = '3rd ORDER WITHOUT TELLURIC CORRECTION', $ 
                     xrange = [xmin, xmax]
          END 
          2 : BEGIN 
               IF plt_lim[0] GT plt_lim[1] THEN BEGIN 
                    dy = SDEV(s2a[1, *])
                    ymax = dy[0] + 10*dy[1]
                    ymin = dy[0] - 10*dy[1]
               ENDIF 
               plot, s2a[0, *], s2a[1, *], $ 
                     yrange = [ymin, ymax], psym = 1, $ 
                     title = '3rd ORDER WITHOUT TELLURIC CORRECTION', $ 
                     xrange = [xmin, xmax]
          END 
          ELSE : BEGIN 
               IF plt_lim[0] GT plt_lim[1] THEN BEGIN 
                    dy = SDEV(a2a[1, *]/s2a[1, *])
                    ymax = dy[0] + 10*dy[1]
                    ymin = dy[0] - 10*dy[1]
               ENDIF 
               xmax = max(a2a[0, *], min = xmin)
               plot, a2a[0, *], (a2a[1, *]/s2a[1, *]), $ 
                     yrange = [ymin, ymax], psym = 1, $ 
                     title = '3rd ORDER WITHOUT TELLURIC CORRECTION', $ 
                     xrange = [xmin, xmax]
          END 
     ENDCASE 
     wait, 2

; APPLY ATRAN FILES TO SPECTRA
     IF noatran EQ 0 THEN BEGIN 
          a3_result = APPLY_ATRAN(a2a, aatran, 3, a2vals[1], outp = a3vals)
          s3_result = APPLY_ATRAN(s2a, satran, 3, s2vals[1], outp = s3vals)
     ENDIF ELSE BEGIN 
          a3_result = a2a
          s3_result = s2a
          a3vals = [0.0, 0.0]
          s3vals = [0.0, 0.0]
     ENDELSE 

; FLAG BAD POINTS
     xa = transpose(a3_result[0, *])
     ya = transpose(a3_result[1, *])
     za = transpose(a3_result[2, *])
     xs = transpose(s3_result[0, *])
     ys = transpose(s3_result[1, *])
     zs = transpose(s3_result[2, *])
     flaga = intarr(1, n_elements(a3_result[0, *])) + 1
     flags = intarr(1, n_elements(s3_result[0, *])) + 1

; For asteroid
     gd = where(flaga EQ 1)
     FOR i = 0, 3 DO BEGIN 
          junk = svdfit(xa[gd], ya[gd], 5, yfit = yy, meas = za[gd])
          dif = (ya[gd] - yy) + mean(yy)
          sd = sdev(dif)
          bad = where(abs(dif - sd[0]) GT thresh*sd[1], bnum)
          IF bnum NE 0 THEN flaga[0, gd[bad]] = -1 ELSE i = 4
          gd = where(flaga GT 0)
     ENDFOR
     a3_result = [a3_result, flaga]

; For standard star
     gd = where(flags EQ 1)
     FOR i = 0, 3 DO BEGIN 
          junk = svdfit(xs[gd], ys[gd], 5, yfit = yy, meas = zs[gd])
          dif = (ys[gd] - yy) + mean(yy)
          sd = sdev(dif)
          bad = where(abs(dif - sd[0]) GT thresh*sd[1], bnum)
          IF bnum NE 0 THEN flags[0, gd[bad]] = -1 ELSE i = 4
          gd = where(flags GT 0)
     ENDFOR
     s3_result = [s3_result, flags]

; LOOK FOR REGIONS OF NO OVERLAP BETWEEN STANDARD AND ASTEROID
     end1 = min(a3_result[0, *])
     end2 = max(a3_result[0, *])
     end3 = min(s3_result[0, *])
     end4 = max(s3_result[0, *])
     IF end1 GT end3 THEN BEGIN 
          endn = n_elements(s3_result[0, *])-1
          endb = WHERE(s3_result[0, *] EQ end1)
          s3_result = s3_result[*, endb:endn]
     ENDIF ELSE BEGIN 
          endn = n_elements(a3_result[0, *])-1
          endb = WHERE(a3_result[0, *] EQ end3)
          a3_result = a3_result[*, endb:endn]
     ENDELSE 
     IF end2 GT end4 THEN BEGIN 
          endb = WHERE(a3_result[0, *] EQ end4)
          a3_result = a3_result[*, 0:endb]
     ENDIF ELSE BEGIN 
          endb = WHERE(s3_result[0, *] EQ end2)
          s3_result = s3_result[*, 0:endb]
     ENDELSE 

; PLOT 3rd ORDER RESULT
     CASE nodiv OF 
          1 : BEGIN 
               result3 = a3_result
          END 
          2 : BEGIN 
               result3 = s3_result
          END 
          ELSE : BEGIN 
               result3 = a3_result
               result3[1, *] = (a3_result[1, *] / s3_result[1, *]) 
               e1 = (a3_result[2, *])^2/(a3_result[1, *])^2
               e2 = (s3_result[2, *])^2/(s3_result[1, *])^2
               e3 = (a3_result[1, *]/s3_result[1, *])^2
               result3[2, *] = sqrt((e1 + e2) * e3)
               result3[3, *] = (a3_result[3, *] + s3_result[3, *]) 
               w = where(result3[3, *] NE 2)
               IF w[0] NE -1 THEN result3[3, w] = -1  
               result3[3, *] = result3[3, *] < 1
          END 
     ENDCASE 
     gd = where(result3[3, *] EQ 1)
     tbad = n_elements(result3[0, *]) - n_elements(gd)
     IF noatran EQ 0 THEN $ 
       plot, result3[0, gd], result3[1, gd], xrange = [xmin, xmax], $ 
       yrange = [ymin, ymax], psym = 2, $ 
       title = '3rd ORDER WITH TELLURIC CORRECTION'
     print, 'BAD POINTS REMOVED FROM 3rd ORDER: ', tbad



; ------------------- 4th ORDER -------------------

; LOAD 4th ORDER SPECTRA FILES
     print, ' '
     print, 'Reading 4th order files'
     a2a = READ_SPEC(asteroid[6], paths[0])
     a2b = READ_SPEC(asteroid[7], paths[0])
     s2a = READ_SPEC(star[6], paths[0])
     s2b = READ_SPEC(star[7], paths[0])

; ADD A AND B BEAM SPECTRA TOGETHER
     n_a =  n_elements(a2a[0, *])*50.0
     n_b =  n_elements(a2b[0, *])*50.0
     wl_a = min(a2a[0, *]) + findgen(n_a)*(a2a[0, 5] - a2a[0, 4])/50.0
     wl_b = min(a2b[0, *]) + findgen(n_b)*(a2b[0, 5] - a2b[0, 4])/50.0
     tempa = interpol(a2a[1, *], a2a[0, *], wl_a, /spline)
     tempb = interpol(a2b[1, *], a2b[0, *], wl_b, /spline)
     lag = findgen(101) - 50 
     ccor = c_correlate(tempa, tempb, lag)
     junk = max(ccor, ix)
     wl_c = a2b[0, *] + lag[ix]*(wl_b[5] - wl_b[4])
     a2b[1, *] = interpol(a2b[1, *], a2b[0, *], wl_c, /spline)
     a2a[1, *] = a2a[1, *] + a2b[1, *]
     s2a[1, *] = s2a[1, *] + s2b[1, *]
     a2a[2, *] = sqrt((a2a[2, *]^2 + a2b[2, *]^2)/2.0)
     s2a[2, *] = sqrt((s2a[2, *]^2 + s2b[2, *]^2)/2.0)

; PLOT ORDER
     xmax = max(a2a[0, *], min = xmin)
     CASE nodiv OF 
          1 : BEGIN 
               IF plt_lim[0] GT plt_lim[1] THEN BEGIN 
                    dy = SDEV(a2a[1, *])
                    ymax = dy[0] + 10*dy[1]
                    ymin = dy[0] - 10*dy[1]
               ENDIF 
               plot, a2a[0, *], a2a[1, *], $ 
                     yrange = [ymin, ymax], psym = 1, $ 
                     title = '4th ORDER WITHOUT TELLURIC CORRECTION', $ 
                     xrange = [xmin, xmax]
          END 
          2 : BEGIN 
               IF plt_lim[0] GT plt_lim[1] THEN BEGIN 
                    dy = SDEV(s2a[1, *])
                    ymax = dy[0] + 10*dy[1]
                    ymin = dy[0] - 10*dy[1]
               ENDIF 
               plot, s2a[0, *], s2a[1, *], $ 
                     yrange = [ymin, ymax], psym = 1, $ 
                     title = '4th ORDER WITHOUT TELLURIC CORRECTION', $ 
                     xrange = [xmin, xmax]
          END 
          ELSE : BEGIN 
               IF plt_lim[0] GT plt_lim[1] THEN BEGIN 
                    dy = SDEV(a2a[1, *]/s2a[1, *])
                    ymax = dy[0] + 10*dy[1]
                    ymin = dy[0] - 10*dy[1]
               ENDIF 
               xmax = max(a2a[0, *], min = xmin)
               plot, a2a[0, *], (a2a[1, *]/s2a[1, *]), $ 
                     yrange = [ymin, ymax], psym = 1, $ 
                     title = '4th ORDER WITHOUT TELLURIC CORRECTION', $ 
                     xrange = [xmin, xmax]
          END 
     ENDCASE 
     wait, 2

; APPLY ATRAN FILES TO SPECTRA
     IF noatran EQ 0 THEN BEGIN 
          a4_result = APPLY_ATRAN(a2a, aatran, 4, a2vals[1], outp = a4vals)
          s4_result = APPLY_ATRAN(s2a, satran, 4, s2vals[1], outp = s4vals)
     ENDIF ELSE BEGIN 
          a4_result = a2a
          s4_result = s2a
          a4vals = [0.0, 0.0]
          s4vals = [0.0, 0.0]
     ENDELSE 

; FLAG BAD POINTS
     xa = transpose(a4_result[0, *])
     ya = transpose(a4_result[1, *])
     za = transpose(a4_result[2, *])
     xs = transpose(s4_result[0, *])
     ys = transpose(s4_result[1, *])
     zs = transpose(s4_result[2, *])
     flaga = intarr(1, n_elements(a4_result[0, *])) + 1
     flags = intarr(1, n_elements(s4_result[0, *])) + 1

; For asteroid
     gd = where(flaga EQ 1)
     FOR i = 0, 3 DO BEGIN 
          junk = svdfit(xa[gd], ya[gd], 5, yfit = yy, meas = za[gd])
          dif = (ya[gd] - yy) + mean(yy)
          sd = sdev(dif)
          bad = where(abs(dif - sd[0]) GT thresh*sd[1], bnum)
          IF bnum NE 0 THEN flaga[0, gd[bad]] = -1 ELSE i = 4
          gd = where(flaga GT 0)
     ENDFOR
     a4_result = [a4_result, flaga]

; For standard star
     gd = where(flags EQ 1)
     FOR i = 0, 3 DO BEGIN 
          junk = svdfit(xs[gd], ys[gd], 5, yfit = yy, meas = zs[gd])
          dif = (ys[gd] - yy) + mean(yy)
          sd = sdev(dif)
          bad = where(abs(dif - sd[0]) GT thresh*sd[1], bnum)
          IF bnum NE 0 THEN flags[0, gd[bad]] = -1 ELSE i = 4
          gd = where(flags GT 0)
     ENDFOR
     s4_result = [s4_result, flags]

; LOOK FOR REGIONS OF NO OVERLAP BETWEEN STANDARD AND ASTEROID
     end1 = min(a4_result[0, *])
     end2 = max(a4_result[0, *])
     end3 = min(s4_result[0, *])
     end4 = max(s4_result[0, *])
     IF end1 GT end3 THEN BEGIN 
          endn = n_elements(s4_result[0, *])-1
          endb = WHERE(s4_result[0, *] EQ end1)
          s4_result = s4_result[*, endb:endn]
     ENDIF ELSE BEGIN 
          endn = n_elements(a4_result[0, *])-1
          endb = WHERE(a4_result[0, *] EQ end3)
          a4_result = a4_result[*, endb:endn]
     ENDELSE 
     IF end2 GT end4 THEN BEGIN 
          endb = WHERE(a4_result[0, *] EQ end4)
          a4_result = a4_result[*, 0:endb]
     ENDIF ELSE BEGIN 
          endb = WHERE(s4_result[0, *] EQ end2)
          s4_result = s4_result[*, 0:endb]
     ENDELSE 

; PLOT 4th ORDER RESULT
     CASE nodiv OF 
          1 : BEGIN 
               result4 = a4_result
          END 
          2 : BEGIN 
               result4 = s4_result
          END 
          ELSE : BEGIN 
               result4 = a4_result
               result4[1, *] = (a4_result[1, *] / s4_result[1, *]) 
               e1 = (a4_result[2, *])^2/(a4_result[1, *])^2
               e2 = (s4_result[2, *])^2/(s4_result[1, *])^2
               e3 = (a4_result[1, *]/s4_result[1, *])^2
               result4[2, *] = sqrt((e1 + e2) * e3)
               result4[3, *] = (a4_result[3, *] + s4_result[3, *]) 
               w = where(result4[3, *] NE 2)
               IF w[0] NE -1 THEN result4[3, w] = -1  
               result4[3, *] = result4[3, *] < 1
          END 
     ENDCASE 
     gd = where(result4[3, *] EQ 1)
     tbad = n_elements(result4[0, *]) - n_elements(gd)
     IF noatran EQ 0 THEN $ 
       plot, result4[0, gd], result4[1, gd], xrange = [xmin, xmax], $ 
       yrange = [ymin, ymax], psym = 2, $ 
       title = '4th ORDER WITH TELLURIC CORRECTION'
     print, 'BAD POINTS REMOVED FROM 4th ORDER: ', tbad



; ------------------- 1st ORDER -------------------

; LOAD 1st ORDER SPECTRA FILES
     print, ' '
     print, 'Reading 1st order files'
     a2a = READ_SPEC(asteroid[0], paths[0])
     a2b = READ_SPEC(asteroid[1], paths[0])
     s2a = READ_SPEC(star[0], paths[0])
     s2b = READ_SPEC(star[1], paths[0])

; ADD A AND B BEAM SPECTRA TOGETHER
     n_a =  n_elements(a2a[0, *])*50.0
     n_b =  n_elements(a2b[0, *])*50.0
     wl_a = min(a2a[0, *]) + findgen(n_a)*(a2a[0, 5] - a2a[0, 4])/50.0
     wl_b = min(a2b[0, *]) + findgen(n_b)*(a2b[0, 5] - a2b[0, 4])/50.0
     tempa = interpol(a2a[1, *], a2a[0, *], wl_a, /spline)
     tempb = interpol(a2b[1, *], a2b[0, *], wl_b, /spline)
     lag = findgen(101) - 50 
     ccor = c_correlate(tempa, tempb, lag)
     junk = max(ccor, ix)
     wl_c = a2b[0, *] + lag[ix]*(wl_b[5] - wl_b[4])
     a2b[1, *] = interpol(a2b[1, *], a2b[0, *], wl_c, /spline)
     a2a[1, *] = a2a[1, *] + a2b[1, *]
     s2a[1, *] = s2a[1, *] + s2b[1, *]
     a2a[2, *] = sqrt((a2a[2, *]^2 + a2b[2, *]^2)/2.0)
     s2a[2, *] = sqrt((s2a[2, *]^2 + s2b[2, *]^2)/2.0)

; PLOT ORDER
     xmax = max(a2a[0, *], min = xmin)
     CASE nodiv OF 
          1 : BEGIN 
               IF plt_lim[0] GT plt_lim[1] THEN BEGIN 
                    dy = SDEV(a2a[1, *])
                    ymax = dy[0] + 10*dy[1]
                    ymin = dy[0] - 10*dy[1]
               ENDIF 
               plot, a2a[0, *], a2a[1, *], $ 
                     yrange = [ymin, ymax], psym = 1, $ 
                     title = '1st ORDER WITHOUT TELLURIC CORRECTION', $ 
                     xrange = [xmin, xmax]
          END 
          2 : BEGIN 
               IF plt_lim[0] GT plt_lim[1] THEN BEGIN 
                    dy = SDEV(s2a[1, *])
                    ymax = dy[0] + 10*dy[1]
                    ymin = dy[0] - 10*dy[1]
               ENDIF 
               plot, s2a[0, *], s2a[1, *], $ 
                     yrange = [ymin, ymax], psym = 1, $ 
                     title = '1st ORDER WITHOUT TELLURIC CORRECTION', $ 
                     xrange = [xmin, xmax]
          END 
          ELSE : BEGIN 
               IF plt_lim[0] GT plt_lim[1] THEN BEGIN 
                    dy = SDEV(a2a[1, *]/s2a[1, *])
                    ymax = dy[0] + 10*dy[1]
                    ymin = dy[0] - 10*dy[1]
               ENDIF 
               plot, a2a[0, *], (a2a[1, *]/s2a[1, *]), $ 
                     yrange = [ymin, ymax], psym = 1, $ 
                     title = '1st ORDER WITHOUT TELLURIC CORRECTION', $ 
                     xrange = [xmin, xmax]
          END 
     ENDCASE 
     wait, 2

; APPLY ATRAN FILES TO SPECTRA
     IF noatran EQ 0 THEN BEGIN 
          a1_result = APPLY_ATRAN(a2a, aatran, 1, a2vals[1], outp = a1vals)
          s1_result = APPLY_ATRAN(s2a, satran, 1, s2vals[1], outp = s1vals)
     ENDIF ELSE BEGIN 
          a1_result = a2a
          s1_result = s2a
          a1vals = [0.0, 0.0]
          s1vals = [0.0, 0.0]
     ENDELSE 

; FLAG BAD POINTS
     xa = transpose(a1_result[0, *])
     ya = transpose(a1_result[1, *])
     za = transpose(a1_result[2, *])
     xs = transpose(s1_result[0, *])
     ys = transpose(s1_result[1, *])
     zs = transpose(s1_result[2, *])
     flaga = intarr(1, n_elements(a1_result[0, *])) + 1
     flags = intarr(1, n_elements(s1_result[0, *])) + 1

; For asteroid
     gd = where(flaga EQ 1)
     FOR i = 0, 3 DO BEGIN 
          junk = svdfit(xa[gd], ya[gd], 5, yfit = yy, meas = za[gd])
          dif = (ya[gd] - yy) + mean(yy)
          sd = sdev(dif)
          bad = where(abs(dif - sd[0]) GT thresh*sd[1], bnum)
          IF bnum NE 0 THEN flaga[0, gd[bad]] = -1 ELSE i = 4
          gd = where(flaga GT 0)
     ENDFOR
     a1_result = [a1_result, flaga]

; For standard star
     gd = where(flags EQ 1)
     FOR i = 0, 3 DO BEGIN 
          junk = svdfit(xs[gd], ys[gd], 5, yfit = yy, meas = zs[gd])
          dif = (ys[gd] - yy) + mean(yy)
          sd = sdev(dif)
          bad = where(abs(dif - sd[0]) GT thresh*sd[1], bnum)
          IF bnum NE 0 THEN flags[0, gd[bad]] = -1 ELSE i = 4
          gd = where(flags GT 0)
     ENDFOR
     s1_result = [s1_result, flags]

; LOOK FOR REGIONS OF NO OVERLAP BETWEEN STANDARD AND ASTEROID
     end1 = min(a1_result[0, *])
     end2 = max(a1_result[0, *])
     end3 = min(s1_result[0, *])
     end4 = max(s1_result[0, *])
     IF end1 GT end3 THEN BEGIN 
          endn = n_elements(s1_result[0, *])-1
          endb = WHERE(s1_result[0, *] EQ end1)
          s1_result = s1_result[*, endb:endn]
     ENDIF ELSE BEGIN 
          endn = n_elements(a1_result[0, *])-1
          endb = WHERE(a1_result[0, *] EQ end3)
          a1_result = a1_result[*, endb:endn]
     ENDELSE 
     IF end2 GT end4 THEN BEGIN 
          endb = WHERE(a1_result[0, *] EQ end4)
          a1_result = a1_result[*, 0:endb]
     ENDIF ELSE BEGIN 
          endb = WHERE(s1_result[0, *] EQ end2)
          s1_result = s1_result[*, 0:endb]
     ENDELSE 

; PLOT 1st ORDER RESULT
     CASE nodiv OF 
          1 : BEGIN 
               result1 = a1_result
          END 
          2 : BEGIN 
               result1 = s1_result
          END 
          ELSE : BEGIN 
               result1 = a1_result
               result1[1, *] = (a1_result[1, *] / s1_result[1, *]) 
               e1 = (a1_result[2, *])^2/(a1_result[1, *])^2
               e2 = (s1_result[2, *])^2/(s1_result[1, *])^2
               e3 = (a1_result[1, *]/s1_result[1, *])^2
               result1[2, *] = sqrt((e1 + e2) * e3)
               result1[3, *] = (a1_result[3, *] + s1_result[3, *]) 
               w = where(result1[3, *] NE 2)
               IF w[0] NE -1 THEN result1[3, w] = -1  
               result1[3, *] = result1[3, *] < 1
          END 
     ENDCASE 
     gd = where(result1[3, *] EQ 1)
     tbad = n_elements(result1[0, *]) - n_elements(gd)
     IF noatran EQ 0 THEN $ 
       plot, result1[0, gd], result1[1, gd], xrange = [xmin, xmax], $ 
       yrange = [ymin, ymax], psym = 2, $ 
       title = '1st ORDER WITH TELLURIC CORRECTION'
     print, 'BAD POINTS REMOVED FROM 1st ORDER: ', tbad
     wait, 2


; ------------------ COMBINE ORDERS -------------------

; REMOVE OVERLAPS
     mn1 = MIN(result1[0, *])
     mn2 = MIN(result2[0, *])
     mn3 = MIN(result3[0, *])
     w21 = WHERE(result2[0, *] GE mn1)
     w32 = WHERE(result3[0, *] GE mn2)
     w43 = WHERE(result4[0, *] GE mn3)
     cut2 = MIN(w21)-1
     cut3 = MIN(w32)-1
     cut4 = MIN(w43)-1
     result2 = result2[*, 0:cut2]
     result3 = result3[*, 0:cut3]
     result4 = result4[*, 0:cut4]

; CREATE COMBINED SPECTRUM
     final = [[result4], [result3], [result2], [result1]]

; FLAG 2.53 to 2.85 microns
     wgap = where(final[0, *] GT 2.53 AND final[0, *] LT 2.85)
     final[3, wgap] = -1

; FLAG BEYOND 4.11 microns
     wend = where(final[0, *] GT 4.11)
     final[3, wend] = -1

; SCALE CONTINUUM TO 1
     wcont = where(final[0, *] LE 2.5)
     cont = final[*, wcont]
     wcont = where(cont[3, *] EQ 1)
     cont1 = SDEV(cont[1, wcont])
     sde = MEDIAN(cont[2, wcont])
     final[1, *] = final[1, *]/cont1[0]
     final[2, *] = final[2, *]*cont1[1]/sde/cont1[0]

; DISPLAY SPECTRUM
     wgood = where(final[3, *] GT 0)
     IF n_elements(wgood) GT 3 THEN BEGIN 
          IF plt_lim[0] GT plt_lim[1] THEN BEGIN 
               dy = SDEV(final[1, wgood])
               ymax = dy[0] + 2*dy[1]
               ymin = dy[0] - 2*dy[1]
          ENDIF 
          plot, final[0, wgood], final[1, wgood], $ 
                title = 'FULL SPECTRUM', xrange = [2.2, 4.2],  $ 
                yrange = [ymin, ymax], psym = 1
     ENDIF ELSE print, ' Too many bad points '


; ------------------ SAVE RESULTS -------------------

     cd, paths[2], current = nowp
     openw, logun, result, /get_lun
     printf, logun, final
     close, logun
     free_lun, logun
     openw, logun, record, /get_lun
     printf, logun, '  '
     printf, logun, 'Runtime:  ', systime()
     printf, logun, '  '
     printf, logun, 'Input Path:  ', paths[0]
     printf, logun, 'Output Path: ', paths[2]
     printf, logun, 'ATRAN Path:  ', paths[1]
     printf, logun, '  '
     printf, logun, 'Asteroid ATRAN File: ', atran[0]
     printf, logun, 'Standard Star ATRAN File: ', atran[1]
     printf, logun, 'Output File: ', result
     printf, logun, '  '
     printf, logun, 'Water Range: ', min_wtr, max_wtr
     printf, logun, 'Asteroid Water: ', a2vals[1]
     printf, logun, 'Star Water: ', s2vals[1]
     printf, logun, '  '
     printf, logun, 'FIRST ORDER'
     printf, logun, '     A-beam asteroid file: ', asteroid[0]
     printf, logun, '     B-beam asteroid file: ', asteroid[1]
     printf, logun, '     A-beam standard file: ', star[0]
     printf, logun, '     B-beam standard file: ', star[1]
     printf, logun, '     Asteroid shift (microns): ', a1vals[0]
     printf, logun, '     Standard shift (microns): ', s1vals[0]
     printf, logun, '  '
     printf, logun, 'SECOND ORDER'
     printf, logun, '     A-beam asteroid file: ', asteroid[2]
     printf, logun, '     B-beam asteroid file: ', asteroid[3]
     printf, logun, '     A-beam standard file: ', star[2]
     printf, logun, '     B-beam standard file: ', star[3]
     printf, logun, '     Asteroid shift (microns): ', a2vals[0]
     printf, logun, '     Standard shift (microns): ', s2vals[0]
     printf, logun, '  '
     printf, logun, 'THIRD ORDER'
     printf, logun, '     A-beam asteroid file: ', asteroid[4]
     printf, logun, '     B-beam asteroid file: ', asteroid[5]
     printf, logun, '     A-beam standard file: ', star[4]
     printf, logun, '     B-beam standard file: ', star[5]
     printf, logun, '     Asteroid shift (microns): ', a3vals[0]
     printf, logun, '     Standard shift (microns): ', s3vals[0]
     printf, logun, '  '
     printf, logun, 'FOURTH ORDER'
     printf, logun, '     A-beam asteroid file: ', asteroid[6]
     printf, logun, '     B-beam asteroid file: ', asteroid[7]
     printf, logun, '     A-beam standard file: ', star[6]
     printf, logun, '     B-beam standard file: ', star[7]
     printf, logun, '     Asteroid shift (microns): ', a4vals[0]
     printf, logun, '     Standard shift (microns): ', s4vals[0]
     printf, logun, '  '
     printf, logun, strg
     IF noatran NE 0 THEN printf, logun, 'TELLURIC CORRECTION NOT PERFORMED'
     close, logun
     free_lun, logun
     cd, nowp
     wait, 2

END
