; NAME: 
;      TRIM_FITS
; 
; VERSION: 1.1
; 
; PURPOSE: 
;      Trims 2-D FITS images by user-specified amount.
; 
; CALLING SEQUENCE: 
;      trim_fits, ifile, [ofile=ofile, paths=paths, xvals=xvals, 
;                 yvals=yvals, yzero=yzero, help=help]
; 
; INPUTS: 
;      IFILE - Filename(s) of fits image to be trimmed.
; 
; OPTIONAL KEYWORD INPUTS: 
;      PATHS - Vector of paths for ifile and ofile.
;              (default is current path)
;      XVALS - Two element vector with minimum and maximum column
;              values of trimmed image.  If not specified, then all
;              column values are kept.
;      YVALS - Same as xvals, but for rows.
;      YZERO - Filename of reference image used to deterimine
;              y-offset.  The position of the maximum pixel-to-pixel
;              change on column 25 rows 950 to 1022 is found.  The
;              comparable value for the image in IFILE is found and 
;              shifted accordingly.
;      HELP - Set to nonzero scalar to have TRIM_FITS print syntax.
; 
; OUTPUT: 
;      OFILE - Filename(s) where trimmed image(s) will be saved. 
;              (default is filenames in ifile)
; 
; EXTERNAL CALLS: 
;      FITS_CLOSE, FITS_WRITE, FXADDPAR, GETIM 
; 
; COMMON BLOCKS: 
;      NONE 
; 
; MODIFICATION HISTORY: 
;      - Written by Eric Volquardsen, NASA IRTF, April 2003
;      - Added YZERO keyword, ELV, May 2004
; 
; ======================================================================
PRO trim_fits, ifile, ofile=ofile, paths=paths, xvals=xvals, $ 
               yvals=yvals, yzero = yzero, help=help

; CHECK INPUTS
     IF n_elements(ifile) LT 1 THEN help = 1 
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN BEGIN
               print, '*******************************************'
               print, '***          TRIM_FITS SYNTAX           ***' 
               print, '*** > trim_fits, ifile, ofile=ofile,    ***' 
               print, '***        paths=paths, xvals=xvals,    ***' 
               print, '***        yvals=yvals, yzero=yzero     ***' 
               print, '***                                     ***' 
               print, '*** IFILE - Filenames of images to trim ***'
               print, '*** OFILE - Filenames of trimmed images ***' 
               print, '*** PATHS - Ifile and ofile paths       ***' 
               print, '*** XVALS - Maximum and minimum column  ***' 
               print, '***         values of trimmed images    ***' 
               print, '*** YVALS - Same for rows               ***' 
               print, '*** YZERO - Filename of reference image ***' 
               print, '***         used to calculate vertical  ***' 
               print, '***         shift                       ***' 
               print, '*******************************************' 
               RETURN
          ENDIF
     ENDIF ELSE help = 0 
     nf = n_elements(ifile)
     IF nf NE n_elements(ofile) THEN ofile = ifile
     np = n_elements(paths)
     CASE np OF 
          2 : 
          1 : paths = [paths, paths]
          ELSE : BEGIN 
               v = !version.os_family
               CASE v OF
                    'vms' : root = '[]'
                    'windows' : root = 'c:\'
                    ELSE : root = '/'
               ENDCASE
               cd, root, current=nowp
               paths = [nowp, nowp]
               cd, nowp
          END
     ENDCASE   ; path check
     nxv = n_elements(xvals)
     nyv = n_elements(yvals)

; GET REFERENCE IMAGE (if designated)
     IF n_elements(yzero) EQ 1 THEN BEGIN 
          GETIM, yzero, refi, paths[0]
          IF n_elements(refi) GT 1e6 THEN BEGIN 
               refii = refi[25, 950:1022] - refi[25, 949:1021]
               junk = max(refii, im0)
          ENDIF ELSE im0 = -1
     ENDIF ELSE im0 = -1

; LOOP THROUGH FILES
     FOR i = 0, (nf-1) DO BEGIN

; Load file
          getim, ifile[i], img, paths[0], hdr=hdr
          IF n_elements(img) EQ 1 THEN RETURN

; Shift image to match reference image (if passed)
          IF im0 GT 0 THEN BEGIN 
               refii = img[25, 950:1022] - img[25, 949:1021]
               junk = max(refii, shft)
               shft = im0 - shft
               IF shft GT 10 THEN shft = 0
               IF shft LT -10 THEN shft = 0
               IF shft NE 0 THEN img = shift(img, 0, shft)
          ENDIF 

; Set size of trimmed image
          sz = size(img, /dimensions)
          IF nxv NE 2 THEN xvals = [0, (sz[0]-1)]
          IF nyv NE 2 THEN yvals = [0, (sz[1]-1)]
          dx = xvals[1] - xvals[0] + 1 
          dy = yvals[1] - yvals[0] + 1 

; Trim image
          trimg = fltarr(dx, dy)
          trimg = img[xvals[0]:xvals[1], yvals[0]:yvals[1]]

; Modify header
          fxaddpar, hdr, 'NAXIS1', dx
          fxaddpar, hdr, 'NAXIS2', dy
          com = '= Trimmed from original ' + strtrim(string(sz[0]), 2) + $ 
               'x' + strtrim(string(sz[1]), 2)
          fxaddpar, hdr, 'COMMENT', com

; Save trimmed image
          cd, paths[1], current=nowp 
          fits_write, ofile[i], trimg, hdr
          fits_close, fcb
          cd, nowp

     ENDFOR   ; file loop

END
