; NAME: 
;      TSHIFT
; 
; VERSION: 1.2
; 
; PURPOSE: 
;      Returns best shift and water amounts for telluric correction
;      of spectra from SpeX LXD mode.
; 
; CALLING SEQUENCE: 
;      result = tshift(reflectance, wl, atran, [water=water, f_o=f_o, 
;                      help=help])
; 
; INPUTS: 
;      reflectance - Reflectance values
;      wl - Wavelengths for reflectance values
;      atran - 61x9000 ATRAN model array scaled to wavelength
;              increments of the spectrum.
; 
; OPTIONAL KEYWORD INPUTS: 
;      water - Set to desired water amount or range of amounts.
;              Negative value will result in default range.
;              (default is to check 0.5mm to 3.0mm)
;      f_o - Fitting order for svdfit (default is 3)
;      help - Set to nonzero scalar to print syntax
; 
; OUTPUT: 
;      result - Vector with best shift amount (in microns) and 
;               water amount (in mm).
; 
; EXTERNAL CALLS: 
;      NONE 
; 
; COMMON BLOCKS: 
;      NONE
; 
; MODIFICATION HISTORY: 
;      - Written by Eric Volquardsen, NASA IRTF, April 2004
;      - Improved water calculations, ELV, March 2005
; 
; =======================================================================
FUNCTION tshift, reflectance, wl, er, atran, water = water, f_o = f_o,  $ 
                 help = help

; CHECK INPUTS
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN help = 1 
     ENDIF ELSE help = 0 
     IF n_elements(reflectance) NE n_elements(wl) THEN help = 1
     IF n_elements(reflectance) NE n_elements(er) THEN help = 1
     IF help EQ 1 THEN BEGIN
          print, '****************************************************'
          print, '***                TSHIFT SYNTAX                 ***'  
          print, '*** IDL> result = tshift(reflectance, wl, er,    ***' 
          print, '***               atran, water=water, f_o=f_o    ***'
          print, '***                                              ***' 
          print, '*** reflectance - Reflectance values             ***'
          print, '*** wl - Wavelengths for reflectance values      ***'
          print, '*** er - Errors of reflectance values            ***' 
          print, '*** atran - 61x9000 ATRAN model array scaled to  ***' 
          print, '***         wavelength increment of spectrum     ***' 
          print, '*** water - Set to desired water amount or range ***' 
          print, '***         (default is to check 0.5mm to 3.0mm) ***'
          print, '*** f_o - Fitting order (default is 3)           ***'
          print, '****************************************************'   
          RETURN, -1
     ENDIF
     shel = [0.00001, 100]      ; Shift spacing and number of increments
     IF n_elements(f_o) NE 1 THEN f_o = 3
     f_o = (f_o) > 1
     IF n_elements(size(reflectance, /dimension)) EQ 2 THEN $ 
       reflectance = transpose(reflectance)
     IF n_elements(size(wl, /dimension)) EQ 2 THEN wl = transpose(wl)
     IF n_elements(size(er, /dimension)) EQ 2 THEN er = transpose(er)
     nw = n_elements(water)
     IF nw EQ 0 THEN water = [0.5, 3.0]
     nw = n_elements(water)
     IF nw EQ 1 THEN water = [water, water]

; SEPARATE EXACT AND APPROXIMATE WATER VALUES 
     IF water[0] EQ water[1] THEN BEGIN 

; Get atran model
          junk = fix(water[0]*10+1)
          subaa1 = interpol(atran[junk, *], atran[0, *], wl)
          subaa2 = interpol(atran[junk+1, *], atran[0, *], wl)
          percnt = (water[0] - (junk-1)/10.0)*10.0
          subaa = subaa1 + percnt*(subaa2-subaa1)

; Loop through shift amounts (ROUGH)
          b = [0, 2*shel[1]]
          fitar = fltarr(2, ((b[1]-b[0]+1)/10)+1)
          FOR i = b[0], b[1], 10 DO BEGIN 
               sh = shel[0]*(i-shel[1])
               wl1 = wl + sh   
               subref = interpol(reflectance, wl, wl1, /spline)
               er1 = interpol(er, wl, wl1, /spline)

; Get chi squared
               astdiv = subref/subaa
               er2 = er1/subaa
               junk = svdfit(wl, astdiv, f_o, yfit = adfit, chisq = cq, $ 
                             measure_errors = er2)

; Look for best shift for water amount
               fitar[0, (i/10-b[0])] = sh
               fitar[1, (i/10-b[0])] = cq
          ENDFOR 
          junk = min(fitar[1, *], mn)
          ares = fitar[*, mn]
          result = [ares[0], water[0], ares[1]]

; Loop through shift amounts (FINE)
          ibest1 = shel[1] + (result[0]/shel[0])
          result1 = result
          FOR i = (ibest1-10), (ibest1+10) DO BEGIN 
               sh = shel[0]*(i-shel[1])
               wl1 = wl + sh   
               subref = interpol(reflectance, wl, wl1, /spline)
               er1 = interpol(er, wl, wl1, /spline)
               astdiv = subref/subaa
               er2 = er1/subaa
               junk = svdfit(wl, astdiv, f_o, yfit = adfit, chisq = cq, $ 
                             measure_errors = er2)
               IF cq LT result[2] THEN BEGIN 
                    result1[0] = sh
                    result1[2] = cq
               ENDIF 
          ENDFOR 

; MULTIPLE WATER AMOUNTS
     ENDIF ELSE BEGIN 
          IF water[1] LT 0 THEN water = [0.5, 3.0] 
          water = abs(water)
          IF water[0] GT water[1] THEN water = [water[1], water[0]]
          min_wtr = fix(water[0]*10+1)
          max_wtr = fix(water[1]*10+1)

; Loop through water amounts (ROUGH)
          FOR j = min_wtr, max_wtr DO BEGIN 
               subaa = interpol(atran[j, *], atran[0, *], wl)
               jw = (j-1)*0.1

; Loop through shift amounts (ROUGH)
               b = [0, 2*shel[1]]
               fitar = fltarr(2, ((b[1]-b[0]+1)/10)+1)
               FOR i = b[0], b[1], 10 DO BEGIN 
                    sh = shel[0]*(i-shel[1])
                    wl1 = wl + sh   
                    subref = interpol(reflectance, wl, wl1, /spline)
                    er1 = interpol(er, wl, wl1, /spline)

; Get chi squared
                    astdiv = subref/subaa
                    er2 = er1/subaa
                    junk = svdfit(wl, astdiv, f_o, yfit = adfit, chisq = cq, $ 
                                  measure_errors = er2)

; Look for best shift for water amount
                    fitar[0, (i/10-b[0])] = sh
                    fitar[1, (i/10-b[0])] = cq
               ENDFOR 
               junk = min(fitar[1, *], mn)
               ares = fitar[*, mn]

; Look for best water amount
               IF j EQ min_wtr THEN BEGIN 
                    result = [ares[0], jw, ares[1]]
               ENDIF ELSE BEGIN 
                    IF ares[1] LT result[2] THEN BEGIN 
                         result = [ares[0], jw, ares[1]]
                    ENDIF 
               ENDELSE  
          ENDFOR

; Interpolate exact water amount
          result1 = result
          jw = result[1]
          j = 10.0*jw+1
          ibest1 = shel[1] + (result[0]/shel[0])
          FOR k = (ibest1-10), (ibest1+10) DO BEGIN 
               sh = shel[0]*(k-shel[1])
               wl1 = wl + sh 
               suba1 = interpol(atran[j-1, *], atran[0, *], wl)
               suba0 = interpol(atran[j, *], atran[0, *], wl)
               subref = interpol(reflectance, wl, wl1, /spline)
               er1 = interpol(er, wl, wl1, /spline)
               dif = (suba1 - suba0)/10.0
               waterx = uindgen(11) - 5.0
               waterx2 = uindgen(11000)/1000.0 - 5.0
               cqx = fltarr(11)
               FOR i = -5, 5 DO BEGIN 
                    xid = i + 5
                    subaa = suba0 + i*dif
                    er2 = er1/subaa
                    astdiv = subref/subaa
                    junk = svdfit(wl, astdiv, f_o, yfit = adfit, $ 
                           chisq = cq, measure_errors = er2)
                    cqx[xid] = cq
               ENDFOR 
               cqx2 = interpol(cqx, waterx, waterx2, /spline)
               junk2 = min(cqx2, dvix)
               IF cqx2[dvix] LT result1[2] THEN BEGIN 
                    result1[2] = cqx2[dvix]
                    result1[1] = result[1] - waterx2[dvix]/100.0
                    result1[0] = sh
               ENDIF 
          ENDFOR 

     ENDELSE 

     RETURN, result1[0:1]

END
