; NAME: 
;      WL_LIN
; 
; PURPOSE: 
;      Linearizes wavelength distribution between cross-dispersed
;      Spex orders with A and B beams.  This is neccesary because 
;      each order is linearized to a different value.
; 
; CALLING SEQUENCE: 
;      result = wl_lin(speca, specb, [dc, help=help])
; 
; INPUTS: 
;      speca - A-beam spectrum
;      specb - B-beam spectrum
; 
; OPTIONAL INPUTS: 
;      dc - Set to desired separation between lines (in microns) 
;           to perform dispersion correction on all orders.
;           Value must be greater than the existing separation 
;           between wavelengths.  Default is 0.001 microns.
; 
; OPTIONAL KEYWORD INPUTS: 
;      help - Set to nonzero scalar to print syntax
;      
; 
; OUTPUT: 
;      Function returns 5 column table with wavelength, followed by 
;      flux and error for A and B beams respectively.
; 
; EXTERNAL CALLS: 
;      NONE 
; 
; COMMON BLOCKS: 
;      NONE
; 
; MODIFICATION HISTORY: 
;      - Written by Eric Volquardsen, NASA IRTF, November 2004
; 
; =======================================================================
FUNCTION wl_lin, speca, specb, dc, help = help

; CHECK INPUTS
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN help = 1 
     ENDIF ELSE help = 0 
     IF n_elements(speca) LT 30 THEN help = 1
     IF n_elements(specb) LT 30 THEN help = 1
     IF help EQ 1 THEN BEGIN
          print, '*************************************************'
          print, '***                WL_LIN SYNTAX              ***'  
          print, '*** IDL> result = wl_lin(speca, specb, dc)    ***'
          print, '***                                           ***' 
          print, '*** speca - A-beam spectrum                   ***'
          print, '*** specb - B-beam spectrum                   ***'
          print, '*** dc - Set to desired separation between    ***' 
          print, '***      lines (in microns). Default is 0.001 ***'
          print, '*************************************************'   
          RETURN, -1
     ENDIF
     IF n_elements(dc) NE 1 THEN dc = 0.001
     result = -1

; DETERMINE WAVELENGTH LIMITS
     tbl_a = speca
     tbl_b = specb
     nt = n_elements(tbl_a[0, *])
     mna = round(1000*tbl_a[0, 1])/1000.0
     mnb = round(1000*tbl_b[0, 1])/1000.0
     mna = max([mna, mnb])
     mxa = max(tbl_a[0, *])
     mxb = max(tbl_b[0, *])
     mxa = min([mxa, mxb])
     rng = fix((mxa-mna)/dc)
     IF rng GE nt THEN BEGIN 
          print, ' '
          print, 'DISPERSION CORRECTION VALUE TOO SMALL'
          print, string((mxa-mxb)/(nt+1))
     ENDIF ELSE BEGIN 
          wl = uindgen(rng)*dc + mna
          ya = INTERPOL(tbl_a[1, *], tbl_a[0, *], wl, /spline)
          yb = INTERPOL(tbl_b[1, *], tbl_b[0, *], wl, /spline)
          za = INTERPOL(tbl_a[2, *], tbl_a[0, *], wl, /quadratic)
          zb = INTERPOL(tbl_b[2, *], tbl_b[0, *], wl, /quadratic)
          wl = transpose(wl)
          ya = transpose(ya)
          za = transpose(za)
          yb = transpose(yb)
          zb = transpose(zb)
          result = [wl, ya, za, yb, zb]
     ENDELSE 

     RETURN, result

END
